@@ -55,7 +55,9 @@ class QQuickFolderListModelPrivate
 public:
     QQuickFolderListModelPrivate(QQuickFolderListModel *q)
         : q_ptr(q),
-          sortField(QQuickFolderListModel::Name), sortReversed(false), showDirs(true), showDirsFirst(false), showDots(false), showOnlyReadable(false)
+          sortField(QQuickFolderListModel::Name), sortReversed(false), showFiles(true),
+          showDirs(true), showDirsFirst(false), showDotAndDotDot(false), showOnlyReadable(false),
+          showHidden(false)
     {
         nameFilters << QLatin1String("*");
     }
@@ -70,10 +72,12 @@ public:
     QQuickFolderListModel::SortField sortField;
     QStringList nameFilters;
     bool sortReversed;
+    bool showFiles;
     bool showDirs;
     bool showDirsFirst;
-    bool showDots;
+    bool showDotAndDotDot;
     bool showOnlyReadable;
+    bool showHidden;
 
     ~QQuickFolderListModelPrivate() {}
     void init();
@@ -98,6 +102,7 @@ void QQuickFolderListModelPrivate::init()
                q, SLOT(_q_directoryUpdated(QString, QList<FileProperty>, int, int)));
     q->connect(&fileInfoThread, SIGNAL(sortFinished(QList<FileProperty>)),
                q, SLOT(_q_sortFinished(QList<FileProperty>)));
+    q->connect(q, SIGNAL(rowCountChanged()), q, SIGNAL(countChanged()));
 }
 
 
@@ -159,8 +164,10 @@ void QQuickFolderListModelPrivate::_q_directoryUpdated(const QString &directory,
         data = list;
         q->beginRemoveRows(parent, fromIndex, toIndex);
         q->endRemoveRows();
-        q->beginInsertRows(parent, fromIndex, list.size()-1);
-        q->endInsertRows();
+        if (list.size() > 0) {
+            q->beginInsertRows(parent, fromIndex, list.size()-1);
+            q->endInsertRows();
+        }
         emit q->rowCountChanged();
     } else if (data.size() < list.size()) {
         //qDebug() << "File added. FromIndex: " << fromIndex << " toIndex: " << toIndex << " list size: " << list.size();
@@ -188,9 +195,11 @@ void QQuickFolderListModelPrivate::_q_sortFinished(const QList<FileProperty> &li
     Q_Q(QQuickFolderListModel);
 
     QModelIndex parent;
-    q->beginRemoveRows(parent, 0, data.size()-1);
-    data.clear();
-    q->endRemoveRows();
+    if (data.size() > 0) {
+        q->beginRemoveRows(parent, 0, data.size()-1);
+        data.clear();
+        q->endRemoveRows();
+    }
 
     q->beginInsertRows(parent, 0, list.size()-1);
     data = list;
@@ -208,7 +217,7 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
 }
 
 /*!
-    \qmlmodule Qt.labs.folderlistmodel 1.0
+    \qmlmodule Qt.labs.folderlistmodel 2.1
     \title Qt Labs FolderListModel QML Types
     \ingroup qmlmodules
     \brief The FolderListModel provides a model of the contents of a file system folder.
@@ -216,14 +225,14 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
     To use this module, import the module with the following line:
 
     \code
-    import Qt.labs.folderlistmodel 1.0
+    import Qt.labs.folderlistmodel 2.1
     \endcode
 */
 
 
 /*!
     \qmltype FolderListModel
-    \inqmlmodule Qt.labs.folderlistmodel 1.0
+    \inqmlmodule Qt.labs.folderlistmodel
     \instantiates QQuickFolderListModel
     \ingroup qtquick-models
     \brief The FolderListModel provides a model of the contents of a file system folder.
@@ -235,7 +244,7 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
     \e{Elements in the Qt.labs module are not guaranteed to remain compatible
     in future versions.}
 
-    \b{import Qt.labs.folderlistmodel 1.0}
+    \b{import Qt.labs.folderlistmodel 2.1}
 
     The \l folder property specifies the folder to access. Information about the
     files and directories in the folder is supplied via the model's interface.
@@ -244,6 +253,7 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
     \list
     \li \c fileName
     \li \c filePath
+    \li \c fileURL (since Qt 5.2)
     \li \c fileBaseName
     \li \c fileSuffix
     \li \c fileSize
@@ -264,9 +274,10 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
     that are applied to names of files and directories, causing only those that
     match the filters to be exposed.
 
-    Directories can be included or excluded using the \l showDirs property, and
+    Directories can be included or excluded using the \l showDirs property,
     navigation directories can also be excluded by setting the \l showDotAndDotDot
-    property to false.
+    property to false, hidden files can be included or excluded using the
+    \l showHidden property.
 
     It is sometimes useful to limit the files and directories exposed to those
     that the user can access. The \l showOnlyReadable property can be set to
@@ -277,7 +288,27 @@ QString QQuickFolderListModelPrivate::resolvePath(const QUrl &path)
     The following example shows a FolderListModel being used to provide a list
     of QML files in a \l ListView:
 
-    \snippet qml/folderlistmodel.qml 0
+    \qml
+    import QtQuick 2.0
+    import Qt.labs.folderlistmodel 2.1
+
+    ListView {
+        width: 200; height: 400
+
+        FolderListModel {
+            id: folderModel
+            nameFilters: ["*.qml"]
+        }
+
+        Component {
+            id: fileDelegate
+            Text { text: fileName }
+        }
+
+        model: folderModel
+        delegate: fileDelegate
+    }
+    \endqml
 
     \section1 Path Separators
 
@@ -301,6 +332,7 @@ QQuickFolderListModel::QQuickFolderListModel(QObject *parent)
     d->roleNames[FileLastModifiedRole] = "fileModified";
     d->roleNames[FileLastReadRole] = "fileAccessed";
     d->roleNames[FileIsDirRole] = "fileIsDir";
+    d->roleNames[FileUrlRole] = "fileURL";
     d->init();
 }
 
@@ -334,14 +366,17 @@ QVariant QQuickFolderListModel::data(const QModelIndex &index, int role) const
             rv = d->data.at(index.row()).size();
             break;
         case FileLastModifiedRole:
-            rv = d->data.at(index.row()).lastModified().date().toString(Qt::ISODate) + " " + d->data.at(index.row()).lastModified().time().toString();
+            rv = d->data.at(index.row()).lastModified();
             break;
         case FileLastReadRole:
-            rv = d->data.at(index.row()).lastRead().date().toString(Qt::ISODate) + " " + d->data.at(index.row()).lastRead().time().toString();
+            rv = d->data.at(index.row()).lastRead();
             break;
         case FileIsDirRole:
             rv = d->data.at(index.row()).isDir();
             break;
+        case FileUrlRole:
+            rv = QUrl::fromLocalFile(d->data.at(index.row()).filePath());
+            break;
         default:
             break;
     }
@@ -378,8 +413,7 @@ QModelIndex QQuickFolderListModel::index(int row, int , const QModelIndex &) con
     The \a folder property holds a URL for the folder that the model is
     currently providing.
 
-    The value is a URL expressed as a string, and must be a \c file: or \c qrc:
-    URL, or a relative URL.
+    The value must be a \c file: or \c qrc: URL, or a relative URL.
 
     By default, the value is an invalid URL.
 */
@@ -419,7 +453,7 @@ void QQuickFolderListModel::setFolder(const QUrl &folder)
 
 
 /*!
-   \qmlproperty string QQuickFolderListModel::rootFolder
+   \qmlproperty url QQuickFolderListModel::rootFolder
 
    When the rootFolder is set, then this folder will
    be threated as the root in the file system, so that
@@ -461,16 +495,12 @@ QUrl QQuickFolderListModel::parentFolder() const
     QString localFile = d->currentDir.toLocalFile();
     if (!localFile.isEmpty()) {
         QDir dir(localFile);
-#if defined(Q_OS_WIN)
-        if (dir.isRoot())
-            dir.setPath("");
-        else
-#endif
-            dir.cdUp();
+        if (dir.isRoot() || !dir.cdUp())
+            return QUrl();
         localFile = dir.path();
     } else {
-        int pos = d->currentDir.path().lastIndexOf(QLatin1Char('/'));
-        if (pos == -1)
+        const int pos = d->currentDir.path().lastIndexOf(QLatin1Char('/'));
+        if (pos <= 0)
             return QUrl();
         localFile = d->currentDir.path().left(pos);
     }
@@ -514,8 +544,8 @@ void QQuickFolderListModel::componentComplete()
 {
     Q_D(QQuickFolderListModel);
 
-    if (!d->currentDir.isValid() || d->currentDir.toLocalFile().isEmpty() || !QDir().exists(d->currentDir.toLocalFile()))
-        setFolder(QUrl(QLatin1String("file://")+QDir::currentPath()));
+    if (!d->currentDir.isValid() || !d->currentDir.isLocalFile() || !QDir().exists(d->currentDir.toLocalFile()))
+        setFolder(QUrl::fromLocalFile(QDir::currentPath()));
 }
 
 /*!
@@ -596,6 +626,31 @@ bool QQuickFolderListModel::isFolder(int index) const
     return false;
 }
 
+/*!
+    \qmlproperty bool FolderListModel::showFiles
+    \since 5.2
+
+    If true, files are included in the model; otherwise only directories
+    are included.
+
+    By default, this property is true.
+
+    \sa showDirs
+*/
+bool QQuickFolderListModel::showFiles() const
+{
+    Q_D(const QQuickFolderListModel);
+    return d->showFiles;
+}
+
+void QQuickFolderListModel::setShowFiles(bool on)
+{
+    Q_D(QQuickFolderListModel);
+
+    d->fileInfoThread.setShowFiles(on);
+    d->showFiles = on;
+}
+
 /*!
     \qmlproperty bool FolderListModel::showDirs
 
@@ -659,15 +714,40 @@ void  QQuickFolderListModel::setShowDirsFirst(bool on)
 bool QQuickFolderListModel::showDotAndDotDot() const
 {
     Q_D(const QQuickFolderListModel);
-    return d->showDots;
+    return d->showDotAndDotDot;
 }
 
 void  QQuickFolderListModel::setShowDotAndDotDot(bool on)
 {
     Q_D(QQuickFolderListModel);
 
-    if (on != d->showDots) {
-        d->fileInfoThread.setShowDotDot(on);
+    if (on != d->showDotAndDotDot) {
+        d->fileInfoThread.setShowDotAndDotDot(on);
+    }
+}
+
+
+/*!
+    \qmlproperty bool FolderListModel::showHidden
+    \since 5.2
+
+    If true, hidden files and directories are included in the model; otherwise
+    they are excluded.
+
+    By default, this property is false.
+*/
+bool QQuickFolderListModel::showHidden() const
+{
+    Q_D(const QQuickFolderListModel);
+    return d->showHidden;
+}
+
+void QQuickFolderListModel::setShowHidden(bool on)
+{
+    Q_D(QQuickFolderListModel);
+
+    if (on != d->showHidden) {
+        d->fileInfoThread.setShowHidden(on);
     }
 }
 
@@ -697,7 +777,7 @@ void QQuickFolderListModel::setShowOnlyReadable(bool on)
 }
 
 /*!
-    \qmlmethod QVariant QQuickFolderListModel::get(int idx, const QString &property) const
+    \qmlmethod var FolderListModel::get(int index, string property)
 
     Get the folder property for the given index. The following properties
     are available.
@@ -705,6 +785,7 @@ void QQuickFolderListModel::setShowOnlyReadable(bool on)
     \list
         \li \c fileName
         \li \c filePath
+        \li \c fileURL (since Qt 5.2)
         \li \c fileBaseName
         \li \c fileSuffix
         \li \c fileSize
