@@ -43,8 +43,6 @@
 #define ELLIPSEEXTRUDER_H
 #include "qquickparticleextruder_p.h"
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickEllipseExtruder : public QQuickParticleExtruder
@@ -61,17 +59,17 @@ public:
         return m_fill;
     }
 
-signals:
+Q_SIGNALS:
 
     void fillChanged(bool arg);
 
-public slots:
+public Q_SLOTS:
 
     void setFill(bool arg)
     {
         if (m_fill != arg) {
             m_fill = arg;
-            emit fillChanged(arg);
+            Q_EMIT fillChanged(arg);
         }
     }
 private:
@@ -79,5 +77,4 @@ private:
 };
 
 QT_END_NAMESPACE
-QT_END_HEADER
 #endif // ELLIPSEEXTRUDER_H
