@@ -43,8 +43,6 @@
 #define GROUPGOALAFFECTOR_H
 #include "qquickparticleaffector_p.h"
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickStochasticEngine;
@@ -70,13 +68,13 @@ public:
 protected:
     virtual bool affectParticle(QQuickParticleData *d, qreal dt);
 
-signals:
+Q_SIGNALS:
 
     void goalStateChanged(QString arg);
 
     void jumpChanged(bool arg);
 
-public slots:
+public Q_SLOTS:
 
     void setGoalState(QString arg);
 
@@ -84,7 +82,7 @@ public slots:
     {
         if (m_jump != arg) {
             m_jump = arg;
-            emit jumpChanged(arg);
+            Q_EMIT jumpChanged(arg);
         }
     }
 
@@ -95,6 +93,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // GROUPGOALAFFECTOR_H
