@@ -46,8 +46,6 @@
 #include <QUrl>
 #include <QImage>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickMaskExtruder : public QQuickParticleExtruder
@@ -64,14 +62,14 @@ public:
         return m_source;
     }
 
-signals:
+Q_SIGNALS:
 
     void sourceChanged(QUrl arg);
 
-public slots:
+public Q_SLOTS:
     void setSource(QUrl arg);
 
-private slots:
+private Q_SLOTS:
     void startMaskLoading();
     void finishMaskLoading();
 
@@ -89,6 +87,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // MASKEXTRUDER_H
