@@ -57,8 +57,6 @@
 // We mean it.
 //
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -73,8 +71,6 @@ public:
 
     static QQmlDebugServer *instance();
 
-    void setConnection(QQmlDebugServerConnection *connection);
-
     bool hasDebuggingClient() const;
     bool blockingMode() const;
 
@@ -104,6 +100,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLDEBUGSERVICE_H
