@@ -58,8 +58,6 @@
 // We mean it.
 //
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -121,7 +119,5 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLDEBUGSERVICE_H
 
