@@ -58,8 +58,6 @@
 #include <QtQml/qtqmlglobal.h>
 #include <QtCore/QList>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -82,7 +80,7 @@ protected:
     virtual void stateChanged(State state);
     virtual void messageReceived(const QByteArray &);
 
-private slots:
+private Q_SLOTS:
     void processMessage(const QByteArray &message);
     void updateState();
 
@@ -96,6 +94,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLINSPECTORSERVICE_H
