@@ -45,6 +45,7 @@
 #include "qqmlcomponent_p.h"
 #include "qqmlcontext.h"
 #include "qqmlcontext_p.h"
+#include "qqmlpropertymap.h"
 #ifdef QML_THREADED_VME_INTERPRETER
 #include "qqmlvme_p.h"
 #endif
@@ -87,7 +88,7 @@ int QQmlCompiledData::indexForUrl(const QUrl &data)
 
 QQmlCompiledData::QQmlCompiledData(QQmlEngine *engine)
 : engine(engine), importCache(0), metaTypeId(-1), listMetaTypeId(-1), isRegisteredWithEngine(false),
-  rootPropertyCache(0)
+  rootPropertyCache(0), compilationUnit(0), qmlUnit(0)
 {
     Q_ASSERT(engine);
 
@@ -105,7 +106,7 @@ void QQmlCompiledData::destroy()
 QQmlCompiledData::~QQmlCompiledData()
 {
     if (isRegisteredWithEngine)
-        QQmlEnginePrivate::get(engine)->unregisterCompositeType(this);
+        QQmlEnginePrivate::get(engine)->unregisterInternalCompositeType(this);
 
     clear();
 
@@ -116,11 +117,17 @@ QQmlCompiledData::~QQmlCompiledData()
             types.at(ii).typePropertyCache->release();
     }
 
-    for (int ii = 0; ii < propertyCaches.count(); ++ii) 
-        propertyCaches.at(ii)->release();
+    for (QHash<int, TypeReference>::Iterator resolvedType = resolvedTypes.begin(), end = resolvedTypes.end();
+         resolvedType != end; ++resolvedType) {
+        if (resolvedType->component)
+            resolvedType->component->release();
+        if (resolvedType->typePropertyCache)
+            resolvedType->typePropertyCache->release();
+    }
 
-    for (int ii = 0; ii < contextCaches.count(); ++ii)
-        contextCaches.at(ii)->release();
+    for (int ii = 0; ii < propertyCaches.count(); ++ii) 
+        if (propertyCaches.at(ii))
+            propertyCaches.at(ii)->release();
 
     for (int ii = 0; ii < scripts.count(); ++ii)
         scripts.at(ii)->release();
@@ -130,12 +137,14 @@ QQmlCompiledData::~QQmlCompiledData()
 
     if (rootPropertyCache)
         rootPropertyCache->release();
+
+    if (compilationUnit)
+        compilationUnit->deref();
+    free(qmlUnit);
 }
 
 void QQmlCompiledData::clear()
 {
-    for (int ii = 0; ii < programs.count(); ++ii)
-        qPersistentDispose(programs[ii].bindings);
 }
 
 /*!
@@ -165,6 +174,27 @@ QQmlPropertyCache *QQmlCompiledData::TypeReference::createPropertyCache(QQmlEngi
     }
 }
 
+template <typename T>
+bool qtTypeInherits(const QMetaObject *mo) {
+    while (mo) {
+        if (mo == &T::staticMetaObject)
+            return true;
+        mo = mo->superClass();
+    }
+    return false;
+}
+
+void QQmlCompiledData::TypeReference::doDynamicTypeCheck()
+{
+    const QMetaObject *mo = 0;
+    if (typePropertyCache)
+        mo = typePropertyCache->firstCppMetaObject();
+    else if (type)
+        mo = type->metaObject();
+    else
+        mo = component->rootPropertyCache->firstCppMetaObject();
+    isFullyDynamicType = qtTypeInherits<QQmlPropertyMap>(mo);
+}
 
 void QQmlCompiledData::dumpInstructions()
 {
@@ -223,7 +253,6 @@ QQmlInstruction::Type QQmlCompiledData::instructionType(const QQmlInstruction *i
         return QQmlInstruction::I;
 
     FOR_EACH_QML_INSTR(QML_CHECK_INSTR_CODE)
-    Q_UNREACHABLE();
     Q_ASSERT_X(false, Q_FUNC_INFO, "Invalid instruction address");
     return static_cast<QQmlInstruction::Type>(0);
 #  undef QML_CHECK_INSTR_CODE
@@ -236,6 +265,9 @@ void QQmlCompiledData::initialize(QQmlEngine *engine)
 {
     Q_ASSERT(!hasEngine());
     QQmlCleanup::addToEngine(engine);
+    QV4::ExecutionEngine *v4 = QV8Engine::getV4(engine);
+    if (compilationUnit && !compilationUnit->engine)
+        compilationUnit->linkToEngine(v4);
 }
 
 QT_END_NAMESPACE
