@@ -49,8 +49,6 @@
 #include <QtCore/qstring.h>
 #include <QtQml/qjsvalue.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -58,7 +56,7 @@ class QByteArray;
 class QQmlEngine;
 class QQmlComponent;
 class QQmlIncubator;
-class QQmlV8Function;
+class QQmlV4Function;
 class QQmlCompiledData;
 class QQmlComponentPrivate;
 class QQmlComponentAttached;
@@ -122,8 +120,8 @@ Q_SIGNALS:
 
 protected:
     QQmlComponent(QQmlComponentPrivate &dd, QObject* parent);
-    Q_INVOKABLE void createObject(QQmlV8Function *);
-    Q_INVOKABLE void incubateObject(QQmlV8Function *);
+    Q_INVOKABLE void createObject(QQmlV4Function *);
+    Q_INVOKABLE void incubateObject(QQmlV4Function *);
 
 private:
     QQmlComponent(QQmlEngine *, QQmlCompiledData *, int, QObject *parent);
@@ -131,6 +129,7 @@ private:
     Q_DISABLE_COPY(QQmlComponent)
     friend class QQmlVME;
     friend class QQmlTypeData;
+    friend class QmlObjectCreator;
 };
 
 QT_END_NAMESPACE
@@ -139,6 +138,4 @@ Q_DECLARE_METATYPE(QQmlComponent::Status)
 QML_DECLARE_TYPE(QQmlComponent)
 QML_DECLARE_TYPEINFO(QQmlComponent, QML_HAS_ATTACHED_PROPERTIES)
 
-QT_END_HEADER
-
 #endif // QQMLCOMPONENT_H
