@@ -48,8 +48,7 @@
 #include "qqmlengine_p.h"
 #include "qqmlengine.h"
 #include "qqmlinfo.h"
-#include <private/qv4bindings_p.h>
-#include <private/qv8bindings_p.h>
+#include "qqmlabstracturlinterceptor.h"
 
 #include <qjsengine.h>
 #include <QtCore/qvarlengtharray.h>
@@ -313,11 +312,12 @@ void QQmlContext::setContextProperty(const QString &name, const QVariant &value)
         }
     }
 
-    if (!data->propertyNames) data->propertyNames = new QQmlIntegerCache();
+    if (data->propertyNames.isEmpty())
+        data->propertyNames = QV4::IdentifierHash<int>(QV8Engine::getV4(engine()->handle()));
 
-    int idx = data->propertyNames->value(name);
+    int idx = data->propertyNames.value(name);
     if (idx == -1) {
-        data->propertyNames->add(name, data->idValueCount + d->propertyValues.count());
+        data->propertyNames.add(name, data->idValueCount + d->propertyValues.count());
         d->propertyValues.append(value);
 
         data->refreshExpressions();
@@ -350,11 +350,12 @@ void QQmlContext::setContextProperty(const QString &name, QObject *value)
         return;
     }
 
-    if (!data->propertyNames) data->propertyNames = new QQmlIntegerCache();
-    int idx = data->propertyNames->value(name);
+    if (data->propertyNames.isEmpty())
+        data->propertyNames = QV4::IdentifierHash<int>(QV8Engine::getV4(engine()->handle()));
+    int idx = data->propertyNames.value(name);
 
     if (idx == -1) {
-        data->propertyNames->add(name, data->idValueCount + d->propertyValues.count());
+        data->propertyNames.add(name, data->idValueCount + d->propertyValues.count());
         d->propertyValues.append(QVariant::fromValue(value));
 
         data->refreshExpressions();
@@ -376,8 +377,8 @@ QVariant QQmlContext::contextProperty(const QString &name) const
 
     QQmlContextData *data = d->data;
 
-    if (data->propertyNames)
-        idx = data->propertyNames->value(name);
+    if (data->propertyNames.count())
+        idx = data->propertyNames.value(name);
 
     if (idx == -1) {
         QByteArray utf8Name = name.toUtf8();
@@ -431,6 +432,7 @@ QUrl QQmlContextData::resolvedUrl(const QUrl &src)
 {
     QQmlContextData *ctxt = this;
 
+    QUrl resolved;
     if (src.isRelative() && !src.isEmpty()) {
         if (ctxt) {
             while(ctxt) {
@@ -441,14 +443,20 @@ QUrl QQmlContextData::resolvedUrl(const QUrl &src)
             }
 
             if (ctxt)
-                return ctxt->url.resolved(src);
+                resolved = ctxt->url.resolved(src);
             else if (engine)
-                return engine->baseUrl().resolved(src);
+                resolved = engine->baseUrl().resolved(src);
         }
-        return QUrl();
     } else {
-        return src;
+        resolved = src;
     }
+
+    if (resolved.isEmpty()) //relative but no ctxt
+        return resolved;
+
+    if (engine && engine->urlInterceptor())
+        resolved = engine->urlInterceptor()->intercept(resolved, QQmlAbstractUrlInterceptor::UrlString);
+    return resolved;
 }
 
 
@@ -516,9 +524,9 @@ QQmlContextData::QQmlContextData()
 : parent(0), engine(0), isInternal(false), ownedByParent(false), isJSContext(false), 
   isPragmaLibraryContext(false), unresolvedNames(false), hasEmittedDestruction(false), isRootObjectInCreation(false),
   publicContext(0), activeVMEData(0),
-  propertyNames(0), contextObject(0), imports(0), childContexts(0), nextChild(0), prevChild(0),
+  contextObject(0), imports(0), childContexts(0), nextChild(0), prevChild(0),
   expressions(0), contextObjects(0), contextGuards(0), idValues(0), idValueCount(0), linkedContext(0),
-  componentAttached(0), v4bindings(0), v8bindings(0)
+  componentAttached(0)
 {
 }
 
@@ -526,9 +534,9 @@ QQmlContextData::QQmlContextData(QQmlContext *ctxt)
 : parent(0), engine(0), isInternal(false), ownedByParent(false), isJSContext(false), 
   isPragmaLibraryContext(false), unresolvedNames(false), hasEmittedDestruction(false), isRootObjectInCreation(false),
   publicContext(ctxt), activeVMEData(0),
-  propertyNames(0), contextObject(0), imports(0), childContexts(0), nextChild(0), prevChild(0),
+  contextObject(0), imports(0), childContexts(0), nextChild(0), prevChild(0),
   expressions(0), contextObjects(0), contextGuards(0), idValues(0), idValueCount(0), linkedContext(0),
-  componentAttached(0), v4bindings(0), v8bindings(0)
+  componentAttached(0)
 {
 }
 
@@ -630,22 +638,9 @@ void QQmlContextData::destroy()
     }
     contextGuards = 0;
 
-    if (propertyNames)
-        propertyNames->release();
-
     if (imports)
         imports->release();
 
-    if (v4bindings)
-        v4bindings->release();
-
-    if (v8bindings)
-        v8bindings->release();
-
-    for (int ii = 0; ii < importedScripts.count(); ++ii) {
-        qPersistentDispose(importedScripts[ii]);
-    }
-
     delete [] idValues;
 
     if (isInternal)
@@ -775,31 +770,33 @@ void QQmlContextData::setIdProperty(int idx, QObject *obj)
     idValues[idx].context = this;
 }
 
-void QQmlContextData::setIdPropertyData(QQmlIntegerCache *data)
+void QQmlContextData::setIdPropertyData(const QVector<ObjectIdMapping> &data)
 {
-    Q_ASSERT(!propertyNames);
-    propertyNames = data;
-    propertyNames->addref();
+    Q_ASSERT(propertyNames.isEmpty());
+    propertyNames = QV4::IdentifierHash<int>(QV8Engine::getV4(engine->handle()));
+    for (QVector<ObjectIdMapping>::ConstIterator it = data.begin(), end = data.end();
+         it != end; ++it)
+        propertyNames.add(it->name, it->id);
 
-    idValueCount = data->count();
+    idValueCount = data.count();
     idValues = new ContextGuard[idValueCount];
 }
 
 QString QQmlContextData::findObjectId(const QObject *obj) const
 {
-    if (!propertyNames)
+    if (propertyNames.isEmpty())
         return QString();
 
     for (int ii = 0; ii < idValueCount; ii++) {
         if (idValues[ii] == obj)
-            return propertyNames->findId(ii);
+            return propertyNames.findId(ii);
     }
 
     if (publicContext) {
         QQmlContextPrivate *p = QQmlContextPrivate::get(publicContext);
         for (int ii = 0; ii < p->propertyValues.count(); ++ii)
             if (p->propertyValues.at(ii) == QVariant::fromValue((QObject *)obj))
-                return propertyNames->findId(ii);
+                return propertyNames.findId(ii);
     }
 
     if (linkedContext)
