@@ -55,11 +55,13 @@
 
 #include <private/qtqmlglobal_p.h>
 #include <private/qobject_p.h>
-#include <private/qv8_p.h>
+
+#include <private/qv4value_p.h>
 
 QT_BEGIN_NAMESPACE
 
 template <class Key, class T> class QHash;
+class QQmlEngine;
 class QQmlGuardImpl;
 class QQmlCompiledData;
 class QQmlAbstractBinding;
@@ -78,11 +80,11 @@ class Q_QML_PRIVATE_EXPORT QQmlData : public QAbstractDeclarativeData
 {
 public:
     QQmlData()
-        : ownMemory(true), ownContext(false), indestructible(true), explicitIndestructibleSet(false), 
+        : ownedByQml1(false), ownMemory(true), ownContext(false), indestructible(true), explicitIndestructibleSet(false),
           hasTaintedV8Object(false), isQueuedForDeletion(false), rootObjectInCreation(false),
           hasVMEMetaObject(false), parentFrozen(false), notifyList(0), context(0), outerContext(0),
           bindings(0), signalHandlers(0), nextContextObject(0), prevContextObject(0), bindingBitsSize(0), bindingBits(0),
-          lineNumber(0), columnNumber(0), compiledData(0), deferredIdx(0), v8objectid(0),
+          lineNumber(0), columnNumber(0), compiledData(0), deferredData(0), jsEngineId(0),
           propertyCache(0), guards(0), extendedData(0) {
         init();
     }
@@ -112,6 +114,7 @@ public:
         if (!explicitIndestructibleSet) indestructible = false;
     }
 
+    quint32 ownedByQml1:1; // This bit is shared with QML1's QDeclarativeData.
     quint32 ownMemory:1;
     quint32 ownContext:1;
     quint32 indestructible:1;
@@ -125,7 +128,7 @@ public:
     quint32 rootObjectInCreation:1;
     quint32 hasVMEMetaObject:1;
     quint32 parentFrozen:1;
-    quint32 dummy:23;
+    quint32 dummy:22;
 
     struct NotifyList {
         quint64 connectionMask;
@@ -173,11 +176,16 @@ public:
     quint16 lineNumber;
     quint16 columnNumber;
 
+    struct DeferredData {
+        unsigned int deferredIdx;
+        QQmlCompiledData *compiledData;//Not always the same as the other compiledData
+        QQmlContextData *context;//Could be either context or outerContext
+    };
     QQmlCompiledData *compiledData;
-    unsigned int deferredIdx;
+    DeferredData *deferredData;
 
-    quint32 v8objectid;
-    v8::Persistent<v8::Object> v8object;
+    quint32 jsEngineId; // id of the engine that cerated the jsWrapper
+    QV4::WeakValue jsWrapper;
 
     QQmlPropertyCache *propertyCache;
 
@@ -198,6 +206,13 @@ public:
         }
     }
 
+    static bool keepAliveDuringGarbageCollection(const QObject *object) {
+        QQmlData *ddata = get(object);
+        if (!ddata || ddata->indestructible || ddata->rootObjectInCreation)
+            return true;
+        return false;
+    }
+
     bool hasExtendedData() const { return extendedData != 0; }
     QHash<int, QObject *> *attachedProperties() const;
 
@@ -208,6 +223,8 @@ public:
 
     static inline void flushPendingBinding(QObject *, int coreIndex);
 
+    static void ensurePropertyCache(QQmlEngine *engine, QObject *object);
+
 private:
     // For attachedProperties
     mutable QQmlDataExtended *extendedData;
