@@ -55,8 +55,6 @@
 #include "qqmlxmlhttprequest_p.h"
 #include "qqmlscriptstring.h"
 #include "qqmlglobal_p.h"
-#include "qquicklistmodel_p.h"
-#include "qquickworkerscript_p.h"
 #include "qqmlcomponent_p.h"
 #include "qqmlnetworkaccessmanagerfactory.h"
 #include "qqmldirparser_p.h"
@@ -65,9 +63,10 @@
 #include "qqmltypenamecache_p.h"
 #include "qqmlnotifier_p.h"
 #include <private/qqmlprofilerservice_p.h>
-#include <private/qv8debugservice_p.h>
+#include <private/qv4debugservice_p.h>
 #include <private/qdebugmessageservice_p.h>
 #include "qqmlincubator.h"
+#include "qqmlabstracturlinterceptor.h"
 #include <private/qv8profilerservice_p.h>
 #include <private/qqmlboundsignal_p.h>
 
@@ -89,12 +88,22 @@
 
 #include <private/qqmllocale_p.h>
 
-#include "qqmlbind_p.h"
-#include "qqmlconnections_p.h"
-#include "qqmltimer_p.h"
+#include <private/qqmlbind_p.h>
+#include <private/qqmlconnections_p.h>
+#include <private/qqmltimer_p.h>
+#include <private/qqmllistmodel_p.h>
+#include <private/qqmlplatform_p.h>
+#include <private/qquickpackage_p.h>
+#include <private/qqmldelegatemodel_p.h>
+#include <private/qqmlobjectmodel_p.h>
+#include <private/qquickworkerscript_p.h>
+#include <private/qqmlinstantiator_p.h>
 
 #ifdef Q_OS_WIN // for %APPDATA%
 #include <qt_windows.h>
+#  if !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
+#    include <shlobj.h>
+#  endif
 #include <qlibrary.h>
 #include <windows.h>
 
@@ -115,7 +124,7 @@ void qmlRegisterBaseTypes(const char *uri, int versionMajor, int versionMinor)
 /*!
   \qmltype QtObject
     \instantiates QObject
-  \inqmlmodule QtQuick 2
+  \inqmlmodule QtQml
   \ingroup qml-utility-elements
   \brief A basic QML type
 
@@ -179,16 +188,22 @@ void QQmlEnginePrivate::registerBaseTypes(const char *uri, int versionMajor, int
     qmlRegisterType<QQmlBind>(uri, versionMajor, versionMinor,"Binding");
     qmlRegisterType<QQmlConnections>(uri, versionMajor, versionMinor,"Connections");
     qmlRegisterType<QQmlTimer>(uri, versionMajor, versionMinor,"Timer");
+    qmlRegisterType<QQmlInstantiator>(uri, versionMajor, (versionMinor < 1 ? 1 : versionMinor), "Instantiator"); //Only available in >=2.1
     qmlRegisterCustomType<QQmlConnections>(uri, versionMajor, versionMinor,"Connections", new QQmlConnectionsParser);
+    qmlRegisterType<QQmlInstanceModel>();
 }
 
 
 // These QtQuick types' implementation resides in the QtQml module
 void QQmlEnginePrivate::registerQtQuick2Types(const char *uri, int versionMajor, int versionMinor)
 {
-    qmlRegisterType<QQuickListElement>(uri, versionMajor, versionMinor, "ListElement");
-    qmlRegisterCustomType<QQuickListModel>(uri, versionMajor, versionMinor, "ListModel", new QQuickListModelParser);
+    qmlRegisterType<QQmlListElement>(uri, versionMajor, versionMinor, "ListElement"); // Now in QtQml.Models, here for compatibility
+    qmlRegisterCustomType<QQmlListModel>(uri, versionMajor, versionMinor, "ListModel", new QQmlListModelParser); // Now in QtQml.Models, here for compatibility
     qmlRegisterType<QQuickWorkerScript>(uri, versionMajor, versionMinor, "WorkerScript");
+    qmlRegisterType<QQuickPackage>(uri, versionMajor, versionMinor, "Package");
+    qmlRegisterType<QQmlDelegateModel>(uri, versionMajor, versionMinor, "VisualDataModel");
+    qmlRegisterType<QQmlDelegateModelGroup>(uri, versionMajor, versionMinor, "VisualDataGroup");
+    qmlRegisterType<QQmlObjectModel>(uri, versionMajor, versionMinor, "VisualItemModel");
 }
 
 void QQmlEnginePrivate::defineQtQuick2Module()
@@ -263,13 +278,11 @@ QQmlImageProviderBase::~QQmlImageProviderBase()
 
 /*!
 \qmltype Qt
-    \instantiates QQmlEnginePrivate
-  \ingroup qml-utility-elements
-\brief The QML global Qt object provides useful enums and functions from Qt.
-
+\inqmlmodule QtQml
+\instantiates QQmlEnginePrivate
+\ingroup qml-utility-elements
 \keyword QmlGlobalQtObject
-
-\brief The \c Qt object provides useful enums and functions from Qt, for use in all QML files.
+\brief Provides a global object with useful enums and functions from Qt.
 
 The \c Qt object is a global object with utility functions, properties and enums.
 
@@ -303,7 +316,7 @@ when the property has one of the following types:
 \li \c size - use \l{Qt::size()}{Qt.size()}
 \endlist
 
-If the QtQuick module has been imported, the following helper functions for
+If the \c QtQuick module has been imported, the following helper functions for
 creating objects of specific data types are also available for clients to use:
 \list
 \li \c color - use \l{Qt::rgba()}{Qt.rgba()}, \l{Qt::hsla()}{Qt.hsla()}, \l{Qt::darker()}{Qt.darker()}, \l{Qt::lighter()}{Qt.lighter()} or \l{Qt::tint()}{Qt.tint()}
@@ -358,9 +371,39 @@ The following functions are also on the Qt object.
 \endlist
 */
 
+/*!
+    \qmlproperty object Qt::platform
+    \since 4.8
+
+    The \c platform object provides info about the underlying platform.
+
+    Its properties are:
+
+    \table
+    \row
+    \li \c platform.os
+    \li
+
+    This read-only property contains the name of the operating system.
+
+    Possible values are:
+
+    \list
+        \li \c "android" - Android
+        \li \c "blackberry" - BlackBerry OS
+        \li \c "ios" - iOS
+        \li \c "linux" - Linux
+        \li \c "osx" - OS X
+        \li \c "unix" - Other Unix-based OS
+        \li \c "windows" - Windows
+        \li \c "wince" - Windows CE
+    \endlist
+    \endtable
+*/
+
 /*!
     \qmlproperty object Qt::application
-    \since QtQuick 1.1
+    \since 5.1
 
     The \c application object provides access to global application state
     properties shared by many QML components.
@@ -371,14 +414,50 @@ The following functions are also on the Qt object.
     \row
     \li \c application.active
     \li
-    This read-only property indicates whether the application is the top-most and focused
-    application, and the user is able to interact with the application. The property
-    is false when the application is in the background, the device keylock or screen
-    saver is active, the screen backlight is turned off, or the global system dialog
-    is being displayed on top of the application. It can be used for stopping and
-    pausing animations, timers and active processing of data in order to save device
-    battery power and free device memory and processor load when the application is not
-    active.
+    Deprecated, use Qt.application.state == Qt.ApplicationActive instead.
+
+    \row
+    \li \c application.state
+    \li
+    This read-only property indicates the current state of the application.
+
+    Possible values are:
+
+    \list
+    \li Qt.ApplicationActive - The application is the top-most and focused application, and the
+                               user is able to interact with the application.
+    \li Qt.ApplicationInactive - The application is visible or partially visible, but not selected
+                                 to be in front, the user cannot interact with the application.
+                                 On desktop platforms, this typically means that the user activated
+                                 another application. On mobile platforms, it is more common to
+                                 enter this state when the OS is interrupting the user with for
+                                 example incoming calls, SMS-messages or dialogs. This is usually a
+                                 transient state during which the application is paused. The user
+                                 may return focus to your application, but most of the time it will
+                                 be the first indication that the application is going to be suspended.
+                                 While in this state, consider pausing or stopping any activity that
+                                 should not continue when the user cannot interact with your
+                                 application, such as a video, a game, animations, or sensors.
+                                 You should also avoid performing CPU-intensive tasks which might
+                                 slow down the application in front.
+    \li Qt.ApplicationSuspended - The application is suspended and not visible to the user. On
+                                  mobile platforms, the application typically enters this state when
+                                  the user returns to the home screen or switches to another
+                                  application. While in this state, the application should ensure
+                                  that the user perceives it as always alive and does not lose his
+                                  progress, saving any persistent data. The application should cease
+                                  all activities and be prepared for code execution to stop. While
+                                  suspended, the application can be killed at any time without
+                                  further warnings (for example when low memory forces the OS to purge
+                                  suspended applications).
+    \li Qt.ApplicationHidden - The application is hidden and runs in the background. This is the
+                               normal state for applications that need to do background processing,
+                               like playing music, while the user interacts with other applications.
+                               The application should free up all graphical resources when entering
+                               this state. A Qt Quick application should not usually handle this state
+                               at the QML level. Instead, you should unload the entire UI and reload
+                               the QML files whenever the application becomes active again.
+    \endlist
 
     \row
     \li \c application.layoutDirection
@@ -398,18 +477,45 @@ The following functions are also on the Qt object.
     \li Qt.RightToLeft - Text and graphics elements should be positioned
                         from right to left.
     \endlist
-
+    \row
+    \li \c application.arguments
+    \li This is a string list of the arguments the executable was invoked with.
+    \row
+    \li \c application.name
+    \li This is the application name set on the QCoreApplication instance. This property can be written
+    to in order to set the application name.
+    \row
+    \li \c application.version
+    \li This is the application version set on the QCoreApplication instance. This property can be written
+    to in order to set the application version.
+    \row
+    \li \c application.organization
+    \li This is the organization name set on the QCoreApplication instance. This property can be written
+    to in order to set the organization name.
+    \row
+    \li \c application.domain
+    \li This is the organization domain set on the QCoreApplication instance. This property can be written
+    to in order to set the organization domain.
     \endtable
 
+    The object also has one signal, aboutToQuit(), which is the same as \l QCoreApplication::aboutToQuit().
+
     The following example uses the \c application object to indicate
     whether the application is currently active:
 
     \snippet qml/application.qml document
+
+    Note that when using QML without a QGuiApplication, the following properties will be undefined:
+    \list
+    \li application.active
+    \li application.state
+    \li application.layoutDirection
+    \endlist
 */
 
 /*!
     \qmlproperty object Qt::inputMethod
-    \since QtQuick 2.0
+    \since 5.0
 
     The \c inputMethod object allows access to application's QInputMethod object
     and all its properties and slots. See the QInputMethod documentation for
@@ -442,18 +548,21 @@ The \c status property will be updated as the operation progresses.
 If provided, \a callback is invoked when the operation completes.  The callback is passed
 the same object as is returned from the Qt.include() call.
 */
-// Qt.include() is implemented in qv8include.cpp
+// Qt.include() is implemented in qv4include.cpp
 
+DEFINE_BOOL_CONFIG_OPTION(qmlUseNewCompiler, QML_NEW_COMPILER)
 
 QQmlEnginePrivate::QQmlEnginePrivate(QQmlEngine *e)
 : propertyCapture(0), rootContext(0), isDebugging(false),
-  outputWarningsToStdErr(true), sharedContext(0), sharedScope(0),
+  outputWarningsToStdErr(true),
   cleanup(0), erroredBindings(0), inProgressCreations(0),
   workerScriptEngine(0), activeVME(0),
-  networkAccessManager(0), networkAccessManagerFactory(0),
-  scarceResourcesRefCount(0), typeLoader(e), importDatabase(e), uniqueId(1),
+  activeObjectCreator(0),
+  networkAccessManager(0), networkAccessManagerFactory(0), urlInterceptor(0),
+  scarceResourcesRefCount(0), importDatabase(e), typeLoader(e), uniqueId(1),
   incubatorCount(0), incubationController(0), mutex(QMutex::Recursive)
 {
+    useNewCompiler = qmlUseNewCompiler();
 }
 
 QQmlEnginePrivate::~QQmlEnginePrivate()
@@ -509,12 +618,18 @@ void QQmlPrivate::qdeclarativeelement_destructor(QObject *o)
 
 void QQmlData::destroyed(QAbstractDeclarativeData *d, QObject *o)
 {
-    static_cast<QQmlData *>(d)->destroyed(o);
+    QQmlData *ddata = static_cast<QQmlData *>(d);
+    if (ddata->ownedByQml1)
+        return;
+    ddata->destroyed(o);
 }
 
 void QQmlData::parentChanged(QAbstractDeclarativeData *d, QObject *o, QObject *p)
 {
-    static_cast<QQmlData *>(d)->parentChanged(o, p);
+    QQmlData *ddata = static_cast<QQmlData *>(d);
+    if (ddata->ownedByQml1)
+        return;
+    ddata->parentChanged(o, p);
 }
 
 class QQmlThreadNotifierProxyObject : public QObject
@@ -543,6 +658,7 @@ void QQmlData::signalEmitted(QAbstractDeclarativeData *, QObject *object, int in
 {
     QQmlData *ddata = QQmlData::get(object, false);
     if (!ddata) return; // Probably being deleted
+    if (ddata->ownedByQml1) return;
 
     // In general, QML only supports QObject's that live on the same thread as the QQmlEngine
     // that they're exposed to.  However, to make writing "worker objects" that calculate data
@@ -600,12 +716,18 @@ void QQmlData::signalEmitted(QAbstractDeclarativeData *, QObject *object, int in
 
 int QQmlData::receivers(QAbstractDeclarativeData *d, const QObject *, int index)
 {
-    return static_cast<QQmlData *>(d)->endpointCount(index);
+    QQmlData *ddata = static_cast<QQmlData *>(d);
+    if (ddata->ownedByQml1)
+        return 0;
+    return ddata->endpointCount(index);
 }
 
 bool QQmlData::isSignalConnected(QAbstractDeclarativeData *d, const QObject *, int index)
 {
-    return static_cast<QQmlData *>(d)->signalHasEndpoint(index);
+    QQmlData *ddata = static_cast<QQmlData *>(d);
+    if (ddata->ownedByQml1)
+        return false;
+    return ddata->signalHasEndpoint(index);
 }
 
 int QQmlData::endpointCount(int index)
@@ -662,17 +784,18 @@ void QQmlData::flushPendingBindingImpl(int coreIndex)
     }
 }
 
+bool QQmlEnginePrivate::baseModulesUninitialized = true;
 void QQmlEnginePrivate::init()
 {
     Q_Q(QQmlEngine);
 
-    static bool firstTime = true;
-    if (firstTime) {
+    if (baseModulesUninitialized) {
         qmlRegisterType<QQmlComponent>("QML", 1, 0, "Component"); // required for the Compiler.
         registerBaseTypes("QtQml", 2, 0); // import which provides language building blocks.
+        qmlRegisterUncreatableType<QQmlLocale>("QtQml", 2, 2, "Locale", QQmlEngine::tr("Locale cannot be instantiated.  Use Qt.locale()"));
 
         QQmlData::init();
-        firstTime = false;
+        baseModulesUninitialized = false;
     }
 
     qRegisterMetaType<QVariant>();
@@ -681,7 +804,7 @@ void QQmlEnginePrivate::init()
     qRegisterMetaType<QQmlComponent::Status>();
     qRegisterMetaType<QList<QObject*> >();
     qRegisterMetaType<QList<int> >();
-    qRegisterMetaType<QQmlV8Handle>();
+    qRegisterMetaType<QQmlV4Handle>();
 
     v8engine()->setEngine(q);
 
@@ -691,17 +814,11 @@ void QQmlEnginePrivate::init()
         QQmlEngineDebugService::isDebuggingEnabled()) {
         isDebugging = true;
         QQmlEngineDebugService::instance()->addEngine(q);
-        QV8DebugService::initialize(v8engine());
+        QV4DebugService::instance()->addEngine(q);
         QV8ProfilerService::initialize();
         QQmlProfilerService::initialize();
         QDebugMessageService::instance();
     }
-
-    QString dataLocation = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
-    if (!dataLocation.isEmpty())
-        offlineStoragePath = dataLocation.replace(QLatin1Char('/'), QDir::separator())
-                           + QDir::separator() + QLatin1String("QML")
-                           + QDir::separator() + QLatin1String("OfflineStorage");
 }
 
 QQuickWorkerScriptEngine *QQmlEnginePrivate::getWorkerScriptEngine()
@@ -741,7 +858,7 @@ QQuickWorkerScriptEngine *QQmlEnginePrivate::getWorkerScriptEngine()
   In this case, the Text item will be created in the engine's
   \l {QQmlEngine::rootContext()}{root context}.
 
-  Note that the QtQuick 1 version is called QDeclarativeEngine.
+  Note that the \l {Qt Quick 1} version is called QDeclarativeEngine.
 
   \sa QQmlComponent, QQmlContext
 */
@@ -756,18 +873,31 @@ QQmlEngine::QQmlEngine(QObject *parent)
     d->init();
 }
 
+/*!
+* \internal
+*/
+QQmlEngine::QQmlEngine(QQmlEnginePrivate &dd, QObject *parent)
+: QJSEngine(dd, parent)
+{
+    Q_D(QQmlEngine);
+    d->init();
+}
+
 /*!
   Destroys the QQmlEngine.
 
   Any QQmlContext's created on this engine will be
   invalidated, but not destroyed (unless they are parented to the
   QQmlEngine object).
+
+  See QJSEngine docs for details on cleaning up the JS engine.
 */
 QQmlEngine::~QQmlEngine()
 {
     Q_D(QQmlEngine);
     if (d->isDebugging) {
         QQmlEngineDebugService::instance()->remEngine(this);
+        QV4DebugService::instance()->removeEngine(this);
     }
 
     // Emit onDestruction signals for the root context before
@@ -782,9 +912,6 @@ QQmlEngine::~QQmlEngine()
     QList<QQmlType*> singletonTypes = QQmlMetaType::qmlSingletonTypes();
     foreach (QQmlType *currType, singletonTypes)
         currType->singletonInstanceInfo()->destroy(this);
-
-    if (d->incubationController)
-        d->incubationController->d = 0;
 }
 
 /*! \fn void QQmlEngine::quit()
@@ -853,6 +980,35 @@ QQmlContext *QQmlEngine::rootContext() const
     return d->rootContext;
 }
 
+/*!
+  \internal
+  This API is private for 5.1
+
+  Sets the \a urlInterceptor to be used when resolving URLs in QML.
+  This also applies to URLs used for loading script files and QML types.
+  This should not be modifed while the engine is loading files, or URL
+  selection may be inconsistent.
+*/
+void QQmlEngine::setUrlInterceptor(QQmlAbstractUrlInterceptor *urlInterceptor)
+{
+    Q_D(QQmlEngine);
+    d->urlInterceptor = urlInterceptor;
+}
+
+/*!
+  \internal
+  This API is private for 5.1
+
+  Returns the current QQmlAbstractUrlInterceptor. It must not be modified outside
+  the GUI thread.
+*/
+QQmlAbstractUrlInterceptor *QQmlEngine::urlInterceptor() const
+{
+    Q_D(const QQmlEngine);
+    return d->urlInterceptor;
+}
+
+
 /*!
   Sets the \a factory to use for creating QNetworkAccessManager(s).
 
@@ -884,7 +1040,9 @@ QQmlNetworkAccessManagerFactory *QQmlEngine::networkAccessManagerFactory() const
 void QQmlEnginePrivate::registerFinalizeCallback(QObject *obj, int index)
 {
     if (activeVME) {
-        activeVME->finalizeCallbacks.append(qMakePair(QQmlGuard<QObject>(obj), index));
+        activeVME->finalizeCallbacks.append(qMakePair(QPointer<QObject>(obj), index));
+    } else if (activeObjectCreator) {
+        activeObjectCreator->finalizeCallbacks.append(qMakePair(QPointer<QObject>(obj), index));
     } else {
         void *args[] = { 0 };
         QMetaObject::metacall(obj, QMetaObject::InvokeMetaMethod, index, args);
@@ -963,7 +1121,7 @@ QQmlImageProviderBase *QQmlEngine::imageProvider(const QString &providerId) cons
 {
     Q_D(const QQmlEngine);
     QMutexLocker locker(&d->mutex);
-    return d->imageProviders.value(providerId).data();
+    return d->imageProviders.value(providerId.toLower()).data();
 }
 
 /*!
@@ -975,7 +1133,7 @@ void QQmlEngine::removeImageProvider(const QString &providerId)
 {
     Q_D(QQmlEngine);
     QMutexLocker locker(&d->mutex);
-    d->imageProviders.take(providerId);
+    d->imageProviders.take(providerId.toLower());
 }
 
 /*!
@@ -1177,27 +1335,22 @@ void QQmlEnginePrivate::doDeleteInEngineThread()
         delete d;
 }
 
-Q_AUTOTEST_EXPORT void qmlExecuteDeferred(QObject *object)
+namespace QtQml {
+
+void qmlExecuteDeferred(QObject *object)
 {
     QQmlData *data = QQmlData::get(object);
 
-    if (data && data->compiledData && data->deferredIdx) {
-        QQmlObjectCreatingProfiler prof;
-        if (prof.enabled) {
-            QQmlType *type = QQmlMetaType::qmlType(object->metaObject());
-            prof.setTypeName(type ? type->qmlTypeName()
-                                  : QString::fromUtf8(object->metaObject()->className()));
-            if (data->outerContext)
-                prof.setLocation(data->outerContext->url, data->lineNumber, data->columnNumber);
-        }
+    if (data && data->deferredData && !data->wasDeleted(object)) {
         QQmlEnginePrivate *ep = QQmlEnginePrivate::get(data->context->engine);
 
         QQmlComponentPrivate::ConstructionState state;
         QQmlComponentPrivate::beginDeferred(ep, object, &state);
 
         // Release the reference for the deferral action (we still have one from construction)
-        data->compiledData->release();
-        data->compiledData = 0;
+        data->deferredData->compiledData->release();
+        delete data->deferredData;
+        data->deferredData = 0;
 
         QQmlComponentPrivate::complete(ep, &state);
     }
@@ -1250,6 +1403,41 @@ QObject *qmlAttachedPropertiesObject(int *idCache, const QObject *object,
     return qmlAttachedPropertiesObjectById(*idCache, object, create);
 }
 
+} // namespace QtQml
+
+#if QT_DEPRECATED_SINCE(5, 1)
+
+// Also define symbols outside namespace to keep binary compatibility with Qt 5.0
+
+Q_QML_EXPORT void qmlExecuteDeferred(QObject *obj)
+{
+    QtQml::qmlExecuteDeferred(obj);
+}
+
+Q_QML_EXPORT QQmlContext *qmlContext(const QObject *obj)
+{
+    return QtQml::qmlContext(obj);
+}
+
+Q_QML_EXPORT QQmlEngine *qmlEngine(const QObject *obj)
+{
+    return QtQml::qmlEngine(obj);
+}
+
+Q_QML_EXPORT QObject *qmlAttachedPropertiesObjectById(int id, const QObject *obj, bool create)
+{
+    return QtQml::qmlAttachedPropertiesObjectById(id, obj, create);
+}
+
+Q_QML_EXPORT QObject *qmlAttachedPropertiesObject(int *idCache, const QObject *object,
+                                                  const QMetaObject *attachedMetaObject,
+                                                  bool create)
+{
+    return QtQml::qmlAttachedPropertiesObject(idCache, object, attachedMetaObject, create);
+}
+
+#endif // QT_DEPRECATED_SINCE(5, 1)
+
 QQmlDebuggingEnabler::QQmlDebuggingEnabler(bool printWarning)
 {
 #ifndef QQML_NO_DEBUG_PROTOCOL
@@ -1403,6 +1591,12 @@ void QQmlData::destroyed(QObject *object)
         compiledData = 0;
     }
 
+    if (deferredData) {
+        deferredData->compiledData->release();
+        delete deferredData;
+        deferredData = 0;
+    }
+
     QQmlAbstractBoundSignal *signalHandler = signalHandlers;
     while (signalHandler) {
         if (signalHandler->isEvaluating()) {
@@ -1462,11 +1656,7 @@ void QQmlData::destroyed(QObject *object)
         delete extendedData;
 
     // Dispose the handle.
-    // We don't simply clear it (and wait for next gc cycle to dispose
-    // via the weak qobject reference callback) as this affects the
-    // outcomes of v8's gc statistical analysis heuristics, which can
-    // cause unnecessary growth of the old pointer space js heap area.
-    qPersistentDispose(v8object);
+    jsWrapper.clear();
 
     if (ownMemory)
         delete this;
@@ -1539,6 +1729,16 @@ void QQmlData::setPendingBindingBit(QObject *obj, int coreIndex)
     QQmlData_setBit(this, obj, coreIndex * 2 + 1);
 }
 
+void QQmlData::ensurePropertyCache(QQmlEngine *engine, QObject *object)
+{
+    Q_ASSERT(engine);
+    QQmlData *ddata = QQmlData::get(object, /*create*/true);
+    if (ddata->propertyCache)
+        return;
+    ddata->propertyCache = QQmlEnginePrivate::get(engine)->cache(object);
+    if (ddata->propertyCache) ddata->propertyCache->addref();
+}
+
 void QQmlEnginePrivate::sendQuit()
 {
     Q_Q(QQmlEngine);
@@ -1579,8 +1779,7 @@ void QQmlEnginePrivate::warning(const QList<QQmlError> &errors)
 
 void QQmlEnginePrivate::warning(QQmlDelayedError *error)
 {
-    Q_Q(QQmlEngine);
-    warning(error->error(q));
+    warning(error->error());
 }
 
 void QQmlEnginePrivate::warning(QQmlEngine *engine, const QQmlError &error)
@@ -1604,7 +1803,7 @@ void QQmlEnginePrivate::warning(QQmlEngine *engine, QQmlDelayedError *error)
     if (engine)
         QQmlEnginePrivate::get(engine)->warning(error);
     else
-        dumpwarning(error->error(0));
+        dumpwarning(error->error());
 }
 
 void QQmlEnginePrivate::warning(QQmlEnginePrivate *engine, const QQmlError &error)
@@ -1650,9 +1849,10 @@ void QQmlEnginePrivate::dereferenceScarceResources()
         // note that the actual SRD is owned by the JS engine,
         // so we cannot delete the SRD; but we can free the
         // memory used by the variant in the SRD.
-        while (ScarceResourceData *sr = scarceResources.first()) {
+        QV4::ExecutionEngine *engine = QV8Engine::getV4(v8engine());
+        while (QV4::ExecutionEngine::ScarceResourceData *sr = engine->scarceResources.first()) {
             sr->data = QVariant();
-            scarceResources.remove(sr);
+            engine->scarceResources.remove(sr);
         }
     }
 }
@@ -1806,6 +2006,16 @@ void QQmlEngine::setOfflineStoragePath(const QString& dir)
 QString QQmlEngine::offlineStoragePath() const
 {
     Q_D(const QQmlEngine);
+
+    if (d->offlineStoragePath.isEmpty()) {
+        QString dataLocation = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
+        QQmlEnginePrivate *e = const_cast<QQmlEnginePrivate *>(d);
+        if (!dataLocation.isEmpty())
+            e->offlineStoragePath = dataLocation.replace(QLatin1Char('/'), QDir::separator())
+                                  + QDir::separator() + QLatin1String("QML")
+                                  + QDir::separator() + QLatin1String("OfflineStorage");
+    }
+
     return d->offlineStoragePath;
 }
 
@@ -2023,7 +2233,7 @@ QQmlPropertyCache *QQmlEnginePrivate::rawPropertyCacheForType(int t)
     }
 }
 
-void QQmlEnginePrivate::registerCompositeType(QQmlCompiledData *data)
+void QQmlEnginePrivate::registerInternalCompositeType(QQmlCompiledData *data)
 {
     QByteArray name = data->rootPropertyCache->className();
 
@@ -2058,7 +2268,7 @@ void QQmlEnginePrivate::registerCompositeType(QQmlCompiledData *data)
     m_compositeTypes.insert(ptr_type, data);
 }
 
-void QQmlEnginePrivate::unregisterCompositeType(QQmlCompiledData *data)
+void QQmlEnginePrivate::unregisterInternalCompositeType(QQmlCompiledData *data)
 {
     int ptr_type = data->metaTypeId;
     int lst_type = data->listMetaTypeId;
@@ -2078,23 +2288,38 @@ bool QQmlEnginePrivate::isScriptLoaded(const QUrl &url) const
     return typeLoader.isScriptLoaded(url);
 }
 
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
+// Normalize a file name using Shell API. As opposed to converting it
+// to a short 8.3 name and back, this also works for drives where 8.3 notation
+// is disabled (see 8dot3name options of fsutil.exe).
+static inline QString shellNormalizeFileName(const QString &name)
+{
+    const QString nativeSeparatorName(QDir::toNativeSeparators(name));
+    const LPCTSTR nameC = reinterpret_cast<LPCTSTR>(nativeSeparatorName.utf16());
+    PIDLIST_ABSOLUTE file;
+    if (FAILED(SHParseDisplayName(nameC, NULL, &file, 0, NULL)))
+        return name;
+    TCHAR buffer[MAX_PATH];
+    if (!SHGetPathFromIDList(file, buffer))
+        return name;
+    QString canonicalName = QString::fromWCharArray(buffer);
+    // Upper case drive letter
+    if (canonicalName.size() > 2 && canonicalName.at(1) == QLatin1Char(':'))
+        canonicalName[0] = canonicalName.at(0).toUpper();
+    return QDir::cleanPath(canonicalName);
+}
+#endif // Q_OS_WIN && !Q_OS_WINCE && !Q_OS_WINRT
+
 bool QQml_isFileCaseCorrect(const QString &fileName, int lengthIn /* = -1 */)
 {
 #if defined(Q_OS_MAC) || defined(Q_OS_WIN)
     QFileInfo info(fileName);
     const QString absolute = info.absoluteFilePath();
 
-#if defined(Q_OS_MAC) || defined(Q_OS_WINCE)
+#if defined(Q_OS_MAC) || defined(Q_OS_WINCE) || defined(Q_OS_WINRT)
     const QString canonical = info.canonicalFilePath();
 #elif defined(Q_OS_WIN)
-    wchar_t buffer[1024];
-
-    DWORD rv = ::GetShortPathName((wchar_t*)absolute.utf16(), buffer, 1024);
-    if (rv == 0 || rv >= 1024) return true;
-    rv = ::GetLongPathName(buffer, buffer, 1024);
-    if (rv == 0 || rv >= 1024) return true;
-
-    const QString canonical = QString::fromWCharArray(buffer);
+    const QString canonical = shellNormalizeFileName(absolute);
 #endif
 
     const int absoluteLength = absolute.length();
