@@ -99,6 +99,7 @@ class QQmlCleanup;
 class QQmlDelayedError;
 class QQuickWorkerScriptEngine;
 class QQmlVME;
+class QmlObjectCreator;
 class QDir;
 class QQmlIncubator;
 
@@ -125,6 +126,9 @@ public:
     ~QQmlEnginePrivate();
 
     void init();
+    // No mutex protecting baseModulesUninitialized, because use outside QQmlEngine
+    // is just qmlClearTypeRegistrations (which can't be called while an engine exists)
+    static bool baseModulesUninitialized;
 
     class PropertyCapture {
     public:
@@ -141,12 +145,10 @@ public:
 
     QQmlContext *rootContext;
     bool isDebugging;
+    bool useNewCompiler;
 
     bool outputWarningsToStdErr;
 
-    QQmlContextData *sharedContext;
-    QObject *sharedScope;
-
     // Registered cleanup handlers
     QQmlCleanup *cleanup;
 
@@ -155,16 +157,21 @@ public:
     int inProgressCreations;
 
     QV8Engine *v8engine() const { return q_func()->handle(); }
+    QV4::ExecutionEngine *v4engine() const { return QV8Engine::getV4(q_func()->handle()); }
 
     QQuickWorkerScriptEngine *getWorkerScriptEngine();
     QQuickWorkerScriptEngine *workerScriptEngine;
 
     QUrl baseUrl;
 
-    typedef QPair<QQmlGuard<QObject>,int> FinalizeCallback;
+    typedef QPair<QPointer<QObject>,int> FinalizeCallback;
     void registerFinalizeCallback(QObject *obj, int index);
 
+    // --- old compiler:
     QQmlVME *activeVME;
+    // --- new compiler:
+    QmlObjectCreator *activeObjectCreator;
+    // ---
 
     QNetworkAccessManager *createNetworkAccessManager(QObject *parent) const;
     QNetworkAccessManager *getNetworkAccessManager() const;
@@ -173,25 +180,14 @@ public:
 
     QHash<QString,QSharedPointer<QQmlImageProviderBase> > imageProviders;
 
-    // Scarce resources are "exceptionally high cost" QVariant types where allowing the
-    // normal JavaScript GC to clean them up is likely to lead to out-of-memory or other
-    // out-of-resource situations.  When such a resource is passed into JavaScript we
-    // add it to the scarceResources list and it is destroyed when we return from the
-    // JavaScript execution that created it.  The user can prevent this behavior by
-    // calling preserve() on the object which removes it from this scarceResource list.
-    class ScarceResourceData {
-    public:
-        ScarceResourceData(const QVariant &data) : data(data) {}
-        QVariant data;
-        QIntrusiveListNode node;
-    };
-    QIntrusiveList<ScarceResourceData, &ScarceResourceData::node> scarceResources;
+    QQmlAbstractUrlInterceptor* urlInterceptor;
+
     int scarceResourcesRefCount;
     void referenceScarceResources();
     void dereferenceScarceResources();
 
-    QQmlTypeLoader typeLoader;
     QQmlImportDatabase importDatabase;
+    QQmlTypeLoader typeLoader;
 
     QString offlineStoragePath;
 
@@ -200,9 +196,9 @@ public:
         return uniqueId++;
     }
 
-    // Unfortunate workaround to avoid a circular dependency between 
+    // Unfortunate workaround to avoid a circular dependency between
     // qqmlengine_p.h and qqmlincubator_p.h
-    struct Incubator {
+    struct Incubator : public QSharedData {
         QIntrusiveListNode next;
         // Unfortunate workaround for MSVC
         QIntrusiveListNode nextWaitingFor;
@@ -235,8 +231,8 @@ public:
     QQmlMetaObject metaObjectForType(int) const;
     QQmlPropertyCache *propertyCacheForType(int);
     QQmlPropertyCache *rawPropertyCacheForType(int);
-    void registerCompositeType(QQmlCompiledData *);
-    void unregisterCompositeType(QQmlCompiledData *);
+    void registerInternalCompositeType(QQmlCompiledData *);
+    void unregisterInternalCompositeType(QQmlCompiledData *);
 
     bool isTypeLoaded(const QUrl &url) const;
     bool isScriptLoaded(const QUrl &url) const;
@@ -255,11 +251,13 @@ public:
     static void warning(QQmlEnginePrivate *, const QList<QQmlError> &);
 
     inline static QV8Engine *getV8Engine(QQmlEngine *e);
+    inline static QV4::ExecutionEngine *getV4Engine(QQmlEngine *e);
     inline static QQmlEnginePrivate *get(QQmlEngine *e);
     inline static const QQmlEnginePrivate *get(const QQmlEngine *e);
     inline static QQmlEnginePrivate *get(QQmlContext *c);
     inline static QQmlEnginePrivate *get(QQmlContextData *c);
     inline static QQmlEngine *get(QQmlEnginePrivate *p);
+    inline static QQmlEnginePrivate *get(QV4::ExecutionEngine *e);
 
     static void registerBaseTypes(const char *uri, int versionMajor, int versionMinor);
     static void registerQtQuick2Types(const char *uri, int versionMajor, int versionMinor);
@@ -481,7 +479,14 @@ QV8Engine *QQmlEnginePrivate::getV8Engine(QQmlEngine *e)
 { 
     Q_ASSERT(e);
 
-    return e->d_func()->v8engine(); 
+    return e->d_func()->v8engine();
+}
+
+QV4::ExecutionEngine *QQmlEnginePrivate::getV4Engine(QQmlEngine *e)
+{
+    Q_ASSERT(e);
+
+    return e->d_func()->v4engine();
 }
 
 QQmlEnginePrivate *QQmlEnginePrivate::get(QQmlEngine *e) 
@@ -512,7 +517,17 @@ QQmlEngine *QQmlEnginePrivate::get(QQmlEnginePrivate *p)
 { 
     Q_ASSERT(p);
 
-    return p->q_func(); 
+    return p->q_func();
+}
+
+QQmlEnginePrivate *QQmlEnginePrivate::get(QV4::ExecutionEngine *e)
+{
+    if (!e->v8Engine)
+        return 0;
+    QQmlEngine *qmlEngine = e->v8Engine->engine();
+    if (!qmlEngine)
+        return 0;
+    return get(qmlEngine);
 }
 
 void QQmlEnginePrivate::captureProperty(QQmlNotifier *n)
