@@ -47,11 +47,8 @@
 #include <QtCore/qurl.h>
 #include <QtCore/qstring.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-
 class QDebug;
 class QQmlErrorPrivate;
 class Q_QML_EXPORT QQmlError
@@ -72,6 +69,8 @@ public:
     void setLine(int);
     int column() const;
     void setColumn(int);
+    QObject *object() const;
+    void setObject(QObject *);
 
     QString toString() const;
 private:
@@ -84,6 +83,4 @@ Q_DECLARE_TYPEINFO(QQmlError, Q_MOVABLE_TYPE);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLERROR_H
