@@ -48,8 +48,6 @@
 #include <QtCore/qobject.h>
 #include <QtCore/qvariant.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -93,14 +91,6 @@ public:
 Q_SIGNALS:
     void valueChanged();
 
-protected:
-    QQmlExpression(QQmlContextData *, QObject *, const QString &,
-                           QQmlExpressionPrivate &dd);
-    QQmlExpression(QQmlContextData *, QObject *, const QString &, bool,
-                           const QString &, int, int, QQmlExpressionPrivate &dd);
-    QQmlExpression(QQmlContextData *, QObject *, const QByteArray &, bool,
-                           const QString &, int, int, QQmlExpressionPrivate &dd);
-
 private:
     QQmlExpression(QQmlContextData *, QObject *, const QString &);
 
@@ -113,7 +103,5 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLEXPRESSION_H
 
