@@ -45,8 +45,6 @@
 #include <QtQml/qtqmlglobal.h>
 #include <QtQml/qqmlerror.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -125,6 +123,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLINCUBATOR_H
