@@ -46,8 +46,6 @@
 #include <QtCore/qlist.h>
 #include <QtCore/qvariant.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -150,6 +148,4 @@ QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QQmlListReference)
 
-QT_END_HEADER
-
 #endif // QQMLLIST_H
