@@ -49,36 +49,34 @@
 #include <private/qv8engine_p.h>
 
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
 class QQmlDateExtension
 {
 public:
-    static void registerExtension(QV8Engine *engine);
+    static void registerExtension(QV4::ExecutionEngine *engine);
 
 private:
-    static v8::Handle<v8::Value> toLocaleString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> toLocaleTimeString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> toLocaleDateString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> fromLocaleString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> fromLocaleTimeString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> fromLocaleDateString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> timeZoneUpdated(const v8::Arguments& args);
+    static QV4::ReturnedValue method_toLocaleString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_toLocaleTimeString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_toLocaleDateString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_fromLocaleString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_fromLocaleTimeString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_fromLocaleDateString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_timeZoneUpdated(QV4::CallContext *ctx);
 };
 
 
 class QQmlNumberExtension
 {
 public:
-    static void registerExtension(QV8Engine *engine);
+    static void registerExtension(QV4::ExecutionEngine *engine);
 
 private:
-    static v8::Handle<v8::Value> toLocaleString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> fromLocaleString(const v8::Arguments& args);
-    static v8::Handle<v8::Value> toLocaleCurrencyString(const v8::Arguments&  args);
+    static QV4::ReturnedValue method_toLocaleString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_fromLocaleString(QV4::CallContext *ctx);
+    static QV4::ReturnedValue method_toLocaleCurrencyString(QV4::CallContext *ctx);
 };
 
 
@@ -120,19 +118,17 @@ public:
         Saturday = Qt::Saturday
     };
 
-    static v8::Handle<v8::Value> locale(QV8Engine *v8engine, const QString &lang);
+    static QV4::ReturnedValue locale(QV8Engine *v8engine, const QString &lang);
 
-    static void registerStringLocaleCompare(QV8Engine *engine);
+    static void registerStringLocaleCompare(QV4::ExecutionEngine *engine);
 
 private:
     QQmlLocale();
 
-    static v8::Handle<v8::Value> localeCompare(const v8::Arguments &args);
+    static QV4::ReturnedValue method_localeCompare(QV4::CallContext *ctx);
 };
 
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
