@@ -50,8 +50,6 @@
 #include <private/qtqmlglobal_p.h>
 #include <private/qobject_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -130,6 +128,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLOPENMETAOBJECT_H
