@@ -57,8 +57,7 @@
 
 #include <QtCore/qglobal.h>
 #include <QtCore/qvariant.h>
-
-QT_BEGIN_HEADER
+#include <QtCore/qurl.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -253,11 +252,29 @@ namespace QQmlPrivate
         // If this is extended ensure "version" is bumped!!!
     };
 
+    struct RegisterCompositeType {
+        QUrl url;
+        const char *uri;
+        int versionMajor;
+        int versionMinor;
+        const char *typeName;
+    };
+
+    struct RegisterCompositeSingletonType {
+        QUrl url;
+        const char *uri;
+        int versionMajor;
+        int versionMinor;
+        const char *typeName;
+    };
+
     enum RegistrationType {
-        TypeRegistration       = 0, 
+        TypeRegistration       = 0,
         InterfaceRegistration  = 1,
         AutoParentRegistration = 2,
-        SingletonRegistration  = 3
+        SingletonRegistration  = 3,
+        CompositeRegistration  = 4,
+        CompositeSingletonRegistration = 5
     };
 
     int Q_QML_EXPORT qmlregister(RegistrationType, void *);
@@ -265,6 +282,4 @@ namespace QQmlPrivate
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLPRIVATE_H
