@@ -61,8 +61,6 @@
 
 #include <private/qobject_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -93,7 +91,5 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLPROXYMETAOBJECT_P_H
 
