@@ -62,14 +62,12 @@
 #include <QtCore/QList>
 #include <QtCore/QUrl>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
 class QByteArray;
 class QQmlPropertyCache;
-namespace QQmlJS { namespace AST { class Node; class StringLiteral; } }
+namespace QQmlJS { class Engine; namespace AST { class Node; class StringLiteral; class UiProgram; class FunctionDeclaration; } }
 namespace QQmlCompilerTypes { struct BindingReference; struct ComponentCompileState; }
 
 namespace QQmlScript {
@@ -122,6 +120,17 @@ public:
     QQmlScript::LocationSpan location;
 };
 
+class Pragma
+{
+public:
+    Pragma() : type(Singleton) {}
+
+    enum Type { Singleton };
+    Type type;
+
+    QQmlScript::LocationSpan location;
+};
+
 class Object;
 class TypeReference : public QQmlPool::Class
 {
@@ -218,9 +227,14 @@ public:
     LocationSpan location;
 
     // Used by compiler
+    struct SignalData {
+        int signalExpressionContextStack;
+        Object *signalScopeObject;
+        int functionIndex; // before gen() index in functionsToCompile, then index in runtime functions
+    };
     union {
         QQmlCompilerTypes::BindingReference *bindingReference;
-        int signalExpressionContextStack;
+        SignalData signalData;
     };
 
     // Used in Property::ValueList lists
@@ -322,6 +336,8 @@ public:
 
     QQmlPropertyCache *metatype;
 
+    QQmlJS::AST::Node *astNode; // responsible for the creation of this object
+
     // The synthesized metaobject, if QML added signals or properties to
     // this type.  Otherwise null
     QByteArray synthdata;  // Generated by compiler
@@ -424,8 +440,8 @@ public:
     {
         DynamicSlot();
 
+        QQmlJS::AST::FunctionDeclaration *funcDecl;
         QHashedStringRef name;
-        QString body;
         QList<QByteArray> parameterNames;
         LocationSpan location;
 
@@ -476,6 +492,7 @@ public:
 
     QQmlScript::Object *tree() const;
     QList<Import> imports() const;
+    QList<Pragma> pragmas() const;
 
     void clear();
 
@@ -501,25 +518,28 @@ public:
     void setScriptFile(const QString &filename) {_scriptFile = filename; }
     QString scriptFile() const { return _scriptFile; }
 
+    QQmlJS::AST::UiProgram *qmlRoot() const { return _qmlRoot; }
+    QQmlJS::Engine *jsEngine() const;
+
 // ### private:
     QList<QQmlError> _errors;
 
     QQmlPool _pool;
     QQmlScript::Object *root;
     QList<Import> _imports;
+    QList<Pragma> _pragmas;
     QList<TypeReference*> _refTypes;
     QString _scriptFile;
+    QQmlJS::AST::UiProgram *_qmlRoot;
     ParserJsASTData *data;
 };
 
 }
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(QQmlScript::Object::ScriptBlock::Pragmas);
+Q_DECLARE_OPERATORS_FOR_FLAGS(QQmlScript::Object::ScriptBlock::Pragmas)
 
 QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QQmlScript::Variant)
 
-QT_END_HEADER
-
 #endif // QQMLSCRIPT_P_H
