@@ -49,8 +49,6 @@
 #include <QtCore/QStringList>
 #include <QtCore/QVariant>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -83,8 +81,8 @@ protected:
     virtual QVariant updateValue(const QString &key, const QVariant &input);
 
     template<class DerivedType>
-    QQmlPropertyMap(DerivedType *derived, QObject *parent)
-        : QObject(*allocatePrivate(), parent)
+    QQmlPropertyMap(DerivedType *derived, QObject *parentObj)
+        : QObject(*allocatePrivate(), parentObj)
     {
         Q_UNUSED(derived)
         init(&DerivedType::staticMetaObject);
@@ -100,6 +98,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
