@@ -83,6 +83,36 @@ bool QuickTestEvent::keyClick(int key, int modifiers, int delay)
     return true;
 }
 
+bool QuickTestEvent::keyPressChar(const QString &character, int modifiers, int delay)
+{
+    QTEST_ASSERT(character.length() == 1);
+    QWindow *window = eventWindow();
+    if (!window)
+        return false;
+    QTest::keyPress(window, character[0].toLatin1(), Qt::KeyboardModifiers(modifiers), delay);
+    return true;
+}
+
+bool QuickTestEvent::keyReleaseChar(const QString &character, int modifiers, int delay)
+{
+    QTEST_ASSERT(character.length() == 1);
+    QWindow *window = eventWindow();
+    if (!window)
+        return false;
+    QTest::keyRelease(window, character[0].toLatin1(), Qt::KeyboardModifiers(modifiers), delay);
+    return true;
+}
+
+bool QuickTestEvent::keyClickChar(const QString &character, int modifiers, int delay)
+{
+    QTEST_ASSERT(character.length() == 1);
+    QWindow *window = eventWindow();
+    if (!window)
+        return false;
+    QTest::keyClick(window, character[0].toLatin1(), Qt::KeyboardModifiers(modifiers), delay);
+    return true;
+}
+
 namespace QTest {
     extern int Q_TESTLIB_EXPORT defaultMouseDelay();
 };
