@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -47,10 +47,12 @@
 #include <QtQml/private/qabstractanimationjob_p.h>
 #include <private/qqmlengine_p.h>
 #include <private/qquickview_p.h>
-#include <private/qquickwindowmanager_p.h>
+#include <private/qsgrenderloop_p.h>
 #include <QtQuick/private/qquickstategroup_p.h>
 #include <QtGui/QImage>
 #include <private/qqmlvme_p.h>
+#include <private/qqmlcomponentattached_p.h>
+#include <private/qqmldata_p.h>
 
 #include "designerwindowmanager_p.h"
 
@@ -375,6 +377,17 @@ void DesignerSupport::resetAnchor(QQuickItem *item, const QString &name)
     }
 }
 
+void DesignerSupport::emitComponentCompleteSignalForAttachedProperty(QQuickItem *item)
+{
+    QQmlData *data = QQmlData::get(item);
+    if (data && data->context) {
+        QQmlComponentAttached *componentAttached = data->context->componentAttached;
+        if (componentAttached) {
+            emit componentAttached->completed();
+        }
+    }
+}
+
 QList<QObject*> DesignerSupport::statesForItem(QQuickItem *item)
 {
     QList<QObject*> objectList;
@@ -429,7 +442,7 @@ void DesignerSupport::updateDirtyNode(QQuickItem *item)
 
 void DesignerSupport::activateDesignerWindowManager()
 {
-    QQuickWindowManager::setInstance(new DesignerWindowManager);
+    QSGRenderLoop::setInstance(new DesignerWindowManager);
 }
 
 void DesignerSupport::activateDesignerMode()
@@ -437,6 +450,16 @@ void DesignerSupport::activateDesignerMode()
     QQmlEnginePrivate::activateDesignerMode();
 }
 
+void DesignerSupport::disableComponentComplete()
+{
+    QQmlVME::disableComponentComplete();
+}
+
+void DesignerSupport::enableComponentComplete()
+{
+    QQmlVME::enableComponentComplete();
+}
+
 void DesignerSupport::createOpenGLContext(QQuickWindow *window)
 {
     DesignerWindowManager::createOpenGLContext(window);
