@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -40,7 +40,7 @@
 ****************************************************************************/
 
 #include "designerwindowmanager_p.h"
-
+#include "private/qquickwindow_p.h"
 #include <QtGui/QOpenGLContext>
 
 #include <QtQuick/QQuickWindow>
@@ -51,6 +51,7 @@ QT_BEGIN_NAMESPACE
 DesignerWindowManager::DesignerWindowManager()
     : m_sgContext(QSGContext::createDefaultContext())
 {
+    m_renderContext.reset(new QSGRenderContext(m_sgContext.data()));
 }
 
 void DesignerWindowManager::show(QQuickWindow *window)
@@ -74,7 +75,7 @@ void DesignerWindowManager::makeOpenGLContext(QQuickWindow *window)
         m_openGlContext->create();
         if (!m_openGlContext->makeCurrent(window))
             qWarning("QQuickWindow: makeCurrent() failed...");
-        m_sgContext->initialize(m_openGlContext.data());
+        m_renderContext->initialize(m_openGlContext.data());
     } else {
         m_openGlContext->makeCurrent(window);
     }
@@ -89,10 +90,6 @@ QImage DesignerWindowManager::grab(QQuickWindow *)
     return QImage();
 }
 
-void DesignerWindowManager::resize(QQuickWindow *, const QSize &)
-{
-}
-
 void DesignerWindowManager::maybeUpdate(QQuickWindow *)
 {
 }
