@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,28 +44,31 @@
 #include <private/qquickitem_p.h>
 #include <private/qquickcanvascontext_p.h>
 #include <private/qquickcontext2d_p.h>
+#include <private/qquickcontext2dtexture_p.h>
 #include <qsgsimpletexturenode.h>
 #include <QtQuick/private/qquickpixmapcache_p.h>
+#include <QtGui/QGuiApplication>
 
 #include <qqmlinfo.h>
 #include <private/qqmlengine_p.h>
 #include <QtCore/QBuffer>
+#include <QtCore/qdatetime.h>
+
+#include <private/qv4value_p.h>
+#include <private/qv4functionobject_p.h>
+#include <private/qv4scopedvalue_p.h>
 
 QT_BEGIN_NAMESPACE
 
-QQuickCanvasPixmap::QQuickCanvasPixmap(const QImage& image, QQuickWindow *window)
+QQuickCanvasPixmap::QQuickCanvasPixmap(const QImage& image)
     : m_pixmap(0)
     , m_image(image)
-    , m_texture(0)
-    , m_window(window)
 {
 
 }
 
-QQuickCanvasPixmap::QQuickCanvasPixmap(QQuickPixmap *pixmap, QQuickWindow *window)
+QQuickCanvasPixmap::QQuickCanvasPixmap(QQuickPixmap *pixmap)
     : m_pixmap(pixmap)
-    , m_texture(0)
-    , m_window(window)
 {
 
 }
@@ -73,8 +76,6 @@ QQuickCanvasPixmap::QQuickCanvasPixmap(QQuickPixmap *pixmap, QQuickWindow *windo
 QQuickCanvasPixmap::~QQuickCanvasPixmap()
 {
     delete m_pixmap;
-    if (m_texture)
-        m_texture->deleteLater();
 }
 
 qreal QQuickCanvasPixmap::width() const
@@ -100,18 +101,6 @@ bool QQuickCanvasPixmap::isValid() const
     return !m_image.isNull();
 }
 
-QSGTexture *QQuickCanvasPixmap::texture()
-{
-    if (!m_texture) {
-        if (m_pixmap) {
-            Q_ASSERT(m_pixmap->textureFactory());
-            m_texture = m_pixmap->textureFactory()->createTexture(m_window);
-        } else {
-            m_texture = QQuickWindowPrivate::get(m_window)->context->createTexture(m_image);
-        }
-    }
-    return m_texture;
-}
 QImage QQuickCanvasPixmap::image()
 {
     if (m_image.isNull() && m_pixmap)
@@ -171,13 +160,12 @@ public:
     uint hasTileSize :1;
     uint hasCanvasWindow :1;
     uint available :1;
-    uint contextInitialized :1;
     QQuickCanvasItem::RenderTarget renderTarget;
     QQuickCanvasItem::RenderStrategy renderStrategy;
     QString contextType;
     QHash<QUrl, QQmlRefPointer<QQuickCanvasPixmap> > pixmaps;
     QUrl baseUrl;
-    QMap<int, v8::Persistent<v8::Function> > animationCallbacks;
+    QMap<int, QV4::PersistentValue> animationCallbacks;
 };
 
 QQuickCanvasItemPrivate::QQuickCanvasItemPrivate()
@@ -189,10 +177,10 @@ QQuickCanvasItemPrivate::QQuickCanvasItemPrivate()
     , hasTileSize(false)
     , hasCanvasWindow(false)
     , available(false)
-    , contextInitialized(false)
-    , renderTarget(QQuickCanvasItem::FramebufferObject)
-    , renderStrategy(QQuickCanvasItem::Cooperative)
+    , renderTarget(QQuickCanvasItem::Image)
+    , renderStrategy(QQuickCanvasItem::Immediate)
 {
+    antialiasing = true;
 }
 
 QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
@@ -204,8 +192,8 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
 /*!
     \qmltype Canvas
     \instantiates QQuickCanvasItem
-    \inqmlmodule QtQuick 2
-    \since QtQuick 2.0
+    \inqmlmodule QtQuick
+    \since 5.0
     \inherits Item
     \ingroup qtquick-canvas
     \ingroup qtquick-visual
@@ -242,10 +230,14 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
 
     The Canvas.FramebufferObject render target utilizes OpenGL hardware
     acceleration rather than rendering into system memory, which in many cases
-    results in faster rendering.
+    results in faster rendering. Canvas.FramebufferObject relies on the
+    OpenGL extensions \c GL_EXT_framebuffer_multisample and
+    \c GL_EXT_framebuffer_blit for antialiasing. It will also use more
+    graphics memory when rendering strategy is anything other than
+    Canvas.Cooperative.
 
     The default render target is Canvas.Image and the default renderStrategy is
-    Canvas.Threaded.
+    Canvas.Immediate.
 
     \section1 Tiled Canvas
     The Canvas item supports tiled rendering by setting \l canvasSize, \l tileSize
@@ -267,7 +259,7 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
     requires the pixel data to be exchanged between the system memory and the
     graphic card, which is significantly more expensive.  Rendering may also be
     synchronized with the V-sync signal (to avoid
-    \l{en.wikipedia.org/wiki/Screen_tearing}{screen tearing}) which will further
+    \l{http://en.wikipedia.org/wiki/Screen_tearing}{screen tearing}) which will further
     impact pixel operations with \c Canvas.FrambufferObject render target.
 
     \section1 Tips for Porting Existing HTML5 Canvas applications
@@ -301,7 +293,7 @@ QQuickCanvasItem::~QQuickCanvasItem()
 }
 
 /*!
-    \qmlproperty size QtQuick2::Canvas::available
+    \qmlproperty size QtQuick::Canvas::available
 
     Indicates when Canvas is able to provide a drawing context to operate on.
 */
@@ -312,7 +304,7 @@ bool QQuickCanvasItem::isAvailable() const
 }
 
 /*!
-    \qmlproperty string QtQuick2::Canvas::contextType
+    \qmlproperty string QtQuick::Canvas::contextType
     The type of drawing context to use.
 
     This property is set to the name of the active context type.
@@ -338,7 +330,7 @@ void QQuickCanvasItem::setContextType(const QString &contextType)
     if (contextType.compare(d->contextType, Qt::CaseInsensitive) == 0)
         return;
 
-    if (d->contextInitialized) {
+    if (d->context) {
         qmlInfo(this) << "Canvas already initialized with a different context type";
         return;
     }
@@ -352,7 +344,7 @@ void QQuickCanvasItem::setContextType(const QString &contextType)
 }
 
 /*!
-    \qmlproperty object QtQuick2::Canvas::context
+    \qmlproperty object QtQuick::Canvas::context
     Holds the active drawing context.
 
     If the canvas is ready and there has been a successful call to getContext()
@@ -360,17 +352,17 @@ void QQuickCanvasItem::setContextType(const QString &contextType)
     this property will contain the current drawing context, otherwise null.
 */
 
-QQmlV8Handle QQuickCanvasItem::context() const
+QQmlV4Handle QQuickCanvasItem::context() const
 {
     Q_D(const QQuickCanvasItem);
-    if (d->contextInitialized)
-        return QQmlV8Handle::fromHandle(d->context->v8value());
+    if (d->context)
+        return QQmlV4Handle(d->context->v4value());
 
-    return QQmlV8Handle::fromHandle(v8::Null());
+    return QQmlV4Handle(QV4::Encode::null());
 }
 
 /*!
-    \qmlproperty size QtQuick2::Canvas::canvasSize
+    \qmlproperty size QtQuick::Canvas::canvasSize
     Holds the logical canvas size that the context paints on.
 
     By default, the canvas size is the same size as the current canvas item
@@ -397,13 +389,13 @@ void QQuickCanvasItem::setCanvasSize(const QSizeF & size)
         d->canvasSize = size;
         emit canvasSizeChanged();
 
-        if (d->contextInitialized)
+        if (d->context)
             polish();
     }
 }
 
 /*!
-    \qmlproperty size QtQuick2::Canvas::tileSize
+    \qmlproperty size QtQuick::Canvas::tileSize
     Holds the canvas rendering tile size.
 
     The Canvas item enters tiled mode by setting canvasSize, tileSize and the
@@ -432,13 +424,13 @@ void QQuickCanvasItem::setTileSize(const QSize & size)
 
         emit tileSizeChanged();
 
-        if (d->contextInitialized)
+        if (d->context)
             polish();
     }
 }
 
 /*!
-    \qmlproperty rect QtQuick2::Canvas::canvasWindow
+    \qmlproperty rect QtQuick::Canvas::canvasWindow
      Holds the current canvas visible window.
 
      By default the canvasWindow size is the same as the Canvas item size with
@@ -465,13 +457,13 @@ void QQuickCanvasItem::setCanvasWindow(const QRectF& rect)
         d->hasCanvasWindow = true;
         emit canvasWindowChanged();
 
-        if (d->contextInitialized)
+        if (d->context)
             polish();
     }
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Canvas::renderTarget
+    \qmlproperty enumeration QtQuick::Canvas::renderTarget
     Holds the current canvas render target.
 
     \list
@@ -485,7 +477,7 @@ void QQuickCanvasItem::setCanvasWindow(const QRectF& rect)
     context will choose appropriate options and Canvas will signal the change
     to the properties.
 
-    The default render target is \c Canvas.FramebufferObject.
+    The default render target is \c Canvas.Image.
 */
 QQuickCanvasItem::RenderTarget QQuickCanvasItem::renderTarget() const
 {
@@ -497,7 +489,7 @@ void QQuickCanvasItem::setRenderTarget(QQuickCanvasItem::RenderTarget target)
 {
     Q_D(QQuickCanvasItem);
     if (d->renderTarget != target) {
-        if (d->contextInitialized) {
+        if (d->context) {
             qmlInfo(this) << "Canvas:renderTarget not changeble once context is active.";
             return;
         }
@@ -508,7 +500,7 @@ void QQuickCanvasItem::setRenderTarget(QQuickCanvasItem::RenderTarget target)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Canvas::renderStrategy
+    \qmlproperty enumeration QtQuick::Canvas::renderStrategy
     Holds the current canvas rendering strategy.
 
     \list
@@ -527,7 +519,7 @@ void QQuickCanvasItem::setRenderTarget(QQuickCanvasItem::RenderTarget target)
     the GUI thread.  Selecting \c Canvas.Cooperative, does not guarantee
     rendering will occur on a thread separate from the GUI thread.
 
-    The default value is \c Canvas.Cooperative.
+    The default value is \c Canvas.Immediate.
 
     \sa renderTarget
 */
@@ -541,7 +533,7 @@ void QQuickCanvasItem::setRenderStrategy(QQuickCanvasItem::RenderStrategy strate
 {
     Q_D(QQuickCanvasItem);
     if (d->renderStrategy != strategy) {
-        if (d->contextInitialized) {
+        if (d->context) {
             qmlInfo(this) << "Canvas:renderStrategy not changeable once context is active.";
             return;
         }
@@ -598,7 +590,7 @@ void QQuickCanvasItem::geometryChanged(const QRectF &newGeometry, const QRectF &
         emit canvasWindowChanged();
     }
 
-    if (d->available)
+    if (d->available && newSize != oldGeometry.size())
         requestPaint();
 }
 
@@ -627,8 +619,11 @@ void QQuickCanvasItem::itemChange(QQuickItem::ItemChange change, const QQuickIte
         return;
 
     Q_D(QQuickCanvasItem);
-    if (d->available)
+    if (d->available) {
+        if (d->dirtyAttributes & QQuickItemPrivate::ContentUpdateMask)
+            requestPaint();
         return;
+    }
 
     if (value.window== 0)
         return;
@@ -646,20 +641,22 @@ void QQuickCanvasItem::updatePolish()
 
     Q_D(QQuickCanvasItem);
 
-    if (d->contextInitialized)
+    if (d->context && d->renderStrategy != QQuickCanvasItem::Cooperative)
         d->context->prepare(d->canvasSize.toSize(), d->tileSize, d->canvasWindow.toRect(), d->dirtyRect.toRect(), d->smooth, d->antialiasing);
 
     if (d->animationCallbacks.size() > 0 && isVisible()) {
-        QMap<int, v8::Persistent<v8::Function> > animationCallbacks = d->animationCallbacks;
+        QMap<int, QV4::PersistentValue> animationCallbacks = d->animationCallbacks;
         d->animationCallbacks.clear();
 
+        QV4::ExecutionEngine *v4 = QQmlEnginePrivate::getV4Engine(qmlEngine(this));
+        QV4::Scope scope(v4);
+        QV4::ScopedCallData callData(scope, 1);
+        callData->thisObject = QV4::QObjectWrapper::wrap(v4, this);
+
         foreach (int key, animationCallbacks.keys()) {
-            v8::HandleScope handle_scope;
-            v8::Handle<v8::Object> self = QQmlEnginePrivate::getV8Engine(qmlEngine(this))->newQObject(this).As<v8::Object>();
-            v8::Handle<v8::Value> args[] = { v8::Uint32::New(QDateTime::currentDateTimeUtc().toTime_t()) };
-            v8::Persistent<v8::Function> f = animationCallbacks.value(key);
-            f->Call(self, 1, args);
-            f.Dispose();
+            QV4::ScopedFunctionObject f(scope, animationCallbacks.value(key).value());
+            callData->args[0] = QV4::Primitive::fromUInt32(QDateTime::currentDateTimeUtc().toTime_t());
+            f->call(callData);
         }
     }
     else {
@@ -672,7 +669,7 @@ void QQuickCanvasItem::updatePolish()
         }
     }
 
-    if (d->contextInitialized) {
+    if (d->context) {
         if (d->renderStrategy == QQuickCanvasItem::Cooperative)
             update();
         else
@@ -680,32 +677,53 @@ void QQuickCanvasItem::updatePolish()
     }
 }
 
+class QQuickCanvasNode : public QSGSimpleTextureNode
+{
+public:
+    ~QQuickCanvasNode()
+    {
+        delete texture();
+    }
+};
+
 QSGNode *QQuickCanvasItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *)
 {
     Q_D(QQuickCanvasItem);
 
-    if (!d->contextInitialized)
+    if (!d->context || d->canvasWindow.size().isEmpty()) {
+        delete oldNode;
         return 0;
+    }
 
-    QSGSimpleTextureNode *node = static_cast<QSGSimpleTextureNode*>(oldNode);
+    QQuickCanvasNode *node = static_cast<QQuickCanvasNode*>(oldNode);
     if (!node)
-        node = new QSGSimpleTextureNode;
+        node = new QQuickCanvasNode();
 
     if (d->smooth)
         node->setFiltering(QSGTexture::Linear);
     else
         node->setFiltering(QSGTexture::Nearest);
 
-    if (d->renderStrategy == QQuickCanvasItem::Cooperative)
+    if (d->renderStrategy == QQuickCanvasItem::Cooperative) {
+        d->context->prepare(d->canvasSize.toSize(), d->tileSize, d->canvasWindow.toRect(), d->dirtyRect.toRect(), d->smooth, d->antialiasing);
         d->context->flush();
+    }
 
-    node->setTexture(d->context->texture());
+    QQuickContext2D *ctx = qobject_cast<QQuickContext2D *>(d->context);
+    QQuickContext2DTexture *factory = ctx->texture();
+    QSGTexture *texture = factory->textureForNextFrame(node->texture());
+    if (!texture) {
+        delete node;
+        return 0;
+    }
+
+    node->setTexture(texture);
     node->setRect(QRectF(QPoint(0, 0), d->canvasWindow.size()));
     return node;
 }
 
 /*!
-    \qmlmethod object QtQuick2::Canvas::getContext(string contextId, any... args)
+    \qmlmethod object QtQuick::Canvas::getContext(string contextId, any... args)
 
     Returns a drawing context or null if no context available.
 
@@ -721,53 +739,57 @@ QSGNode *QQuickCanvasItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData
     Canvas only supports a 2d context.
 */
 
-void QQuickCanvasItem::getContext(QQmlV8Function *args)
+void QQuickCanvasItem::getContext(QQmlV4Function *args)
 {
     Q_D(QQuickCanvasItem);
 
-    if (args->Length() < 1 || !(*args)[0]->IsString()) {
+    QV4::Scope scope(args->v4engine());
+    QV4::ScopedString str(scope, (*args)[0]);
+    if (!str) {
         qmlInfo(this) << "getContext should be called with a string naming the required context type";
-        args->returnValue(v8::Null());
+        args->setReturnValue(QV4::Encode::null());
         return;
     }
 
     if (!d->available) {
         qmlInfo(this) << "Unable to use getContext() at this time, please wait for available: true";
-        args->returnValue(v8::Null());
+        args->setReturnValue(QV4::Encode::null());
         return;
     }
 
-    QString contextId = QString::fromUtf16(*v8::String::Value((*args)[0]));
+    QString contextId = str->toQString();
 
     if (d->context != 0) {
         if (d->context->contextNames().contains(contextId, Qt::CaseInsensitive)) {
-            args->returnValue(d->context->v8value());
+            args->setReturnValue(d->context->v4value());
             return;
         }
 
         qmlInfo(this) << "Canvas already initialized with a different context type";
-        args->returnValue(v8::Null());
+        args->setReturnValue(QV4::Encode::null());
         return;
     }
 
     if (createContext(contextId))
-        args->returnValue(d->context->v8value());
+        args->setReturnValue(d->context->v4value());
     else
-        args->returnValue(v8::Null());
+        args->setReturnValue(QV4::Encode::null());
 }
 
 /*!
-    \qmlmethod long QtQuick2::Canvas::requestAnimationFrame(callback)
+    \qmlmethod long QtQuick::Canvas::requestAnimationFrame(callback)
 
-    This function schedules callback to be invoked before composing the QtQuick
+    This function schedules callback to be invoked before composing the Qt Quick
     scene.
 */
 
-void QQuickCanvasItem::requestAnimationFrame(QQmlV8Function *args)
+void QQuickCanvasItem::requestAnimationFrame(QQmlV4Function *args)
 {
-    if (args->Length() < 1 || !(*args)[0]->IsFunction()) {
+    QV4::Scope scope(args->v4engine());
+    QV4::ScopedFunctionObject f(scope, (*args)[0]);
+    if (!f) {
         qmlInfo(this) << "requestAnimationFrame should be called with an animation callback function";
-        args->returnValue(v8::Null());
+        args->setReturnValue(QV4::Encode::null());
         return;
     }
 
@@ -775,34 +797,36 @@ void QQuickCanvasItem::requestAnimationFrame(QQmlV8Function *args)
 
     static int id = 0;
 
-    d->animationCallbacks.insert(++id, v8::Persistent<v8::Function>::New(((*args)[0]).As<v8::Function>()));
+    d->animationCallbacks.insert(++id, QV4::PersistentValue(f));
 
     if (isVisible())
         polish();
 
-    args->returnValue(v8::Int32::New(id));
+    args->setReturnValue(QV4::Encode(id));
 }
 
 /*!
-    \qmlmethod QtQuick2::Canvas::cancelRequestAnimationFrame(long handle)
+    \qmlmethod QtQuick::Canvas::cancelRequestAnimationFrame(long handle)
 
     This function will cancel the animation callback referenced by \a handle.
 */
 
-void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV8Function *args)
+void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV4Function *args)
 {
-    if (args->Length() < 1 || !(*args)[0]->IsInt32()) {
+    QV4::Scope scope(args->v4engine());
+    QV4::ScopedValue v(scope, (*args)[0]);
+    if (!v->isInteger()) {
         qmlInfo(this) << "cancelRequestAnimationFrame should be called with an animation callback id";
-        args->returnValue(v8::Null());
+        args->setReturnValue(QV4::Encode::null());
         return;
     }
 
-    d_func()->animationCallbacks.remove((*args)[0]->Int32Value());
+    d_func()->animationCallbacks.remove(v->integerValue());
 }
 
 
 /*!
-    \qmlmethod QtQuick2::Canvas::requestPaint()
+    \qmlmethod QtQuick::Canvas::requestPaint()
 
     Request the entire visible region be re-drawn.
 
@@ -815,13 +839,13 @@ void QQuickCanvasItem::requestPaint()
 }
 
 /*!
-    \qmlmethod QtQuick2::Canvas::markDirty(rect area)
+    \qmlmethod QtQuick::Canvas::markDirty(rect area)
 
     Mark the given \a area as dirty, so that when this area is visible the
     canvas renderer will redraw it. This will trigger the onPaint signal
     handler function.
 
-    \sa paint, requestPaint()
+    \sa onPaint, requestPaint()
 */
 
 void QQuickCanvasItem::markDirty(const QRectF& rect)
@@ -842,7 +866,7 @@ void QQuickCanvasItem::checkAnimationCallbacks()
 }
 
 /*!
-  \qmlmethod bool QtQuick2::Canvas::save(string filename)
+  \qmlmethod bool QtQuick::Canvas::save(string filename)
 
    Save the current canvas content into an image file \a filename.
    The saved image format is automatically decided by the \a filename's
@@ -871,7 +895,7 @@ QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& ur
 }
 
 /*!
-    \qmlsignal QtQuick2::Canvas::onImageLoaded()
+    \qmlsignal QtQuick::Canvas::onImageLoaded()
 
     This handler is called when an image has been loaded.
 
@@ -879,7 +903,7 @@ QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& ur
 */
 
 /*!
-  \qmlmethod QtQuick2::Canvas::loadImage(url image)
+  \qmlmethod QtQuick::Canvas::loadImage(url image)
     Loads the given \c image asynchronously.
 
     When the image is ready, onImageLoaded will be emitted.
@@ -896,7 +920,7 @@ void QQuickCanvasItem::loadImage(const QUrl& url)
     if (!d->pixmaps.contains(fullPathUrl)) {
         QQuickPixmap* pix = new QQuickPixmap();
         QQmlRefPointer<QQuickCanvasPixmap> canvasPix;
-        canvasPix.take(new QQuickCanvasPixmap(pix, d->window));
+        canvasPix.take(new QQuickCanvasPixmap(pix));
         d->pixmaps.insert(fullPathUrl, canvasPix);
 
         pix->load(qmlEngine(this)
@@ -907,7 +931,7 @@ void QQuickCanvasItem::loadImage(const QUrl& url)
     }
 }
 /*!
-  \qmlmethod QtQuick2::Canvas::unloadImage(url image)
+  \qmlmethod QtQuick::Canvas::unloadImage(url image)
   Unloads the \c image.
 
   Once an image is unloaded it cannot be painted by the canvas context
@@ -923,7 +947,7 @@ void QQuickCanvasItem::unloadImage(const QUrl& url)
 }
 
 /*!
-  \qmlmethod QtQuick2::Canvas::isImageError(url image)
+  \qmlmethod QtQuick::Canvas::isImageError(url image)
   Returns true if the \a image failed to load.
 
   \sa loadImage()
@@ -937,7 +961,7 @@ bool QQuickCanvasItem::isImageError(const QUrl& url) const
 }
 
 /*!
-  \qmlmethod QtQuick2::Canvas::isImageLoading(url image)
+  \qmlmethod QtQuick::Canvas::isImageLoading(url image)
   Returns true if the \a image is currently loading.
 
   \sa loadImage()
@@ -950,7 +974,7 @@ bool QQuickCanvasItem::isImageLoading(const QUrl& url) const
         && d->pixmaps.value(fullPathUrl)->pixmap()->isLoading();
 }
 /*!
-  \qmlmethod QtQuick2::Canvas::isImageLoaded(url image)
+  \qmlmethod QtQuick::Canvas::isImageLoaded(url image)
   Returns true if the \a image is successfully loaded and ready to use.
 
   \sa loadImage()
@@ -966,7 +990,7 @@ bool QQuickCanvasItem::isImageLoaded(const QUrl& url) const
 QImage QQuickCanvasItem::toImage(const QRectF& rect) const
 {
     Q_D(const QQuickCanvasItem);
-    if (d->contextInitialized) {
+    if (d->context) {
         if (rect.isEmpty())
             return d->context->toImage(canvasWindow());
         else
@@ -977,7 +1001,7 @@ QImage QQuickCanvasItem::toImage(const QRectF& rect) const
 }
 
 /*!
-  \qmlmethod string QtQuick2::Canvas::toDataURL(string mimeType)
+  \qmlmethod string QtQuick::Canvas::toDataURL(string mimeType)
 
    Returns a data URL for the image in the canvas.
 
@@ -1022,7 +1046,7 @@ void QQuickCanvasItem::delayedCreate()
 {
     Q_D(QQuickCanvasItem);
 
-    if (!d->contextInitialized && !d->contextType.isNull())
+    if (!d->context && !d->contextType.isNull())
         createContext(d->contextType);
 
     requestPaint();
@@ -1051,7 +1075,6 @@ void QQuickCanvasItem::initializeContext(QQuickCanvasContext *context, const QVa
     d->context = context;
     d->context->init(this, args);
     d->context->setV8Engine(QQmlEnginePrivate::getV8Engine(qmlEngine(this)));
-    d->contextInitialized = true;
     connect(d->context, SIGNAL(textureChanged()), SLOT(update()));
     connect(d->context, SIGNAL(textureChanged()), SIGNAL(painted()));
     emit contextChanged();
@@ -1074,7 +1097,7 @@ QRect QQuickCanvasItem::tiledRect(const QRectF &window, const QSize &tileSize)
 }
 
 /*!
-    \qmlsignal QtQuick2::Canvas::onPaint(rect region)
+    \qmlsignal QtQuick::Canvas::onPaint(rect region)
 
     This handler is called to render the \a region. If a context is active it
     can be referenced from the context property.
@@ -1084,7 +1107,7 @@ QRect QQuickCanvasItem::tiledRect(const QRectF &window, const QSize &tileSize)
 */
 
 /*!
-    \qmlsignal QtQuick2::Canvas::onPainted()
+    \qmlsignal QtQuick::Canvas::onPainted()
 
     This handler is called after all context painting commands are executed and
     the Canvas has been rendered.
