@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -55,6 +55,7 @@
 #include <private/qv8engine_p.h>
 #include <QtCore/QWaitCondition>
 
+#include <private/qv4value_p.h>
 
 //#define QQUICKCONTEXT2D_DEBUG //enable this for just DEBUG purpose!
 
@@ -62,20 +63,19 @@
 #include <QElapsedTimer>
 #endif
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickContext2DCommandBuffer;
 class QQuickContext2DTexture;
 class QQuickPixmap;
 class QSGTexture;
-class QQuickWindowManager;
 class QSurface;
 class QOpenGLContext;
 
 class QQuickContext2D : public QQuickCanvasContext
 {
+    Q_OBJECT
+
 public:
     Q_DISABLE_COPY(QQuickContext2D)
 
@@ -133,10 +133,11 @@ public:
             , shadowBlur(0)
             , shadowColor(qRgba(0, 0, 0, 0))
             , globalCompositeOperation(QPainter::CompositionMode_SourceOver)
-            , font(QFont(QLatin1String("sans-serif"), 10))
+            , font(QFont(QLatin1String("sans-serif")))
             , textAlign(QQuickContext2D::Start)
             , textBaseline(QQuickContext2D::Alphabetic)
         {
+            font.setPixelSize(10);
         }
 
         QTransform matrix;
@@ -172,11 +173,11 @@ public:
     void prepare(const QSize& canvasSize, const QSize& tileSize, const QRect& canvasWindow, const QRect& dirtyRect, bool smooth, bool antialiasing);
     void flush();
     void sync();
-    QThread *thread() const {return m_thread;}
-    QSGDynamicTexture *texture() const;
+    QThread *thread() const { return m_thread; }
+    QQuickContext2DTexture *texture() const;
     QImage toImage(const QRectF& bounds);
 
-    v8::Handle<v8::Object> v8value() const;
+    QV4::ReturnedValue v4value() const;
     void setV8Engine(QV8Engine *eng);
 
     QQuickCanvasItem* canvas() const { return m_canvas; }
@@ -236,14 +237,13 @@ public:
     QQuickCanvasItem* m_canvas;
     QQuickContext2DCommandBuffer* m_buffer;
     QPainterPath m_path;
-    v8::Local<v8::Value> m_fillStyle;
-    v8::Local<v8::Value> m_strokeStyle;
-    v8::Handle<v8::Value> m_v8path;
+    QV4::PersistentValue m_fillStyle;
+    QV4::PersistentValue m_strokeStyle;
+    QV4::PersistentValue m_v4path;
     QV8Engine *m_v8engine;
-    QQuickWindowManager *m_windowManager;
     QSurface *m_surface;
     QOpenGLContext *m_glContext;
-    v8::Persistent<v8::Object> m_v8value;
+    QV4::PersistentValue m_v4value;
     QQuickContext2DTexture *m_texture;
     QQuickCanvasItem::RenderTarget m_renderTarget;
     QQuickCanvasItem::RenderStrategy m_renderStrategy;
@@ -259,6 +259,4 @@ public:
 QT_END_NAMESPACE
 QML_DECLARE_TYPE(QQuickContext2D)
 
-QT_END_HEADER
-
 #endif // QQUICKCONTEXT2D_P_H
