@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,8 +52,6 @@
 #include <QtGui/qaccessible.h>
 #include <private/qtquickglobal_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -75,7 +73,7 @@ public:
     {
         if (role != m_role) {
             m_role = role;
-            emit roleChanged();
+            Q_EMIT roleChanged();
             // There is no way to signify role changes at the moment.
             // QAccessible::updateAccessibility(parent(), 0, QAccessible::);
         }
@@ -84,7 +82,7 @@ public:
     void setName(const QString &name) {
         if (name != m_name) {
             m_name = name;
-            emit nameChanged();
+            Q_EMIT nameChanged();
             QAccessibleEvent ev(parent(), QAccessible::NameChanged);
             QAccessible::updateAccessibility(&ev);
         }
@@ -95,7 +93,7 @@ public:
     {
         if (m_description != description) {
             m_description = description;
-            emit descriptionChanged();
+            Q_EMIT descriptionChanged();
             QAccessibleEvent ev(parent(), QAccessible::DescriptionChanged);
             QAccessible::updateAccessibility(&ev);
         }
@@ -144,6 +142,10 @@ public Q_SLOTS:
         QAccessibleValueChangeEvent ev(parent(), parent()->property("value"));
         QAccessible::updateAccessibility(&ev);
     }
+    void cursorPositionChanged() {
+        QAccessibleTextCursorEvent ev(parent(), parent()->property("cursorPosition").toInt());
+        QAccessible::updateAccessibility(&ev);
+    }
 
 Q_SIGNALS:
     void roleChanged();
@@ -165,8 +167,6 @@ QT_END_NAMESPACE
 QML_DECLARE_TYPE(QQuickAccessibleAttached)
 QML_DECLARE_TYPEINFO(QQuickAccessibleAttached, QML_HAS_ATTACHED_PROPERTIES)
 
-QT_END_HEADER
-
 #endif // QT_NO_ACCESSIBILITY
 
 #endif
