@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,7 +44,6 @@
 
 #include "qquickimagebase_p.h"
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 class QQuickScaleGrid;
@@ -102,6 +101,5 @@ private:
 
 QT_END_NAMESPACE
 QML_DECLARE_TYPE(QQuickBorderImage)
-QT_END_HEADER
 
 #endif // QQUICKBORDERIMAGE_P_H
