@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -45,8 +45,6 @@
 #include "qquickitem.h"
 #include <private/qtquickglobal_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickFlickablePrivate;
@@ -278,6 +276,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickFlickable)
 
-QT_END_HEADER
-
 #endif // QQUICKFLICKABLE_P_H
