@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -48,8 +48,6 @@
 #include <QtGui/qvector3d.h>
 #include <QtCore/qobject.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickFlipablePrivate;
@@ -96,6 +94,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickFlipable)
 
-QT_END_HEADER
-
 #endif // QQUICKFLIPABLE_P_H
