@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -45,8 +45,6 @@
 #include "qquickimagebase_p.h"
 #include <QtQuick/qsgtextureprovider.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickImagePrivate;
@@ -114,6 +112,4 @@ private:
 
 QT_END_NAMESPACE
 QML_DECLARE_TYPE(QQuickImage)
-QT_END_HEADER
-
 #endif // QQUICKIMAGE_P_H
