@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,8 +52,6 @@
 #include <QtGui/qfont.h>
 #include <QtGui/qaccessible.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickItem;
@@ -82,7 +80,7 @@ private:
 
 class QCursor;
 class QQuickItemLayer;
-class QQmlV8Function;
+class QQmlV4Function;
 class QQuickState;
 class QQuickAnchorLine;
 class QQuickTransition;
@@ -134,6 +132,7 @@ class Q_QUICK_EXPORT QQuickItem : public QObject, public QQmlParserStatus
 
     Q_PROPERTY(bool focus READ hasFocus WRITE setFocus NOTIFY focusChanged FINAL)
     Q_PROPERTY(bool activeFocus READ hasActiveFocus NOTIFY activeFocusChanged FINAL)
+    Q_PROPERTY(bool activeFocusOnTab READ activeFocusOnTab WRITE setActiveFocusOnTab NOTIFY activeFocusOnTabChanged FINAL REVISION 1)
 
     Q_PROPERTY(qreal rotation READ rotation WRITE setRotation NOTIFY rotationChanged)
     Q_PROPERTY(qreal scale READ scale WRITE setScale NOTIFY scaleChanged)
@@ -263,6 +262,9 @@ public:
     bool smooth() const;
     void setSmooth(bool);
 
+    bool activeFocusOnTab() const;
+    void setActiveFocusOnTab(bool);
+
     bool antialiasing() const;
     void setAntialiasing(bool);
 
@@ -276,6 +278,7 @@ public:
     bool hasActiveFocus() const;
     bool hasFocus() const;
     void setFocus(bool);
+    void setFocus(bool focus, Qt::FocusReason reason);
     bool isFocusScope() const;
     QQuickItem *scopedFocusItem() const;
 
@@ -317,9 +320,11 @@ public:
 
     void polish();
 
-    Q_INVOKABLE void mapFromItem(QQmlV8Function*) const;
-    Q_INVOKABLE void mapToItem(QQmlV8Function*) const;
+    Q_INVOKABLE void mapFromItem(QQmlV4Function*) const;
+    Q_INVOKABLE void mapToItem(QQmlV4Function*) const;
     Q_INVOKABLE void forceActiveFocus();
+    Q_INVOKABLE void forceActiveFocus(Qt::FocusReason reason);
+    Q_REVISION(1) Q_INVOKABLE QQuickItem *nextItemInFocusChain(bool forward = true);
     Q_INVOKABLE QQuickItem *childAt(qreal x, qreal y) const;
 
 #ifndef QT_NO_IM
@@ -345,11 +350,13 @@ Q_SIGNALS:
     void stateChanged(const QString &);
     void focusChanged(bool);
     void activeFocusChanged(bool);
+    Q_REVISION(1) void activeFocusOnTabChanged(bool);
     void parentChanged(QQuickItem *);
     void transformOriginChanged(TransformOrigin);
     void smoothChanged(bool);
     void antialiasingChanged(bool);
     void clipChanged(bool);
+    Q_REVISION(1) void windowChanged(QQuickWindow* window);
 
     // XXX todo
     void childrenChanged();
@@ -425,6 +432,8 @@ protected:
     QQuickItem(QQuickItemPrivate &dd, QQuickItem *parent = 0);
 
 private:
+    Q_PRIVATE_SLOT(d_func(), void _q_resourceObjectDeleted(QObject *))
+
     friend class QQuickWindow;
     friend class QQuickWindowPrivate;
     friend class QSGRenderer;
@@ -446,6 +455,4 @@ QT_END_NAMESPACE
 QML_DECLARE_TYPE(QQuickItem)
 QML_DECLARE_TYPE(QQuickTransform)
 
-QT_END_HEADER
-
 #endif // QQUICKITEM_H
