@@ -1118,11 +1118,13 @@ void QQuickItemViewPrivate::itemGeometryChanged(QQuickItem *item, const QRectF &
     if (header && header->item == item) {
         updateHeader();
         markExtentsDirty();
+        updateViewport();
         if (!q->isMoving() && !q->isFlicking())
             fixupPosition();
     } else if (footer && footer->item == item) {
         updateFooter();
         markExtentsDirty();
+        updateViewport();
         if (!q->isMoving() && !q->isFlicking())
             fixupPosition();
     }
@@ -2063,7 +2065,7 @@ void QQuickItemViewPrivate::removeItem(FxViewItem *item, const QQmlChangeSet::Re
 {
     if (removeResult->visiblePos.isValid()) {
         if (item->position() < removeResult->visiblePos)
-            removeResult->sizeChangesBeforeVisiblePos += item->size();
+            updateSizeChangesBeforeVisiblePos(item, removeResult);
         else
             removeResult->sizeChangesAfterVisiblePos += item->size();
     }
@@ -2078,6 +2080,11 @@ void QQuickItemViewPrivate::removeItem(FxViewItem *item, const QQmlChangeSet::Re
         removeResult->changedFirstItem = true;
 }
 
+void QQuickItemViewPrivate::updateSizeChangesBeforeVisiblePos(FxViewItem *item, ChangeResult *removeResult)
+{
+    removeResult->sizeChangesBeforeVisiblePos += item->size();
+}
+
 void QQuickItemViewPrivate::repositionFirstItem(FxViewItem *prevVisibleItemsFirst,
                                                    qreal prevVisibleItemsFirstPos,
                                                    FxViewItem *prevFirstVisible,
