@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,11 +44,9 @@
 
 #include "qquickflickable_p.h"
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-class QQuickChangeSet;
+class QQmlChangeSet;
 
 class QQuickItemViewPrivate;
 
@@ -204,13 +202,14 @@ public:
     Q_INVOKABLE QQuickItem *itemAt(qreal x, qreal y) const;
     Q_INVOKABLE void positionViewAtBeginning();
     Q_INVOKABLE void positionViewAtEnd();
+    Q_REVISION(1) Q_INVOKABLE void forceLayout();
 
     virtual void setContentX(qreal pos);
     virtual void setContentY(qreal pos);
     virtual qreal originX() const;
     virtual qreal originY() const;
 
-signals:
+Q_SIGNALS:
     void modelChanged();
     void delegateChanged();
     void countChanged();
@@ -255,12 +254,12 @@ protected:
     virtual qreal minXExtent() const;
     virtual qreal maxXExtent() const;
 
-protected slots:
+protected Q_SLOTS:
     void destroyRemoved();
-    void createdItem(int index, QQuickItem *item);
-    virtual void initItem(int index, QQuickItem *item);
-    void modelUpdated(const QQuickChangeSet &changeSet, bool reset);
-    void destroyingItem(QQuickItem *item);
+    void createdItem(int index, QObject *item);
+    virtual void initItem(int index, QObject *item);
+    void modelUpdated(const QQmlChangeSet &changeSet, bool reset);
+    void destroyingItem(QObject *item);
     void animStopped();
     void trackedPositionChanged();
 
@@ -289,7 +288,7 @@ public:
     void setIsCurrentItem(bool c) {
         if (m_isCurrent != c) {
             m_isCurrent = c;
-            emit currentItemChanged();
+            Q_EMIT currentItemChanged();
         }
     }
 
@@ -297,7 +296,7 @@ public:
     void setDelayRemove(bool delay) {
         if (m_delayRemove != delay) {
             m_delayRemove = delay;
-            emit delayRemoveChanged();
+            Q_EMIT delayRemoveChanged();
         }
     }
 
@@ -305,7 +304,7 @@ public:
     void setSection(const QString &sect) {
         if (m_section != sect) {
             m_section = sect;
-            emit sectionChanged();
+            Q_EMIT sectionChanged();
         }
     }
 
@@ -313,7 +312,7 @@ public:
     void setPrevSection(const QString &sect) {
         if (m_prevSection != sect) {
             m_prevSection = sect;
-            emit prevSectionChanged();
+            Q_EMIT prevSectionChanged();
         }
     }
 
@@ -321,14 +320,29 @@ public:
     void setNextSection(const QString &sect) {
         if (m_nextSection != sect) {
             m_nextSection = sect;
-            emit nextSectionChanged();
+            Q_EMIT nextSectionChanged();
         }
     }
 
-    void emitAdd() { emit add(); }
-    void emitRemove() { emit remove(); }
+    void setSections(const QString &prev, const QString &sect, const QString &next) {
+        bool prevChanged = prev != m_prevSection;
+        bool sectChanged = sect != m_section;
+        bool nextChanged = next != m_nextSection;
+        m_prevSection = prev;
+        m_section = sect;
+        m_nextSection = next;
+        if (prevChanged)
+            Q_EMIT prevSectionChanged();
+        if (sectChanged)
+            Q_EMIT sectionChanged();
+        if (nextChanged)
+            Q_EMIT nextSectionChanged();
+    }
 
-signals:
+    void emitAdd() { Q_EMIT add(); }
+    void emitRemove() { Q_EMIT remove(); }
+
+Q_SIGNALS:
     void currentItemChanged();
     void delayRemoveChanged();
 
@@ -352,7 +366,5 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQUICKITEMVIEW_P_H
 
