@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -47,6 +47,7 @@
 #include <QMouseEvent>
 #include <math.h>
 #include <QDebug>
+#include <qpa/qplatformnativeinterface.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -55,7 +56,7 @@ DEFINE_BOOL_CONFIG_OPTION(qmlVisualTouchDebugging, QML_VISUAL_TOUCH_DEBUGGING)
 /*!
     \qmltype TouchPoint
     \instantiates QQuickTouchPoint
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-input-events
     \brief Describes a touch point in a MultiPointTouchArea
 
@@ -64,7 +65,7 @@ DEFINE_BOOL_CONFIG_OPTION(qmlVisualTouchDebugging, QML_VISUAL_TOUCH_DEBUGGING)
 */
 
 /*!
-    \qmlproperty int QtQuick2::TouchPoint::pointId
+    \qmlproperty int QtQuick::TouchPoint::pointId
 
     This property holds the point id of the touch point.
 
@@ -79,8 +80,8 @@ void QQuickTouchPoint::setPointId(int id)
 }
 
 /*!
-    \qmlproperty real QtQuick2::TouchPoint::x
-    \qmlproperty real QtQuick2::TouchPoint::y
+    \qmlproperty real QtQuick::TouchPoint::x
+    \qmlproperty real QtQuick::TouchPoint::y
 
     These properties hold the current position of the touch point.
 */
@@ -102,9 +103,9 @@ void QQuickTouchPoint::setY(qreal y)
 }
 
 /*!
-    \qmlproperty real QtQuick2::TouchPoint::pressure
-    \qmlproperty vector2d QtQuick2::TouchPoint::velocity
-    \qmlproperty rectangle QtQuick2::TouchPoint::area
+    \qmlproperty real QtQuick::TouchPoint::pressure
+    \qmlproperty vector2d QtQuick::TouchPoint::velocity
+    \qmlproperty rectangle QtQuick::TouchPoint::area
 
     These properties hold additional information about the current state of the touch point.
 
@@ -142,7 +143,7 @@ void QQuickTouchPoint::setArea(const QRectF &area)
 }
 
 /*!
-    \qmlproperty bool QtQuick2::TouchPoint::pressed
+    \qmlproperty bool QtQuick::TouchPoint::pressed
 
     This property holds whether the touch point is currently pressed.
 */
@@ -155,8 +156,8 @@ void QQuickTouchPoint::setPressed(bool pressed)
 }
 
 /*!
-    \qmlproperty real QtQuick2::TouchPoint::startX
-    \qmlproperty real QtQuick2::TouchPoint::startY
+    \qmlproperty real QtQuick::TouchPoint::startX
+    \qmlproperty real QtQuick::TouchPoint::startY
 
     These properties hold the starting position of the touch point.
 */
@@ -178,8 +179,8 @@ void QQuickTouchPoint::setStartY(qreal startY)
 }
 
 /*!
-    \qmlproperty real QtQuick2::TouchPoint::previousX
-    \qmlproperty real QtQuick2::TouchPoint::previousY
+    \qmlproperty real QtQuick::TouchPoint::previousX
+    \qmlproperty real QtQuick::TouchPoint::previousY
 
     These properties hold the previous position of the touch point.
 */
@@ -200,8 +201,8 @@ void QQuickTouchPoint::setPreviousY(qreal previousY)
 }
 
 /*!
-    \qmlproperty real QtQuick2::TouchPoint::sceneX
-    \qmlproperty real QtQuick2::TouchPoint::sceneY
+    \qmlproperty real QtQuick::TouchPoint::sceneX
+    \qmlproperty real QtQuick::TouchPoint::sceneY
 
     These properties hold the current position of the touch point in scene coordinates.
 */
@@ -225,7 +226,7 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 /*!
     \qmltype MultiPointTouchArea
     \instantiates QQuickMultiPointTouchArea
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-input
     \brief Enables handling of multiple touch points
@@ -250,7 +251,7 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 */
 
 /*!
-    \qmlsignal QtQuick2::MultiPointTouchArea::onPressed(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::onPressed(list<TouchPoint> touchPoints)
 
     This handler is called when new touch points are added. \a touchPoints is a list of these new points.
 
@@ -259,19 +260,19 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 */
 
 /*!
-    \qmlsignal QtQuick2::MultiPointTouchArea::onUpdated(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::onUpdated(list<TouchPoint> touchPoints)
 
     This handler is called when existing touch points are updated. \a touchPoints is a list of these updated points.
 */
 
 /*!
-    \qmlsignal QtQuick2::MultiPointTouchArea::onReleased(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::onReleased(list<TouchPoint> touchPoints)
 
     This handler is called when existing touch points are removed. \a touchPoints is a list of these removed points.
 */
 
 /*!
-    \qmlsignal QtQuick2::MultiPointTouchArea::onCanceled(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::onCanceled(list<TouchPoint> touchPoints)
 
     This handler is called when new touch events have been canceled because another item stole the touch event handling.
 
@@ -286,7 +287,7 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 */
 
 /*!
-    \qmlsignal QtQuick2::MultiPointTouchArea::onGestureStarted(GestureEvent gesture)
+    \qmlsignal QtQuick::MultiPointTouchArea::onGestureStarted(GestureEvent gesture)
 
     This handler is called when the global drag threshold has been reached.
 
@@ -299,7 +300,7 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 */
 
 /*!
-    \qmlsignal QtQuick2::MultiPointTouchArea::onTouchUpdated(list<TouchPoint> touchPoints)
+    \qmlsignal QtQuick::MultiPointTouchArea::onTouchUpdated(list<TouchPoint> touchPoints)
 
     This handler is called when the touch points handled by the MultiPointTouchArea change. This includes adding new touch points,
     removing or canceling previous touch points, as well as updating current touch point data. \a touchPoints is the list of all current touch
@@ -307,7 +308,7 @@ void QQuickTouchPoint::setSceneY(qreal sceneY)
 */
 
 /*!
-    \qmlproperty list<TouchPoint> QtQuick2::MultiPointTouchArea::touchPoints
+    \qmlproperty list<TouchPoint> QtQuick::MultiPointTouchArea::touchPoints
 
     This property holds a set of user-defined touch point objects that can be bound to.
 
@@ -331,6 +332,9 @@ QQuickMultiPointTouchArea::QQuickMultiPointTouchArea(QQuickItem *parent)
     if (qmlVisualTouchDebugging()) {
         setFlag(QQuickItem::ItemHasContents);
     }
+#ifdef Q_OS_OSX
+    setAcceptHoverEvents(true); // needed to enable touch events on mouse hover.
+#endif
 }
 
 QQuickMultiPointTouchArea::~QQuickMultiPointTouchArea()
@@ -344,8 +348,8 @@ QQuickMultiPointTouchArea::~QQuickMultiPointTouchArea()
 }
 
 /*!
-    \qmlproperty int QtQuick2::MultiPointTouchArea::minimumTouchPoints
-    \qmlproperty int QtQuick2::MultiPointTouchArea::maximumTouchPoints
+    \qmlproperty int QtQuick::MultiPointTouchArea::minimumTouchPoints
+    \qmlproperty int QtQuick::MultiPointTouchArea::maximumTouchPoints
 
     These properties hold the range of touch points to be handled by the touch area.
 
@@ -542,6 +546,32 @@ void QQuickMultiPointTouchArea::addTouchPoint(const QTouchEvent::TouchPoint *p)
     _pressedTouchPoints.append(dtp);
 }
 
+#ifdef Q_OS_OSX
+void QQuickMultiPointTouchArea::hoverEnterEvent(QHoverEvent *event)
+{
+    Q_UNUSED(event);
+    setTouchEventsEnabled(true);
+}
+
+void QQuickMultiPointTouchArea::hoverLeaveEvent(QHoverEvent *event)
+{
+    Q_UNUSED(event);
+    setTouchEventsEnabled(false);
+}
+
+void QQuickMultiPointTouchArea::setTouchEventsEnabled(bool enable)
+{
+    // Resolve function for enabling touch events from the (cocoa) platform plugin.
+    typedef void (*RegisterTouchWindowFunction)(QWindow *, bool);
+    RegisterTouchWindowFunction registerTouchWindow = reinterpret_cast<RegisterTouchWindowFunction>(
+        QGuiApplication::platformNativeInterface()->nativeResourceFunctionForIntegration("registertouchwindow"));
+    if (!registerTouchWindow)
+        return; // Not necessarily an error, Qt might be using a different platform plugin.
+
+    registerTouchWindow(window(), enable);
+}
+#endif // Q_OS_OSX
+
 void QQuickMultiPointTouchArea::addTouchPrototype(QQuickTouchPoint *prototype)
 {
     int id = _touchPrototypes.count();
