@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -50,8 +50,6 @@
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qstylehints.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickMultiPointTouchArea;
@@ -252,6 +250,11 @@ protected:
     bool shouldFilter(QEvent *event);
     void grabGesture();
     virtual QSGNode *updatePaintNode(QSGNode *, UpdatePaintNodeData *);
+#ifdef Q_OS_OSX
+    void hoverEnterEvent(QHoverEvent *event);
+    void hoverLeaveEvent(QHoverEvent *event);
+    void setTouchEventsEnabled(bool enable);
+#endif
 
 private:
     void ungrab();
@@ -271,6 +274,4 @@ QML_DECLARE_TYPE(QQuickTouchPoint)
 QML_DECLARE_TYPE(QQuickGrabGestureEvent)
 QML_DECLARE_TYPE(QQuickMultiPointTouchArea)
 
-QT_END_HEADER
-
 #endif // QQUICKMULTIPOINTTOUCHAREA_H
