@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -42,12 +42,14 @@
 #include "qquickpathview_p.h"
 #include "qquickpathview_p_p.h"
 #include "qquickwindow.h"
+#include "qquickflickablebehavior_p.h" //Contains flicking behavior defines
 
 #include <QtQuick/private/qquickstate_p.h>
 #include <private/qqmlglobal_p.h>
 #include <private/qqmlopenmetaobject_p.h>
-#include <private/qquickchangeset_p.h>
+#include <private/qqmlchangeset_p.h>
 
+#include <QtQml/qqmlinfo.h>
 #include <QtGui/qevent.h>
 #include <QtGui/qevent.h>
 #include <QtGui/qguiapplication.h>
@@ -55,22 +57,6 @@
 #include <QtCore/qmath.h>
 #include <math.h>
 
-// The number of samples to use in calculating the velocity of a flick
-#ifndef QML_FLICK_SAMPLEBUFFER
-#define QML_FLICK_SAMPLEBUFFER 1
-#endif
-
-// The number of samples to discard when calculating the flick velocity.
-// Touch panels often produce inaccurate results as the finger is lifted.
-#ifndef QML_FLICK_DISCARDSAMPLES
-#define QML_FLICK_DISCARDSAMPLES 0
-#endif
-
-// The default maximum velocity of a flick.
-#ifndef QML_FLICK_DEFAULTMAXVELOCITY
-#define QML_FLICK_DEFAULTMAXVELOCITY 2500
-#endif
-
 
 QT_BEGIN_NAMESPACE
 
@@ -117,7 +103,7 @@ QQuickPathViewPrivate::QQuickPathViewPrivate()
     , offset(0.0), offsetAdj(0.0), mappedRange(1.0), mappedCache(0.0)
     , stealMouse(false), ownModel(false), interactive(true), haveHighlightRange(true)
     , autoHighlight(true), highlightUp(false), layoutScheduled(false)
-    , moving(false), flicking(false), dragging(false), inRequest(false)
+    , moving(false), flicking(false), dragging(false), inRequest(false), delegateValidated(false)
     , dragMargin(0), deceleration(100), maximumFlickVelocity(QML_FLICK_DEFAULTMAXVELOCITY)
     , moveOffset(this, &QQuickPathViewPrivate::setAdjustedOffset), flickDuration(0)
     , firstIndex(-1), pathItems(-1), requestedIndex(-1), cacheSize(0), requestedZ(0)
@@ -150,8 +136,18 @@ QQuickItem *QQuickPathViewPrivate::getItem(int modelIndex, qreal z, bool async)
     requestedIndex = modelIndex;
     requestedZ = z;
     inRequest = true;
-    QQuickItem *item = model->item(modelIndex, async);
-    if (item) {
+    QObject *object = model->object(modelIndex, async);
+    QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
+    if (!item) {
+        if (object) {
+            model->release(object);
+            if (!delegateValidated) {
+                delegateValidated = true;
+                QObject* delegate = q->delegate();
+                qmlInfo(delegate ? delegate : q) << QQuickPathView::tr("Delegate must be of Item type");
+            }
+        }
+    } else {
         item->setParentItem(q);
         requestedIndex = -1;
         QQuickItemPrivate *itemPrivate = QQuickItemPrivate::get(item);
@@ -161,9 +157,10 @@ QQuickItem *QQuickPathViewPrivate::getItem(int modelIndex, qreal z, bool async)
     return item;
 }
 
-void QQuickPathView::createdItem(int index, QQuickItem *item)
+void QQuickPathView::createdItem(int index, QObject *object)
 {
     Q_D(QQuickPathView);
+    QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
     if (d->requestedIndex != index) {
         qPathViewAttachedType = d->attachedType();
         QQuickPathViewAttached *att = static_cast<QQuickPathViewAttached *>(qmlAttachedPropertiesObject<QQuickPathView>(item));
@@ -181,10 +178,11 @@ void QQuickPathView::createdItem(int index, QQuickItem *item)
     }
 }
 
-void QQuickPathView::initItem(int index, QQuickItem *item)
+void QQuickPathView::initItem(int index, QObject *object)
 {
     Q_D(QQuickPathView);
-    if (d->requestedIndex == index) {
+    QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
+    if (item && d->requestedIndex == index) {
         QQuickItemPrivate::get(item)->setCulled(true);
         item->setParentItem(this);
         qPathViewAttachedType = d->attachedType();
@@ -209,10 +207,14 @@ void QQuickPathViewPrivate::releaseItem(QQuickItem *item)
         return;
     QQuickItemPrivate *itemPrivate = QQuickItemPrivate::get(item);
     itemPrivate->removeItemChangeListener(this, QQuickItemPrivate::Geometry);
-    if (model->release(item) == 0) {
+    QQmlInstanceModel::ReleaseFlags flags = model->release(item);
+    if (!flags) {
         // item was not destroyed, and we no longer reference it.
         if (QQuickPathViewAttached *att = attached(item))
             att->setOnPath(false);
+    } else if (flags & QQmlInstanceModel::Destroyed) {
+        // but we still reference it
+        item->setParentItem(0);
     }
 }
 
@@ -475,7 +477,7 @@ void QQuickPathViewPrivate::setDragging(bool d)
 /*!
     \qmltype PathView
     \instantiates QQuickPathView
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-paths
     \ingroup qtquick-views
     \inherits Item
@@ -536,7 +538,7 @@ void QQuickPathViewPrivate::setDragging(bool d)
     to set \e {clip: true} in order to have the out of view items clipped
     nicely.
 
-    \sa Path, {quick/modelviews/pathview}{PathView example}
+    \sa Path, {QML Data Models}, ListView, GridView, {Qt Quick Examples - Views}
 */
 
 QQuickPathView::QQuickPathView(QQuickItem *parent)
@@ -557,14 +559,14 @@ QQuickPathView::~QQuickPathView()
 }
 
 /*!
-    \qmlattachedproperty PathView QtQuick2::PathView::view
+    \qmlattachedproperty PathView QtQuick::PathView::view
     This attached property holds the view that manages this delegate instance.
 
     It is attached to each instance of the delegate.
 */
 
 /*!
-    \qmlattachedproperty bool QtQuick2::PathView::onPath
+    \qmlattachedproperty bool QtQuick::PathView::onPath
     This attached property holds whether the item is currently on the path.
 
     If a pathItemCount has been set, it is possible that some items may
@@ -584,7 +586,7 @@ QQuickPathView::~QQuickPathView()
 */
 
 /*!
-    \qmlattachedproperty bool QtQuick2::PathView::isCurrentItem
+    \qmlattachedproperty bool QtQuick::PathView::isCurrentItem
     This attached property is true if this delegate is the current item; otherwise false.
 
     It is attached to each instance of the delegate.
@@ -595,7 +597,7 @@ QQuickPathView::~QQuickPathView()
 */
 
 /*!
-    \qmlproperty model QtQuick2::PathView::model
+    \qmlproperty model QtQuick::PathView::model
     This property holds the model providing data for the view.
 
     The model provides a set of data that is used to create the items for the view.
@@ -619,19 +621,19 @@ void QQuickPathView::setModel(const QVariant &model)
         return;
 
     if (d->model) {
-        qmlobject_disconnect(d->model, QQuickVisualModel, SIGNAL(modelUpdated(QQuickChangeSet,bool)),
-                             this, QQuickPathView, SLOT(modelUpdated(QQuickChangeSet,bool)));
-        qmlobject_disconnect(d->model, QQuickVisualModel, SIGNAL(createdItem(int,QQuickItem*)),
-                             this, QQuickPathView, SLOT(createdItem(int,QQuickItem*)));
-        qmlobject_disconnect(d->model, QQuickVisualModel, SIGNAL(initItem(int,QQuickItem*)),
-                             this, QQuickPathView, SLOT(initItem(int,QQuickItem*)));
+        qmlobject_disconnect(d->model, QQmlInstanceModel, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
+                             this, QQuickPathView, SLOT(modelUpdated(QQmlChangeSet,bool)));
+        qmlobject_disconnect(d->model, QQmlInstanceModel, SIGNAL(createdItem(int,QObject*)),
+                             this, QQuickPathView, SLOT(createdItem(int,QObject*)));
+        qmlobject_disconnect(d->model, QQmlInstanceModel, SIGNAL(initItem(int,QObject*)),
+                             this, QQuickPathView, SLOT(initItem(int,QObject*)));
         d->clear();
     }
 
     d->modelVariant = model;
     QObject *object = qvariant_cast<QObject*>(model);
-    QQuickVisualModel *vim = 0;
-    if (object && (vim = qobject_cast<QQuickVisualModel *>(object))) {
+    QQmlInstanceModel *vim = 0;
+    if (object && (vim = qobject_cast<QQmlInstanceModel *>(object))) {
         if (d->ownModel) {
             delete d->model;
             d->ownModel = false;
@@ -639,23 +641,23 @@ void QQuickPathView::setModel(const QVariant &model)
         d->model = vim;
     } else {
         if (!d->ownModel) {
-            d->model = new QQuickVisualDataModel(qmlContext(this));
+            d->model = new QQmlDelegateModel(qmlContext(this));
             d->ownModel = true;
             if (isComponentComplete())
-                static_cast<QQuickVisualDataModel *>(d->model.data())->componentComplete();
+                static_cast<QQmlDelegateModel *>(d->model.data())->componentComplete();
         }
-        if (QQuickVisualDataModel *dataModel = qobject_cast<QQuickVisualDataModel*>(d->model))
+        if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model))
             dataModel->setModel(model);
     }
     int oldModelCount = d->modelCount;
     d->modelCount = 0;
     if (d->model) {
-        qmlobject_connect(d->model, QQuickVisualModel, SIGNAL(modelUpdated(QQuickChangeSet,bool)),
-                          this, QQuickPathView, SLOT(modelUpdated(QQuickChangeSet,bool)));
-        qmlobject_connect(d->model, QQuickVisualModel, SIGNAL(createdItem(int,QQuickItem*)),
-                          this, QQuickPathView, SLOT(createdItem(int,QQuickItem*)));
-        qmlobject_connect(d->model, QQuickVisualModel, SIGNAL(initItem(int,QQuickItem*)),
-                          this, QQuickPathView, SLOT(initItem(int,QQuickItem*)));
+        qmlobject_connect(d->model, QQmlInstanceModel, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
+                          this, QQuickPathView, SLOT(modelUpdated(QQmlChangeSet,bool)));
+        qmlobject_connect(d->model, QQmlInstanceModel, SIGNAL(createdItem(int,QObject*)),
+                          this, QQuickPathView, SLOT(createdItem(int,QObject*)));
+        qmlobject_connect(d->model, QQmlInstanceModel, SIGNAL(initItem(int,QObject*)),
+                          this, QQuickPathView, SLOT(initItem(int,QObject*)));
         d->modelCount = d->model->count();
     }
     if (isComponentComplete()) {
@@ -675,7 +677,7 @@ void QQuickPathView::setModel(const QVariant &model)
 }
 
 /*!
-    \qmlproperty int QtQuick2::PathView::count
+    \qmlproperty int QtQuick::PathView::count
     This property holds the number of items in the model.
 */
 int QQuickPathView::count() const
@@ -685,7 +687,7 @@ int QQuickPathView::count() const
 }
 
 /*!
-    \qmlproperty Path QtQuick2::PathView::path
+    \qmlproperty Path QtQuick::PathView::path
     This property holds the path used to lay out the items.
     For more information see the \l Path documentation.
 */
@@ -718,7 +720,7 @@ void QQuickPathView::setPath(QQuickPath *path)
 }
 
 /*!
-    \qmlproperty int QtQuick2::PathView::currentIndex
+    \qmlproperty int QtQuick::PathView::currentIndex
     This property holds the index of the current item.
 */
 int QQuickPathView::currentIndex() const
@@ -727,10 +729,6 @@ int QQuickPathView::currentIndex() const
     return d->currentIndex;
 }
 
-/*!
-    \qmlproperty int QtQuick2::PathView::currentItem
-    This property holds the current item in the view.
-*/
 void QQuickPathView::setCurrentIndex(int idx)
 {
     Q_D(QQuickPathView);
@@ -770,6 +768,10 @@ void QQuickPathView::setCurrentIndex(int idx)
     }
 }
 
+/*!
+    \qmlproperty Item QtQuick::PathView::currentItem
+    This property holds the current item in the view.
+*/
 QQuickItem *QQuickPathView::currentItem() const
 {
     Q_D(const QQuickPathView);
@@ -777,7 +779,7 @@ QQuickItem *QQuickPathView::currentItem() const
 }
 
 /*!
-    \qmlmethod QtQuick2::PathView::incrementCurrentIndex()
+    \qmlmethod QtQuick::PathView::incrementCurrentIndex()
 
     Increments the current index.
 
@@ -791,7 +793,7 @@ void QQuickPathView::incrementCurrentIndex()
 }
 
 /*!
-    \qmlmethod QtQuick2::PathView::decrementCurrentIndex()
+    \qmlmethod QtQuick::PathView::decrementCurrentIndex()
 
     Decrements the current index.
 
@@ -805,7 +807,7 @@ void QQuickPathView::decrementCurrentIndex()
 }
 
 /*!
-    \qmlproperty real QtQuick2::PathView::offset
+    \qmlproperty real QtQuick::PathView::offset
 
     The offset specifies how far along the path the items are from their initial positions.
     This is a real number that ranges from 0.0 to the count of items in the model.
@@ -846,7 +848,7 @@ void QQuickPathViewPrivate::setAdjustedOffset(qreal o)
 }
 
 /*!
-    \qmlproperty Component QtQuick2::PathView::highlight
+    \qmlproperty Component QtQuick::PathView::highlight
     This property holds the component to use as the highlight.
 
     An instance of the highlight component will be created for each view.
@@ -887,7 +889,7 @@ void QQuickPathView::setHighlight(QQmlComponent *highlight)
 }
 
 /*!
-  \qmlproperty Item QtQuick2::PathView::highlightItem
+  \qmlproperty Item QtQuick::PathView::highlightItem
 
   \c highlightItem holds the highlight item, which was created
   from the \l highlight component.
@@ -900,39 +902,42 @@ QQuickItem *QQuickPathView::highlightItem()
     return d->highlightItem;
 }
 /*!
-    \qmlproperty real QtQuick2::PathView::preferredHighlightBegin
-    \qmlproperty real QtQuick2::PathView::preferredHighlightEnd
-    \qmlproperty enumeration QtQuick2::PathView::highlightRangeMode
+    \qmlproperty real QtQuick::PathView::preferredHighlightBegin
+    \qmlproperty real QtQuick::PathView::preferredHighlightEnd
+    \qmlproperty enumeration QtQuick::PathView::highlightRangeMode
 
     These properties set the preferred range of the highlight (current item)
-    within the view.  The preferred values must be in the range 0.0-1.0.
+    within the view. The preferred values must be in the range 0.0-1.0.
 
-    If highlightRangeMode is set to \e PathView.NoHighlightRange
+    Valid values for \c highlightRangeMode are:
 
-    If highlightRangeMode is set to \e PathView.ApplyRange the view will
-    attempt to maintain the highlight within the range, however
-    the highlight can move outside of the range at the ends of the path
-    or due to a mouse interaction.
+    \list
+        \li \e PathView.NoHighlightRange - no range is applied and the
+            highlight will move freely within the view.
+        \li \e PathView.ApplyRange - the view will attempt to maintain
+            the highlight within the range, however the highlight can
+            move outside of the range at the ends of the path or due to
+            a mouse interaction.
+        \li \e PathView.StrictlyEnforceRange - the highlight will never
+            move outside of the range. This means that the current item
+            will change if a keyboard or mouse action would cause the
+            highlight to move outside of the range.
+    \endlist
 
-    If highlightRangeMode is set to \e PathView.StrictlyEnforceRange the highlight will never
-    move outside of the range.  This means that the current item will change
-    if a keyboard or mouse action would cause the highlight to move
-    outside of the range.
+    The default value is \e PathView.StrictlyEnforceRange.
 
-    Note that this is the correct way to influence where the
+    Defining a highlight range is the correct way to influence where the
     current item ends up when the view moves. For example, if you want the
     currently selected item to be in the middle of the path, then set the
-    highlight range to be 0.5,0.5 and highlightRangeMode to PathView.StrictlyEnforceRange.
+    highlight range to be 0.5,0.5 and highlightRangeMode to \e PathView.StrictlyEnforceRange.
     Then, when the path scrolls,
     the currently selected item will be the item at that position. This also applies to
     when the currently selected item changes - it will scroll to within the preferred
     highlight range. Furthermore, the behaviour of the current item index will occur
     whether or not a highlight exists.
 
-    The default value is \e PathView.StrictlyEnforceRange.
-
-    Note that a valid range requires preferredHighlightEnd to be greater
-    than or equal to preferredHighlightBegin.
+    \note A valid range requires \c preferredHighlightEnd to be greater
+    than or equal to \c preferredHighlightBegin.
 */
 qreal QQuickPathView::preferredHighlightBegin() const
 {
@@ -991,7 +996,7 @@ void QQuickPathView::setHighlightRangeMode(HighlightRangeMode mode)
 }
 
 /*!
-    \qmlproperty int QtQuick2::PathView::highlightMoveDuration
+    \qmlproperty int QtQuick::PathView::highlightMoveDuration
     This property holds the move animation duration of the highlight delegate.
 
     If the highlightRangeMode is StrictlyEnforceRange then this property
@@ -1015,7 +1020,7 @@ void QQuickPathView::setHighlightMoveDuration(int duration)
 }
 
 /*!
-    \qmlproperty real QtQuick2::PathView::dragMargin
+    \qmlproperty real QtQuick::PathView::dragMargin
     This property holds the maximum distance from the path that initiate mouse dragging.
 
     By default the path can only be dragged by clicking on an item.  If
@@ -1038,7 +1043,7 @@ void QQuickPathView::setDragMargin(qreal dragMargin)
 }
 
 /*!
-    \qmlproperty real QtQuick2::PathView::flickDeceleration
+    \qmlproperty real QtQuick::PathView::flickDeceleration
     This property holds the rate at which a flick will decelerate.
 
     The default is 100.
@@ -1059,7 +1064,7 @@ void QQuickPathView::setFlickDeceleration(qreal dec)
 }
 
 /*!
-    \qmlproperty real QtQuick2::PathView::maximumFlickVelocity
+    \qmlproperty real QtQuick::PathView::maximumFlickVelocity
     This property holds the approximate maximum velocity that the user can flick the view in pixels/second.
 
     The default value is platform dependent.
@@ -1081,7 +1086,7 @@ void QQuickPathView::setMaximumFlickVelocity(qreal vel)
 
 
 /*!
-    \qmlproperty bool QtQuick2::PathView::interactive
+    \qmlproperty bool QtQuick::PathView::interactive
 
     A user cannot drag or flick a PathView that is not interactive.
 
@@ -1106,7 +1111,7 @@ void QQuickPathView::setInteractive(bool interactive)
 }
 
 /*!
-    \qmlproperty bool QtQuick2::PathView::moving
+    \qmlproperty bool QtQuick::PathView::moving
 
     This property holds whether the view is currently moving
     due to the user either dragging or flicking the view.
@@ -1118,7 +1123,7 @@ bool QQuickPathView::isMoving() const
 }
 
 /*!
-    \qmlproperty bool QtQuick2::PathView::flicking
+    \qmlproperty bool QtQuick::PathView::flicking
 
     This property holds whether the view is currently moving
     due to the user flicking the view.
@@ -1130,7 +1135,7 @@ bool QQuickPathView::isFlicking() const
 }
 
 /*!
-    \qmlproperty bool QtQuick2::PathView::dragging
+    \qmlproperty bool QtQuick::PathView::dragging
 
     This property holds whether the view is currently moving
     due to the user dragging the view.
@@ -1142,14 +1147,14 @@ bool QQuickPathView::isDragging() const
 }
 
 /*!
-    \qmlsignal QtQuick2::PathView::onMovementStarted()
+    \qmlsignal QtQuick::PathView::onMovementStarted()
 
     This handler is called when the view begins moving due to user
     interaction.
 */
 
 /*!
-    \qmlsignal QtQuick2::PathView::onMovementEnded()
+    \qmlsignal QtQuick::PathView::onMovementEnded()
 
     This handler is called when the view stops moving due to user
     interaction.  If a flick was generated, this handler will
@@ -1159,7 +1164,7 @@ bool QQuickPathView::isDragging() const
 */
 
 /*!
-    \qmlsignal QtQuick2::PathView::onFlickStarted()
+    \qmlsignal QtQuick::PathView::onFlickStarted()
 
     This handler is called when the view is flicked.  A flick
     starts from the point that the mouse or touch is released,
@@ -1167,20 +1172,20 @@ bool QQuickPathView::isDragging() const
 */
 
 /*!
-    \qmlsignal QtQuick2::PathView::onFlickEnded()
+    \qmlsignal QtQuick::PathView::onFlickEnded()
 
     This handler is called when the view stops moving due to a flick.
 */
 
 /*!
-    \qmlsignal QtQuick2::PathView::onDragStarted()
+    \qmlsignal QtQuick::PathView::onDragStarted()
 
     This handler is called when the view starts to be dragged due to user
     interaction.
 */
 
 /*!
-    \qmlsignal QtQuick2::PathView::onDragEnded()
+    \qmlsignal QtQuick::PathView::onDragEnded()
 
     This handler is called when the user stops dragging the view.
 
@@ -1189,7 +1194,7 @@ bool QQuickPathView::isDragging() const
 */
 
 /*!
-    \qmlproperty Component QtQuick2::PathView::delegate
+    \qmlproperty Component QtQuick::PathView::delegate
 
     The delegate provides a template defining each item instantiated by the view.
     The index is exposed as an accessible \c index property.  Properties of the
@@ -1210,7 +1215,7 @@ QQmlComponent *QQuickPathView::delegate() const
 {
     Q_D(const QQuickPathView);
      if (d->model) {
-        if (QQuickVisualDataModel *dataModel = qobject_cast<QQuickVisualDataModel*>(d->model))
+        if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model))
             return dataModel->delegate();
     }
 
@@ -1223,10 +1228,10 @@ void QQuickPathView::setDelegate(QQmlComponent *delegate)
     if (delegate == this->delegate())
         return;
     if (!d->ownModel) {
-        d->model = new QQuickVisualDataModel(qmlContext(this));
+        d->model = new QQmlDelegateModel(qmlContext(this));
         d->ownModel = true;
     }
-    if (QQuickVisualDataModel *dataModel = qobject_cast<QQuickVisualDataModel*>(d->model)) {
+    if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model)) {
         int oldCount = dataModel->count();
         dataModel->setDelegate(delegate);
         d->modelCount = dataModel->count();
@@ -1234,11 +1239,12 @@ void QQuickPathView::setDelegate(QQmlComponent *delegate)
         if (oldCount != dataModel->count())
             emit countChanged();
         emit delegateChanged();
+        d->delegateValidated = false;
     }
 }
 
 /*!
-  \qmlproperty int QtQuick2::PathView::pathItemCount
+  \qmlproperty int QtQuick::PathView::pathItemCount
   This property holds the number of items visible on the path at any one time.
 
   Setting pathItemCount to undefined will show all items on the path.
@@ -1277,7 +1283,7 @@ void QQuickPathView::resetPathItemCount()
 }
 
 /*!
-    \qmlproperty int QtQuick2::PathView::cacheItemCount
+    \qmlproperty int QtQuick::PathView::cacheItemCount
     This property holds the maximum number of items to cache off the path.
 
     For example, a PathView with a model containing 20 items, a pathItemCount
@@ -1314,7 +1320,7 @@ void QQuickPathView::setCacheItemCount(int i)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::PathView::snapMode
+    \qmlproperty enumeration QtQuick::PathView::snapMode
 
     This property determines how the items will settle following a drag or flick.
     The possible values are:
@@ -1349,7 +1355,7 @@ void QQuickPathView::setSnapMode(SnapMode mode)
 }
 
 /*!
-    \qmlmethod QtQuick2::PathView::positionViewAtIndex(int index, PositionMode mode)
+    \qmlmethod QtQuick::PathView::positionViewAtIndex(int index, PositionMode mode)
 
     Positions the view such that the \a index is at the position specified by
     \a mode:
@@ -1437,7 +1443,7 @@ void QQuickPathView::positionViewAtIndex(int index, int mode)
 }
 
 /*!
-    \qmlmethod int QtQuick2::PathView::indexAt(int x, int y)
+    \qmlmethod int QtQuick::PathView::indexAt(int x, int y)
 
     Returns the index of the item containing the point \a x, \a y in content
     coordinates.  If there is no item at the point specified, -1 is returned.
@@ -1461,7 +1467,7 @@ int QQuickPathView::indexAt(qreal x, qreal y) const
 }
 
 /*!
-    \qmlmethod Item QtQuick2::PathView::itemAt(int x, int y)
+    \qmlmethod Item QtQuick::PathView::itemAt(int x, int y)
 
     Returns the item containing the point \a x, \a y in content
     coordinates.  If there is no item at the point specified, null is returned.
@@ -1547,11 +1553,9 @@ qreal QQuickPathViewPrivate::calcVelocity() const
 
 qint64 QQuickPathViewPrivate::computeCurrentTime(QInputEvent *event)
 {
-    if (0 != event->timestamp() && QQuickItemPrivate::consistentTime == -1) {
+    if (0 != event->timestamp())
         return event->timestamp();
-    }
-
-    return QQuickItemPrivate::elapsed(timer);
+    return timer.elapsed();
 }
 
 void QQuickPathView::mousePressEvent(QMouseEvent *event)
@@ -1592,7 +1596,7 @@ void QQuickPathViewPrivate::handleMousePressEvent(QMouseEvent *event)
     else
         stealMouse = false;
 
-    QQuickItemPrivate::start(timer);
+    timer.start();
     lastPosTime = computeCurrentTime(event);
     tl.clear();
 }
@@ -1743,37 +1747,48 @@ bool QQuickPathView::sendMouseEvent(QMouseEvent *event)
 
     QQuickWindow *c = window();
     QQuickItem *grabber = c ? c->mouseGrabberItem() : 0;
-    bool stealThisEvent = d->stealMouse;
-    if ((stealThisEvent || contains(localPos)) && (!grabber || !grabber->keepMouseGrab())) {
-        QMouseEvent mouseEvent(event->type(), localPos, event->windowPos(), event->screenPos(),
-                               event->button(), event->buttons(), event->modifiers());
-        mouseEvent.setAccepted(false);
+    if (grabber == this && d->stealMouse) {
+        // we are already the grabber and we do want the mouse event to ourselves.
+        return true;
+    }
 
-        switch (mouseEvent.type()) {
+    bool grabberDisabled = grabber && !grabber->isEnabled();
+    bool stealThisEvent = d->stealMouse;
+    if ((stealThisEvent || contains(localPos)) && (!grabber || !grabber->keepMouseGrab() || grabberDisabled)) {
+        QScopedPointer<QMouseEvent> mouseEvent(QQuickWindowPrivate::cloneMouseEvent(event, &localPos));
+        mouseEvent->setAccepted(false);
+
+        switch (mouseEvent->type()) {
         case QEvent::MouseMove:
-            d->handleMouseMoveEvent(&mouseEvent);
+            d->handleMouseMoveEvent(mouseEvent.data());
             break;
         case QEvent::MouseButtonPress:
-            d->handleMousePressEvent(&mouseEvent);
+            d->handleMousePressEvent(mouseEvent.data());
             stealThisEvent = d->stealMouse;   // Update stealThisEvent in case changed by function call above
             break;
         case QEvent::MouseButtonRelease:
-            d->handleMouseReleaseEvent(&mouseEvent);
+            d->handleMouseReleaseEvent(mouseEvent.data());
             break;
         default:
             break;
         }
         grabber = c->mouseGrabberItem();
-        if (grabber && stealThisEvent && !grabber->keepMouseGrab() && grabber != this)
+        if ((grabber && stealThisEvent && !grabber->keepMouseGrab() && grabber != this) || grabberDisabled) {
             grabMouse();
+        }
 
-        return d->stealMouse;
+        const bool filtered = stealThisEvent || grabberDisabled;
+        if (filtered) {
+            event->setAccepted(false);
+        }
+        return filtered;
     } else if (d->timer.isValid()) {
         d->timer.invalidate();
         d->fixOffset();
     }
-    if (event->type() == QEvent::MouseButtonRelease)
+    if (event->type() == QEvent::MouseButtonRelease || (grabber && grabber->keepMouseGrab() && !grabberDisabled)) {
         d->stealMouse = false;
+    }
     return false;
 }
 
@@ -1821,7 +1836,7 @@ void QQuickPathView::componentComplete()
 {
     Q_D(QQuickPathView);
     if (d->model && d->ownModel)
-        static_cast<QQuickVisualDataModel *>(d->model.data())->componentComplete();
+        static_cast<QQmlDelegateModel *>(d->model.data())->componentComplete();
 
     QQuickItem::componentComplete();
 
@@ -1983,7 +1998,7 @@ void QQuickPathView::refill()
         d->releaseItem(d->itemCache.takeLast());
 }
 
-void QQuickPathView::modelUpdated(const QQuickChangeSet &changeSet, bool reset)
+void QQuickPathView::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
 {
     Q_D(QQuickPathView);
     if (!d->model || !d->model->isValid() || !d->path || !isComponentComplete())
@@ -2001,10 +2016,10 @@ void QQuickPathView::modelUpdated(const QQuickChangeSet &changeSet, bool reset)
 
     const int modelCount = d->modelCount;
     int moveId = -1;
-    int moveOffset;
+    int moveOffset = 0;
     bool currentChanged = false;
     bool changedOffset = false;
-    foreach (const QQuickChangeSet::Remove &r, changeSet.removes()) {
+    foreach (const QQmlChangeSet::Remove &r, changeSet.removes()) {
         if (moveId == -1 && d->currentIndex >= r.index + r.count) {
             d->currentIndex -= r.count;
             currentChanged = true;
@@ -2030,7 +2045,7 @@ void QQuickPathView::modelUpdated(const QQuickChangeSet &changeSet, bool reset)
         }
         d->modelCount -= r.count;
     }
-    foreach (const QQuickChangeSet::Insert &i, changeSet.inserts()) {
+    foreach (const QQmlChangeSet::Insert &i, changeSet.inserts()) {
         if (d->modelCount) {
             if (moveId == -1 && i.index <= d->currentIndex) {
                 d->currentIndex += i.count;
@@ -2082,7 +2097,7 @@ void QQuickPathView::modelUpdated(const QQuickChangeSet &changeSet, bool reset)
         emit countChanged();
 }
 
-void QQuickPathView::destroyingItem(QQuickItem *item)
+void QQuickPathView::destroyingItem(QObject *item)
 {
     Q_UNUSED(item);
 }
