@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,11 +46,9 @@
 
 #include <private/qquickpath_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-class QQuickChangeSet;
+class QQmlChangeSet;
 
 class QQuickPathViewPrivate;
 class QQuickPathViewAttached;
@@ -216,10 +214,10 @@ private Q_SLOTS:
     void refill();
     void ticked();
     void movementEnding();
-    void modelUpdated(const QQuickChangeSet &changeSet, bool reset);
-    void createdItem(int index, QQuickItem *item);
-    void initItem(int index, QQuickItem *item);
-    void destroyingItem(QQuickItem *item);
+    void modelUpdated(const QQmlChangeSet &changeSet, bool reset);
+    void createdItem(int index, QObject *item);
+    void initItem(int index, QObject *item);
+    void destroyingItem(QObject *item);
     void pathUpdated();
 
 private:
@@ -247,7 +245,7 @@ public:
     void setIsCurrentItem(bool c) {
         if (m_isCurrent != c) {
             m_isCurrent = c;
-            emit currentItemChanged();
+            Q_EMIT currentItemChanged();
         }
     }
 
@@ -258,7 +256,7 @@ public:
     void setOnPath(bool on) {
         if (on != m_onPath) {
             m_onPath = on;
-            emit pathChanged();
+            Q_EMIT pathChanged();
         }
     }
     qreal m_percent;
@@ -281,6 +279,5 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickPathView)
 QML_DECLARE_TYPEINFO(QQuickPathView, QML_HAS_ATTACHED_PROPERTIES)
-QT_END_HEADER
 
 #endif // QQUICKPATHVIEW_P_H
