@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -55,14 +55,13 @@
 
 #include "qquickpathview_p.h"
 #include "qquickitem_p.h"
-#include "qquickvisualdatamodel_p.h"
 
 #include <QtQml/qqml.h>
 #include <QtCore/qdatetime.h>
 #include <QtCore/qcoreapplication.h>
 
 #include <private/qquickanimation_p_p.h>
-#include <private/qqmlguard_p.h>
+#include <private/qqmldelegatemodel_p.h>
 #include <private/qquicktimeline_p_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -132,7 +131,7 @@ public:
 
     QQuickPath *path;
     int currentIndex;
-    QQmlGuard<QQuickItem> currentItem;
+    QPointer<QQuickItem> currentItem;
     qreal currentItemOffset;
     qreal startPc;
     QPointF startPoint;
@@ -152,6 +151,7 @@ public:
     bool dragging : 1;
     bool requestedOnPath : 1;
     bool inRequest : 1;
+    bool delegateValidated : 1;
     QElapsedTimer timer;
     qint64 lastPosTime;
     QPointF lastPos;
@@ -168,7 +168,7 @@ public:
     qreal requestedZ;
     QList<QQuickItem *> items;
     QList<QQuickItem *> itemCache;
-    QQmlGuard<QQuickVisualModel> model;
+    QPointer<QQmlInstanceModel> model;
     QVariant modelVariant;
     enum MovementReason { Other, SetIndex, Mouse };
     MovementReason moveReason;
