@@ -221,6 +221,7 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 
 
 /*!
+    \qmlpropertygroup QtQuick::PinchArea::pinch
     \qmlproperty Item QtQuick::PinchArea::pinch.target
     \qmlproperty bool QtQuick::PinchArea::pinch.active
     \qmlproperty real QtQuick::PinchArea::pinch.minimumScale
@@ -247,14 +248,13 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 
 QQuickPinchArea::QQuickPinchArea(QQuickItem *parent)
   : QQuickItem(*(new QQuickPinchAreaPrivate), parent)
-  , _currentWindow(0)
 {
     Q_D(QQuickPinchArea);
     d->init();
     setAcceptedMouseButtons(Qt::LeftButton);
     setFiltersChildMouseEvents(true);
-#ifdef Q_OS_MAC
-    connect(this, &QQuickItem::windowChanged, this, &QQuickPinchArea::setTouchEventsEnabledForWindow);
+#ifdef Q_OS_OSX
+    setAcceptHoverEvents(true); // needed to enable touch events on mouse hover.
 #endif
 }
 
@@ -536,6 +536,32 @@ void QQuickPinchArea::itemChange(ItemChange change, const ItemChangeData &value)
     QQuickItem::itemChange(change, value);
 }
 
+#ifdef Q_OS_OSX
+void QQuickPinchArea::hoverEnterEvent(QHoverEvent *event)
+{
+    Q_UNUSED(event);
+    setTouchEventsEnabled(true);
+}
+
+void QQuickPinchArea::hoverLeaveEvent(QHoverEvent *event)
+{
+    Q_UNUSED(event);
+    setTouchEventsEnabled(false);
+}
+
+void QQuickPinchArea::setTouchEventsEnabled(bool enable)
+{
+    // Resolve function for enabling touch events from the (cocoa) platform plugin.
+    typedef void (*RegisterTouchWindowFunction)(QWindow *, bool);
+    RegisterTouchWindowFunction registerTouchWindow = reinterpret_cast<RegisterTouchWindowFunction>(
+        QGuiApplication::platformNativeInterface()->nativeResourceFunctionForIntegration("registertouchwindow"));
+    if (!registerTouchWindow)
+        return; // Not necessarily an error, Qt might be using a different platform plugin.
+
+    registerTouchWindow(window(), enable);
+}
+#endif // Q_OS_OSX
+
 QQuickPinch *QQuickPinchArea::pinch()
 {
     Q_D(QQuickPinchArea);
@@ -544,29 +570,5 @@ QQuickPinch *QQuickPinchArea::pinch()
     return d->pinch;
 }
 
-void QQuickPinchArea::setTouchEventsEnabledForWindow(QWindow *window)
-{
-#ifdef Q_OS_MAC
-    // Resolve function for enabling touch events from the (cocoa) platform plugin.
-    typedef void (*RegisterTouchWindowFunction)(QWindow *, bool);
-    RegisterTouchWindowFunction registerTouchWindow = reinterpret_cast<RegisterTouchWindowFunction>(
-        QGuiApplication::platformNativeInterface()->nativeResourceFunctionForIntegration("registertouchwindow"));
-    if (!registerTouchWindow)
-        return; // Not necessarily an error, Qt migh be using a different platform plugin.
-
-    // Disable touch on the old window, enable on the new window.
-    if (_currentWindow)
-        registerTouchWindow(_currentWindow, false);
-    if (window)
-        registerTouchWindow(window, true);
-    // Save the current window, setTouchEventsEnabledForWindow will be called
-    // with a null window on disable.
-    _currentWindow = window;
-#else // Q_OS_MAC
-    Q_UNUSED(window)
-#endif
-}
-
-
 QT_END_NAMESPACE
 
