@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -452,7 +452,6 @@ void QQuickBasePositioner::updateAttachedProperties(QQuickPositionerAttached *sp
     QQuickPositionerAttached *prevLastProperty = 0;
     QQuickPositionerAttached *lastProperty = 0;
 
-    int visibleItemIndex = 0;
     for (int ii = 0; ii < positionedItems.count(); ++ii) {
         const PositionedItem &child = positionedItems.at(ii);
         if (!child.item)
@@ -468,34 +467,53 @@ void QQuickBasePositioner::updateAttachedProperties(QQuickPositionerAttached *sp
             property = static_cast<QQuickPositionerAttached *>(qmlAttachedPropertiesObject<QQuickBasePositioner>(child.item, false));
         }
 
-        if (child.isVisible) {
-            if (property) {
-              property->setIndex(visibleItemIndex);
-              property->setIsFirstItem(visibleItemIndex == 0);
+        if (property) {
+            property->setIndex(ii);
+            property->setIsFirstItem(ii == 0);
 
-              if (property->isLastItem())
+            if (property->isLastItem()) {
+                if (prevLastProperty)
+                    prevLastProperty->setIsLastItem(false); // there can be only one last property
                 prevLastProperty = property;
             }
-
-            lastProperty = property;
-            ++visibleItemIndex;
-        } else if (property) {
-            property->setIndex(-1);
-            property->setIsFirstItem(false);
-            property->setIsLastItem(false);
         }
+
+        lastProperty = property;
     }
 
     if (prevLastProperty && prevLastProperty != lastProperty)
         prevLastProperty->setIsLastItem(false);
     if (lastProperty)
         lastProperty->setIsLastItem(true);
+
+    // clear attached properties for unpositioned items
+    for (int ii = 0; ii < unpositionedItems.count(); ++ii) {
+        const PositionedItem &child = unpositionedItems.at(ii);
+        if (!child.item)
+            continue;
+
+        QQuickPositionerAttached *property = 0;
+
+        if (specificProperty) {
+            if (specificPropertyOwner == child.item) {
+                property = specificProperty;
+            }
+        } else {
+            property = static_cast<QQuickPositionerAttached *>(qmlAttachedPropertiesObject<QQuickBasePositioner>(child.item, false));
+        }
+
+        if (property) {
+            property->setIndex(-1);
+            property->setIsFirstItem(false);
+            property->setIsLastItem(false);
+        }
+    }
 }
 
 /*!
     \qmltype Positioner
     \instantiates QQuickPositionerAttached
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-positioners
     \brief Provides attached properties that contain details on where an item exists in a positioner
 
@@ -539,7 +557,7 @@ QQuickPositionerAttached::QQuickPositionerAttached(QObject *parent) : QObject(pa
 }
 
 /*!
-    \qmlattachedproperty int QtQuick2::Positioner::index
+    \qmlattachedproperty int QtQuick::Positioner::index
 
     This property allows the item to determine
     its index within the positioner.
@@ -553,8 +571,8 @@ void QQuickPositionerAttached::setIndex(int index)
 }
 
 /*!
-    \qmlattachedproperty bool QtQuick2::Positioner::isFirstItem
-    \qmlattachedproperty bool QtQuick2::Positioner::isLastItem
+    \qmlattachedproperty bool QtQuick::Positioner::isFirstItem
+    \qmlattachedproperty bool QtQuick::Positioner::isLastItem
 
     These properties allow the item to determine if it
     is the first or last item in the positioner, respectively.
@@ -578,7 +596,7 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
 /*!
     \qmltype Column
     \instantiates QQuickColumn
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
     \brief Positions its children in a column
@@ -608,7 +626,7 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
     more information about its position within the Column.
 
     For more information on using Column and other related positioner-types, see
-    \l{Item Layouts}.
+    \l{Item Positioners}.
 
 
     \section1 Using Transitions
@@ -626,10 +644,10 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
 
     \image verticalpositioner_transition.gif
 
-    \sa Row, Grid, Flow, Positioner, {qml/positioners}{Positioners example}
+    \sa Row, Grid, Flow, Positioner, ColumnLayout, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Column::populate
+    \qmlproperty Transition QtQuick::Column::populate
 
     This property holds the transition to be run for items that are part of
     this positioner at the time of its creation. The transition is run when the positioner
@@ -639,10 +657,10 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
     the item that is being added. See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \sa add, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Column::add
+    \qmlproperty Transition QtQuick::Column::add
 
     This property holds the transition to be run for items that are added to this
     positioner. For a positioner, this applies to:
@@ -661,10 +679,10 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
     \note This transition is not applied to the items that already part of the positioner
     at the time of its creation. In this case, the \l populate transition is applied instead.
 
-    \sa populate, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa populate, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Column::move
+    \qmlproperty Transition QtQuick::Column::move
 
     This property holds the transition to run for items that have moved within the
     positioner. For a positioner, this applies to:
@@ -682,14 +700,14 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
     cases, these lists will be empty.  See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \note In QtQuick 1, this transition was applied to all items that were part of the
-    positioner at the time of its creation. From QtQuick 2 onwards, positioners apply the
+    \note In \l {Qt Quick 1}, this transition was applied to all items that were part of the
+    positioner at the time of its creation. From \l {Qt Quick}{Qt Quick 2} onwards, positioners apply the
     \l populate transition to these items instead.
 
-    \sa add, ViewTransition, {qml/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-  \qmlproperty real QtQuick2::Column::spacing
+  \qmlproperty real QtQuick::Column::spacing
 
   The spacing is the amount in pixels left empty between adjacent
   items. The default spacing is 0.
@@ -747,7 +765,7 @@ void QQuickColumn::reportConflictingAnchors()
 /*!
     \qmltype Row
     \instantiates QQuickRow
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
     \brief Positions its children in a row
@@ -777,13 +795,13 @@ void QQuickColumn::reportConflictingAnchors()
     more information about its position within the Row.
 
     For more information on using Row and other related positioner-types, see
-    \l{Item Layouts}.
+    \l{Item Positioners}.
 
 
-    \sa Column, Grid, Flow, Positioner, {qml/positioners}{Positioners example}
+    \sa Column, Grid, Flow, Positioner, RowLayout, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Row::populate
+    \qmlproperty Transition QtQuick::Row::populate
 
     This property holds the transition to be run for items that are part of
     this positioner at the time of its creation. The transition is run when the positioner
@@ -793,10 +811,10 @@ void QQuickColumn::reportConflictingAnchors()
     the item that is being added. See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \sa add, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Row::add
+    \qmlproperty Transition QtQuick::Row::add
 
     This property holds the transition to be run for items that are added to this
     positioner. For a positioner, this applies to:
@@ -815,10 +833,10 @@ void QQuickColumn::reportConflictingAnchors()
     \note This transition is not applied to the items that already part of the positioner
     at the time of its creation. In this case, the \l populate transition is applied instead.
 
-    \sa populate, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa populate, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Row::move
+    \qmlproperty Transition QtQuick::Row::move
 
     This property holds the transition to run for items that have moved within the
     positioner. For a positioner, this applies to:
@@ -836,14 +854,14 @@ void QQuickColumn::reportConflictingAnchors()
     cases, these lists will be empty.  See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \note In QtQuick 1, this transition was applied to all items that were part of the
-    positioner at the time of its creation. From QtQuick 2 onwards, positioners apply the
+    \note In \l {Qt Quick 1}, this transition was applied to all items that were part of the
+    positioner at the time of its creation. From \l {Qt Quick}{QtQuick 2} onwards, positioners apply the
     \l populate transition to these items instead.
 
-    \sa add, ViewTransition, {qml/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-  \qmlproperty real QtQuick2::Row::spacing
+  \qmlproperty real QtQuick::Row::spacing
 
   The spacing is the amount in pixels left empty between adjacent
   items. The default spacing is 0.
@@ -856,7 +874,7 @@ QQuickRow::QQuickRow(QQuickItem *parent)
 {
 }
 /*!
-    \qmlproperty enumeration QtQuick2::Row::layoutDirection
+    \qmlproperty enumeration QtQuick::Row::layoutDirection
 
     This property holds the layoutDirection of the row.
 
@@ -869,7 +887,7 @@ QQuickRow::QQuickRow(QQuickItem *parent)
     the right anchor remains to the right of the row.
     \endlist
 
-    \sa Grid::layoutDirection, Flow::layoutDirection, {qml/righttoleft/layoutdirection}{Layout directions example}
+    \sa Grid::layoutDirection, Flow::layoutDirection, {Qt Quick Examples - Right to Left}
 */
 
 Qt::LayoutDirection QQuickRow::layoutDirection() const
@@ -893,7 +911,7 @@ void QQuickRow::setLayoutDirection(Qt::LayoutDirection layoutDirection)
     }
 }
 /*!
-    \qmlproperty enumeration QtQuick2::Row::effectiveLayoutDirection
+    \qmlproperty enumeration QtQuick::Row::effectiveLayoutDirection
     This property holds the effective layout direction of the row.
 
     When using the attached property \l {LayoutMirroring::enabled}{LayoutMirroring::enabled} for locale layouts,
@@ -979,7 +997,7 @@ void QQuickRow::reportConflictingAnchors()
 /*!
     \qmltype Grid
     \instantiates QQuickGrid
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
     \brief Positions its children in grid formation
@@ -1010,13 +1028,13 @@ void QQuickRow::reportConflictingAnchors()
     or anchor itself with any of the \l {Item::anchors}{anchor} properties.
 
     For more information on using Grid and other related positioner-types, see
-    \l{Item Layouts}.
+    \l{Item Positioners}.
 
 
-    \sa Flow, Row, Column, Positioner, {qml/positioners}{Positioners example}
+    \sa Flow, Row, Column, Positioner, GridLayout, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Grid::populate
+    \qmlproperty Transition QtQuick::Grid::populate
 
     This property holds the transition to be run for items that are part of
     this positioner at the time of its creation. The transition is run when the positioner
@@ -1026,10 +1044,10 @@ void QQuickRow::reportConflictingAnchors()
     the item that is being added. See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \sa add, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Grid::add
+    \qmlproperty Transition QtQuick::Grid::add
 
     This property holds the transition to be run for items that are added to this
     positioner. For a positioner, this applies to:
@@ -1048,10 +1066,10 @@ void QQuickRow::reportConflictingAnchors()
     \note This transition is not applied to the items that already part of the positioner
     at the time of its creation. In this case, the \l populate transition is applied instead.
 
-    \sa populate, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa populate, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Grid::move
+    \qmlproperty Transition QtQuick::Grid::move
 
     This property holds the transition to run for items that have moved within the
     positioner. For a positioner, this applies to:
@@ -1069,14 +1087,14 @@ void QQuickRow::reportConflictingAnchors()
     cases, these lists will be empty.  See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \note In QtQuick 1, this transition was applied to all items that were part of the
-    positioner at the time of its creation. From QtQuick 2 onwards, positioners apply the
+    \note In \l {Qt Quick 1}, this transition was applied to all items that were part of the
+    positioner at the time of its creation. From \l {Qt Quick}{QtQuick 2} onwards, positioners apply the
     \l populate transition to these items instead.
 
-    \sa add, ViewTransition, {qml/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-  \qmlproperty qreal QtQuick2::Grid::spacing
+  \qmlproperty qreal QtQuick::Grid::spacing
 
   The spacing is the amount in pixels left empty between adjacent
   items. The amount of spacing applied will be the same in the
@@ -1102,11 +1120,13 @@ QQuickGrid::QQuickGrid(QQuickItem *parent)
     , m_useRowSpacing(false)
     , m_useColumnSpacing(false)
     , m_flow(LeftToRight)
+    , m_hItemAlign(AlignLeft)
+    , m_vItemAlign(AlignTop)
 {
 }
 
 /*!
-    \qmlproperty int QtQuick2::Grid::columns
+    \qmlproperty int QtQuick::Grid::columns
 
     This property holds the number of columns in the grid. The default
     number of columns is 4.
@@ -1116,7 +1136,7 @@ QQuickGrid::QQuickGrid(QQuickItem *parent)
 */
 
 /*!
-    \qmlproperty int QtQuick2::Grid::rows
+    \qmlproperty int QtQuick::Grid::rows
     This property holds the number of rows in the grid.
 
     If the grid does not have enough items to fill the specified
@@ -1142,7 +1162,7 @@ void QQuickGrid::setRows(const int rows)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Grid::flow
+    \qmlproperty enumeration QtQuick::Grid::flow
     This property holds the flow of the layout.
 
     Possible values are:
@@ -1169,7 +1189,7 @@ void QQuickGrid::setFlow(Flow flow)
 }
 
 /*!
-    \qmlproperty qreal QtQuick2::Grid::rowSpacing
+    \qmlproperty qreal QtQuick::Grid::rowSpacing
 
     This property holds the spacing in pixels between rows.
 
@@ -1178,7 +1198,7 @@ void QQuickGrid::setFlow(Flow flow)
     By default this property is not set.
 
     \sa columnSpacing
-    \since QtQuick2.0
+    \since 5.0
 */
 void QQuickGrid::setRowSpacing(const qreal rowSpacing)
 {
@@ -1191,7 +1211,7 @@ void QQuickGrid::setRowSpacing(const qreal rowSpacing)
 }
 
 /*!
-    \qmlproperty qreal QtQuick2::Grid::columnSpacing
+    \qmlproperty qreal QtQuick::Grid::columnSpacing
 
     This property holds the spacing in pixels between columns.
 
@@ -1200,7 +1220,7 @@ void QQuickGrid::setRowSpacing(const qreal rowSpacing)
     By default this property is not set.
 
     \sa rowSpacing
-    \since QtQuick2.0
+    \since 5.0
 */
 void QQuickGrid::setColumnSpacing(const qreal columnSpacing)
 {
@@ -1213,7 +1233,7 @@ void QQuickGrid::setColumnSpacing(const qreal columnSpacing)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Grid::layoutDirection
+    \qmlproperty enumeration QtQuick::Grid::layoutDirection
 
     This property holds the layout direction of the layout.
 
@@ -1228,7 +1248,7 @@ void QQuickGrid::setColumnSpacing(const qreal columnSpacing)
     \l Grid::flow property.
     \endlist
 
-    \sa Flow::layoutDirection, Row::layoutDirection, {qml/righttoleft/layoutdirection}{Layout directions example}
+    \sa Flow::layoutDirection, Row::layoutDirection, {Qt Quick Examples - Right to Left}
 */
 Qt::LayoutDirection QQuickGrid::layoutDirection() const
 {
@@ -1248,11 +1268,12 @@ void QQuickGrid::setLayoutDirection(Qt::LayoutDirection layoutDirection)
         prePositioning();
         emit layoutDirectionChanged();
         emit effectiveLayoutDirectionChanged();
+        emit effectiveHorizontalAlignmentChanged(effectiveHAlign());
     }
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Grid::effectiveLayoutDirection
+    \qmlproperty enumeration QtQuick::Grid::effectiveLayoutDirection
     This property holds the effective layout direction of the grid.
 
     When using the attached property \l {LayoutMirroring::enabled}{LayoutMirroring::enabled} for locale layouts,
@@ -1266,6 +1287,97 @@ Qt::LayoutDirection QQuickGrid::effectiveLayoutDirection() const
     return QQuickBasePositionerPrivate::getEffectiveLayoutDirection(this);
 }
 
+/*!
+    \qmlproperty enumeration QtQuick::Grid::horizontalItemAlignment
+    \qmlproperty enumeration QtQuick::Grid::verticalItemAlignment
+    \qmlproperty enumeration QtQuick::Grid::effectiveHorizontalItemAlignment
+
+    Sets the horizontal and vertical alignment of items in the Grid. By default,
+    the items are vertically aligned to the top. Horizontal
+    alignment follows the layoutDirection of the Grid, for example when having a layoutDirection
+    from LeftToRight, the items will be aligned on the left.
+
+    The valid values for \c horizontalItemAlignment are, \c Grid.AlignLeft, \c Grid.AlignRight and
+    \c Grid.AlignHCenter.
+
+    The valid values for \c verticalItemAlignment are \c Grid.AlignTop, \c Grid.AlignBottom
+    and \c Grid.AlignVCenter.
+
+    The below images show three examples of how to align items.
+
+    \table
+    \row
+        \li
+        \li \inlineimage gridLayout_aligntopleft.png
+        \li \inlineimage gridLayout_aligntop.png
+        \li \inlineimage gridLayout_aligncenter.png
+    \row
+        \li Horizontal alignment
+        \li AlignLeft
+        \li AlignHCenter
+        \li AlignHCenter
+    \row
+        \li Vertical alignment
+        \li AlignTop
+        \li AlignTop
+        \li AlignVCenter
+    \endtable
+
+
+    When mirroring the layout using either the attached property LayoutMirroring::enabled or
+    by setting the layoutDirection, the horizontal alignment of items will be mirrored as well.
+    However, the property \c horizontalItemAlignment will remain unchanged.
+    To query the effective horizontal alignment of items, use the read-only property
+    \c effectiveHorizontalItemAlignment.
+
+    \sa Grid::layoutDirection, {LayoutMirroring}{LayoutMirroring}
+*/
+QQuickGrid::HAlignment QQuickGrid::hItemAlign() const
+{
+    return m_hItemAlign;
+}
+void QQuickGrid::setHItemAlign(HAlignment align)
+{
+    if (m_hItemAlign != align) {
+        m_hItemAlign = align;
+        prePositioning();
+        emit horizontalAlignmentChanged(align);
+        emit effectiveHorizontalAlignmentChanged(effectiveHAlign());
+    }
+}
+
+QQuickGrid::HAlignment QQuickGrid::effectiveHAlign() const
+{
+    HAlignment effectiveAlignment = m_hItemAlign;
+    if (effectiveLayoutDirection() == Qt::RightToLeft) {
+        switch (hItemAlign()) {
+        case AlignLeft:
+            effectiveAlignment = AlignRight;
+            break;
+        case AlignRight:
+            effectiveAlignment = AlignLeft;
+            break;
+        default:
+            break;
+        }
+    }
+    return effectiveAlignment;
+}
+
+
+QQuickGrid::VAlignment QQuickGrid::vItemAlign() const
+{
+    return m_vItemAlign;
+}
+void QQuickGrid::setVItemAlign(VAlignment align)
+{
+    if (m_vItemAlign != align) {
+        m_vItemAlign = align;
+        prePositioning();
+        emit verticalAlignmentChanged(align);
+    }
+}
+
 void QQuickGrid::doPositioning(QSizeF *contentSize)
 {
     //Precondition: All items in the positioned list have a valid item pointer and should be positioned
@@ -1362,9 +1474,22 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
     for (int i = 0; i < positionedItems.count(); ++i) {
         PositionedItem &child = positionedItems[i];
         qreal childXOffset = xoffset;
+
+        if (effectiveHAlign() == AlignRight)
+            childXOffset += maxColWidth[curCol] - child.item->width();
+        else if (hItemAlign() == AlignHCenter)
+            childXOffset += (maxColWidth[curCol] - child.item->width())/2.0;
+
         if (!d->isLeftToRight())
-            childXOffset -= child.item->width();
-        positionItem(childXOffset, yoffset, &child);
+            childXOffset -= maxColWidth[curCol];
+
+        qreal alignYOffset = yoffset;
+        if (m_vItemAlign == AlignVCenter)
+            alignYOffset += (maxRowHeight[curRow] - child.item->height())/2.0;
+        else if (m_vItemAlign == AlignBottom)
+            alignYOffset += maxRowHeight[curRow] - child.item->height();
+
+        positionItem(childXOffset, alignYOffset, &child);
 
         if (m_flow == LeftToRight) {
             if (d->isLeftToRight())
@@ -1421,7 +1546,7 @@ void QQuickGrid::reportConflictingAnchors()
 /*!
     \qmltype Flow
     \instantiates QQuickFlow
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
     \brief Positions its children side by side, wrapping as necessary
@@ -1445,12 +1570,12 @@ void QQuickGrid::reportConflictingAnchors()
     or anchor itself with any of the \l {Item::anchors}{anchor} properties.
 
     For more information on using Flow and other related positioner-types, see
-    \l{Item Layouts}.
+    \l{Item Positioners}.
 
-  \sa Column, Row, Grid, Positioner, {qml/positioners}{Positioners example}
+  \sa Column, Row, Grid, Positioner, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Flow::populate
+    \qmlproperty Transition QtQuick::Flow::populate
 
     This property holds the transition to be run for items that are part of
     this positioner at the time of its creation. The transition is run when the positioner
@@ -1460,10 +1585,10 @@ void QQuickGrid::reportConflictingAnchors()
     the item that is being added. See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \sa add, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Flow::add
+    \qmlproperty Transition QtQuick::Flow::add
 
     This property holds the transition to be run for items that are added to this
     positioner. For a positioner, this applies to:
@@ -1482,10 +1607,10 @@ void QQuickGrid::reportConflictingAnchors()
     \note This transition is not applied to the items that already part of the positioner
     at the time of its creation. In this case, the \l populate transition is applied instead.
 
-    \sa populate, ViewTransition, {declarative/positioners}{Positioners example}
+    \sa populate, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-    \qmlproperty Transition QtQuick2::Flow::move
+    \qmlproperty Transition QtQuick::Flow::move
 
     This property holds the transition to run for items that have moved within the
     positioner. For a positioner, this applies to:
@@ -1503,14 +1628,14 @@ void QQuickGrid::reportConflictingAnchors()
     cases, these lists will be empty.  See the \l ViewTransition documentation for more details
     and examples on using these transitions.
 
-    \note In QtQuick 1, this transition was applied to all items that were part of the
-    positioner at the time of its creation. From QtQuick 2 onwards, positioners apply the
+    \note In \l {Qt Quick 1}, this transition was applied to all items that were part of the
+    positioner at the time of its creation. From \l {Qt Quick}{QtQuick 2} onwards, positioners apply the
     \l populate transition to these items instead.
 
-    \sa add, ViewTransition, {qml/positioners}{Positioners example}
+    \sa add, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-  \qmlproperty real QtQuick2::Flow::spacing
+  \qmlproperty real QtQuick::Flow::spacing
 
   spacing is the amount in pixels left empty between each adjacent
   item, and defaults to 0.
@@ -1539,7 +1664,7 @@ QQuickFlow::QQuickFlow(QQuickItem *parent)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Flow::flow
+    \qmlproperty enumeration QtQuick::Flow::flow
     This property holds the flow of the layout.
 
     Possible values are:
@@ -1570,7 +1695,7 @@ void QQuickFlow::setFlow(Flow flow)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Flow::layoutDirection
+    \qmlproperty enumeration QtQuick::Flow::layoutDirection
 
     This property holds the layout direction of the layout.
 
@@ -1585,7 +1710,7 @@ void QQuickFlow::setFlow(Flow flow)
     \l Flow::flow property.
     \endlist
 
-    \sa Grid::layoutDirection, Row::layoutDirection, {qml/righttoleft/layoutdirection}{Layout directions example}
+    \sa Grid::layoutDirection, Row::layoutDirection, {Qt Quick Examples - Right to Left}
 */
 
 Qt::LayoutDirection QQuickFlow::layoutDirection() const
@@ -1606,7 +1731,7 @@ void QQuickFlow::setLayoutDirection(Qt::LayoutDirection layoutDirection)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::Flow::effectiveLayoutDirection
+    \qmlproperty enumeration QtQuick::Flow::effectiveLayoutDirection
     This property holds the effective layout direction of the flow.
 
     When using the attached property \l {LayoutMirroring::enabled}{LayoutMirroring::enabled} for locale layouts,
