@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -130,7 +130,7 @@ bool QQuickPen::isValid() const
 /*!
     \qmltype GradientStop
     \instantiates QQuickGradientStop
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-visual-utility
     \brief Defines the color at a position in a Gradient
 
@@ -138,8 +138,8 @@ bool QQuickPen::isValid() const
 */
 
 /*!
-    \qmlproperty real QtQuick2::GradientStop::position
-    \qmlproperty color QtQuick2::GradientStop::color
+    \qmlproperty real QtQuick::GradientStop::position
+    \qmlproperty color QtQuick::GradientStop::color
 
     The position and color properties describe the color used at a given
     position in a gradient, as represented by a gradient stop.
@@ -182,7 +182,7 @@ void QQuickGradientStop::updateGradient()
 /*!
     \qmltype Gradient
     \instantiates QQuickGradient
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-visual-utility
     \brief Defines a gradient fill
 
@@ -232,7 +232,7 @@ void QQuickGradientStop::updateGradient()
 */
 
 /*!
-    \qmlproperty list<GradientStop> QtQuick2::Gradient::stops
+    \qmlproperty list<GradientStop> QtQuick::Gradient::stops
     \default
 
     This property holds the gradient stops describing the gradient.
@@ -265,7 +265,7 @@ int QQuickRectanglePrivate::doUpdateSlotIdx = -1;
 /*!
     \qmltype Rectangle
     \instantiates QQuickRectangle
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-visual
     \brief Paints a filled rectangle with an optional border
@@ -302,7 +302,7 @@ int QQuickRectanglePrivate::doUpdateSlotIdx = -1;
     \clearfloat
     \section1 Performance
 
-    Using the \l antialiasing property improves the appearance of a rounded rectangle at
+    Using the \l Item::antialiasing property improves the appearance of a rounded rectangle at
     the cost of rendering performance. You should consider unsetting this property
     for rectangles in motion, and only set it when they are stationary.
 
@@ -321,8 +321,9 @@ void QQuickRectangle::doUpdate()
 }
 
 /*!
-    \qmlproperty int QtQuick2::Rectangle::border.width
-    \qmlproperty color QtQuick2::Rectangle::border.color
+    \qmlpropertygroup QtQuick::Rectangle::border
+    \qmlproperty int QtQuick::Rectangle::border.width
+    \qmlproperty color QtQuick::Rectangle::border.color
 
     The width and color used to draw the border of the rectangle.
 
@@ -340,7 +341,7 @@ QQuickPen *QQuickRectangle::border()
 }
 
 /*!
-    \qmlproperty Gradient QtQuick2::Rectangle::gradient
+    \qmlproperty Gradient QtQuick::Rectangle::gradient
 
     The gradient to use to fill the rectangle.
 
@@ -388,7 +389,7 @@ void QQuickRectangle::resetGradient()
 }
 
 /*!
-    \qmlproperty real QtQuick2::Rectangle::radius
+    \qmlproperty real QtQuick::Rectangle::radius
     This property holds the corner radius used to draw a rounded rectangle.
 
     If radius is non-zero, the rectangle will be painted as a rounded rectangle, otherwise it will be
@@ -413,7 +414,7 @@ void QQuickRectangle::setRadius(qreal radius)
 }
 
 /*!
-    \qmlproperty color QtQuick2::Rectangle::color
+    \qmlproperty color QtQuick::Rectangle::color
     This property holds the color used to fill the rectangle.
 
     The default color is white.
@@ -475,7 +476,7 @@ QSGNode *QQuickRectangle::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData
     }
 
     rectangle->setRadius(d->radius);
-    rectangle->setAntialiasing(d->antialiasing);
+    rectangle->setAntialiasing(d->antialiasing || d->radius > 0);
 
     QGradientStops stops;
     if (d->gradient) {
