@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,11 +44,9 @@
 
 #include "qquickitem.h"
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-class QQuickChangeSet;
+class QQmlChangeSet;
 
 class QQuickRepeaterPrivate;
 class Q_AUTOTEST_EXPORT QQuickRepeater : public QQuickItem
@@ -91,9 +89,9 @@ protected:
     void itemChange(ItemChange change, const ItemChangeData &value);
 
 private Q_SLOTS:
-    void createdItem(int index, QQuickItem *item);
-    void initItem(int, QQuickItem *item);
-    void modelUpdated(const QQuickChangeSet &changeSet, bool reset);
+    void createdItem(int index, QObject *item);
+    void initItem(int, QObject *item);
+    void modelUpdated(const QQmlChangeSet &changeSet, bool reset);
 
 private:
     Q_DISABLE_COPY(QQuickRepeater)
@@ -104,6 +102,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickRepeater)
 
-QT_END_HEADER
-
 #endif // QQUICKREPEATER_P_H
