@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -61,7 +61,7 @@
 QT_BEGIN_NAMESPACE
 
 class QQmlContext;
-class QQuickVisualModel;
+class QQmlInstanceModel;
 class QQuickRepeaterPrivate : public QQuickItemPrivate
 {
     Q_DECLARE_PUBLIC(QQuickRepeater)
@@ -73,12 +73,13 @@ public:
 private:
     void createItems();
 
-    QQuickVisualModel *model;
+    QPointer<QQmlInstanceModel> model;
     QVariant dataSource;
-    QQmlGuard<QObject> dataSourceAsObject;
+    QPointer<QObject> dataSourceAsObject;
     bool ownModel : 1;
     bool inRequest : 1;
     bool dataSourceIsObject : 1;
+    bool delegateValidated : 1;
     int itemCount;
     int createFrom;
 
