@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -55,13 +55,12 @@
 
 #define QSG_DEBUG_FBO_OVERLAY
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QSGNode;
 class UpdatePaintNodeData;
 class QOpenGLFramebufferObject;
+class QSGSimpleRectNode;
 
 class QQuickShaderEffectSourceTextureProvider;
 
@@ -113,6 +112,8 @@ public:
     bool recursive() const { return bool(m_recursive); }
     void setRecursive(bool recursive);
 
+    void setDevicePixelRatio(qreal ratio) { m_device_pixel_ratio = ratio; }
+
     void scheduleUpdate();
 
     QImage toImage() const;
@@ -123,6 +124,7 @@ Q_SIGNALS:
 
 public Q_SLOTS:
     void markDirtyTexture();
+    void invalidated();
 
 private:
     void grab();
@@ -130,6 +132,7 @@ private:
     QSGNode *m_item;
     QRectF m_rect;
     QSize m_size;
+    qreal m_device_pixel_ratio;
     GLenum m_format;
 
     QSGRenderer *m_renderer;
@@ -137,11 +140,13 @@ private:
     QOpenGLFramebufferObject *m_secondaryFbo;
     QSharedPointer<QSGDepthStencilBuffer> m_depthStencilBuffer;
 
+    GLuint m_transparentTexture;
+
 #ifdef QSG_DEBUG_FBO_OVERLAY
-    QSGRectangleNode *m_debugOverlay;
+    QSGSimpleRectNode *m_debugOverlay;
 #endif
 
-    QSGContext *m_context;
+    QSGRenderContext *m_context;
 
     uint m_mipmap : 1;
     uint m_live : 1;
@@ -257,6 +262,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQUICKSHADEREFFECTSOURCE_P_H
