@@ -75,7 +75,6 @@
 
 // ### these should come from QStyleHints
 const int textCursorWidth = 1;
-const bool fullWidthSelection = true;
 
 QT_BEGIN_NAMESPACE
 
@@ -298,6 +297,8 @@ void QQuickTextControlPrivate::setContent(Qt::TextFormat format, const QString &
     bool previousUndoRedoState = doc->isUndoRedoEnabled();
     doc->setUndoRedoEnabled(false);
 
+    const int oldCursorPos = cursor.position();
+
     // avoid multiple textChanged() signals being emitted
     qmlobject_disconnect(doc, QTextDocument, SIGNAL(contentsChanged()), q, QQuickTextControl, SIGNAL(textChanged()));
 
@@ -341,7 +342,8 @@ void QQuickTextControlPrivate::setContent(Qt::TextFormat format, const QString &
     doc->setModified(false);
 
     q->updateCursorRectangle(true);
-    emit q->cursorPositionChanged();
+    if (cursor.position() != oldCursorPos)
+        emit q->cursorPositionChanged();
 }
 
 void QQuickTextControlPrivate::setCursorPosition(const QPointF &pos)
@@ -359,7 +361,7 @@ void QQuickTextControlPrivate::setCursorPosition(int pos, QTextCursor::MoveMode
 
     if (mode != QTextCursor::KeepAnchor) {
         selectedWordOnDoubleClick = QTextCursor();
-        selectedBlockOnTrippleClick = QTextCursor();
+        selectedBlockOnTripleClick = QTextCursor();
     }
 }
 
@@ -527,18 +529,18 @@ void QQuickTextControlPrivate::extendBlockwiseSelection(int suggestedNewPosition
     Q_Q(QQuickTextControl);
 
     // if inside the initial selected line keep that
-    if (suggestedNewPosition >= selectedBlockOnTrippleClick.selectionStart()
-        && suggestedNewPosition <= selectedBlockOnTrippleClick.selectionEnd()) {
-        q->setTextCursor(selectedBlockOnTrippleClick);
+    if (suggestedNewPosition >= selectedBlockOnTripleClick.selectionStart()
+        && suggestedNewPosition <= selectedBlockOnTripleClick.selectionEnd()) {
+        q->setTextCursor(selectedBlockOnTripleClick);
         return;
     }
 
-    if (suggestedNewPosition < selectedBlockOnTrippleClick.position()) {
-        cursor.setPosition(selectedBlockOnTrippleClick.selectionEnd());
+    if (suggestedNewPosition < selectedBlockOnTripleClick.position()) {
+        cursor.setPosition(selectedBlockOnTripleClick.selectionEnd());
         cursor.setPosition(suggestedNewPosition, QTextCursor::KeepAnchor);
         cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
     } else {
-        cursor.setPosition(selectedBlockOnTrippleClick.selectionStart());
+        cursor.setPosition(selectedBlockOnTripleClick.selectionStart());
         cursor.setPosition(suggestedNewPosition, QTextCursor::KeepAnchor);
         cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
         cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
@@ -586,6 +588,7 @@ QQuickTextControl::QQuickTextControl(QTextDocument *doc, QObject *parent)
     qmlobject_connect(doc, QTextDocument, SIGNAL(contentsChanged()), this, QQuickTextControl, SIGNAL(textChanged()));
     qmlobject_connect(doc, QTextDocument, SIGNAL(contentsChanged()), this, QQuickTextControl, SLOT(_q_updateCurrentCharFormatAndSelection()));
     qmlobject_connect(doc, QTextDocument, SIGNAL(cursorPositionChanged(QTextCursor)), this, QQuickTextControl, SLOT(_q_emitCursorPosChanged(QTextCursor)));
+    connect(doc, &QTextDocument::contentsChange, this, &QQuickTextControl::contentsChange);
 
     layout->setProperty("cursorWidth", textCursorWidth);
 
@@ -697,6 +700,9 @@ void QQuickTextControl::processEvent(QEvent *e, const QMatrix &matrix)
         case QEvent::KeyPress:
             d->keyPressEvent(static_cast<QKeyEvent *>(e));
             break;
+        case QEvent::KeyRelease:
+            d->keyReleaseEvent(static_cast<QKeyEvent *>(e));
+            break;
         case QEvent::MouseButtonPress: {
             QMouseEvent *ev = static_cast<QMouseEvent *>(e);
             d->mousePressEvent(ev, matrix.map(ev->localPos()));
@@ -713,6 +719,12 @@ void QQuickTextControl::processEvent(QEvent *e, const QMatrix &matrix)
             QMouseEvent *ev = static_cast<QMouseEvent *>(e);
             d->mouseDoubleClickEvent(ev, matrix.map(ev->localPos()));
             break; }
+        case QEvent::HoverEnter:
+        case QEvent::HoverMove:
+        case QEvent::HoverLeave: {
+            QHoverEvent *ev = static_cast<QHoverEvent *>(e);
+            d->hoverEvent(ev, matrix.map(ev->posF()));
+            break; }
 #ifndef QT_NO_IM
         case QEvent::InputMethod:
             d->inputMethodEvent(static_cast<QInputMethodEvent *>(e));
@@ -791,8 +803,8 @@ void QQuickTextControl::timerEvent(QTimerEvent *e)
         d->cursorOn = !d->cursorOn;
 
         d->repaintCursor();
-    } else if (e->timerId() == d->trippleClickTimer.timerId()) {
-        d->trippleClickTimer.stop();
+    } else if (e->timerId() == d->tripleClickTimer.timerId()) {
+        d->tripleClickTimer.stop();
     }
 }
 
@@ -808,9 +820,25 @@ void QQuickTextControl::setHtml(const QString &text)
     d->setContent(Qt::RichText, text);
 }
 
+
+void QQuickTextControlPrivate::keyReleaseEvent(QKeyEvent *e)
+{
+    if (e->key() == Qt::Key_Back) {
+         e->ignore();
+         return;
+    }
+    return;
+}
+
 void QQuickTextControlPrivate::keyPressEvent(QKeyEvent *e)
 {
     Q_Q(QQuickTextControl);
+
+    if (e->key() == Qt::Key_Back) {
+         e->ignore();
+         return;
+    }
+
 #ifndef QT_NO_SHORTCUT
     if (e == QKeySequence::SelectAll) {
             e->accept();
@@ -1022,17 +1050,17 @@ void QQuickTextControlPrivate::mousePressEvent(QMouseEvent *e, const QPointF &po
     commitPreedit();
 #endif
 
-    if (trippleClickTimer.isActive()
-        && ((pos - trippleClickPoint).toPoint().manhattanLength() < qApp->styleHints()->startDragDistance())) {
+    if (tripleClickTimer.isActive()
+        && ((pos - tripleClickPoint).toPoint().manhattanLength() < qApp->styleHints()->startDragDistance())) {
 
         cursor.movePosition(QTextCursor::StartOfBlock);
         cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
         cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
-        selectedBlockOnTrippleClick = cursor;
+        selectedBlockOnTripleClick = cursor;
 
         anchorOnMousePress = QString();
 
-        trippleClickTimer.stop();
+        tripleClickTimer.stop();
     } else {
         int cursorPos = q->hitTest(pos, Qt::FuzzyHit);
         if (cursorPos == -1) {
@@ -1046,7 +1074,7 @@ void QQuickTextControlPrivate::mousePressEvent(QMouseEvent *e, const QPointF &po
                 selectedWordOnDoubleClick.select(QTextCursor::WordUnderCursor);
             }
 
-            if (selectedBlockOnTrippleClick.hasSelection())
+            if (selectedBlockOnTripleClick.hasSelection())
                 extendBlockwiseSelection(cursorPos);
             else if (selectedWordOnDoubleClick.hasSelection())
                 extendWordwiseSelection(cursorPos, pos.x());
@@ -1079,7 +1107,7 @@ void QQuickTextControlPrivate::mouseMoveEvent(QMouseEvent *e, const QPointF &mou
         if (!(mousePressed
               || editable
               || selectedWordOnDoubleClick.hasSelection()
-              || selectedBlockOnTrippleClick.hasSelection()))
+              || selectedBlockOnTripleClick.hasSelection()))
             return;
 
         const QTextCursor oldSelection = cursor;
@@ -1114,7 +1142,7 @@ void QQuickTextControlPrivate::mouseMoveEvent(QMouseEvent *e, const QPointF &mou
             selectedWordOnDoubleClick.select(QTextCursor::WordUnderCursor);
         }
 
-        if (selectedBlockOnTrippleClick.hasSelection())
+        if (selectedBlockOnTripleClick.hasSelection())
             extendBlockwiseSelection(newCursorPos);
         else if (selectedWordOnDoubleClick.hasSelection())
             extendWordwiseSelection(newCursorPos, mouseX);
@@ -1219,8 +1247,8 @@ void QQuickTextControlPrivate::mouseDoubleClickEvent(QMouseEvent *e, const QPoin
         cursorIsFocusIndicator = false;
         selectedWordOnDoubleClick = cursor;
 
-        trippleClickPoint = pos;
-        trippleClickTimer.start(qApp->styleHints()->mouseDoubleClickInterval(), q);
+        tripleClickPoint = pos;
+        tripleClickTimer.start(qApp->styleHints()->mouseDoubleClickInterval(), q);
         if (doEmit) {
             selectionChanged();
 #ifndef QT_NO_CLIPBOARD
@@ -1380,6 +1408,20 @@ void QQuickTextControlPrivate::focusEvent(QFocusEvent *e)
     }
 }
 
+void QQuickTextControlPrivate::hoverEvent(QHoverEvent *e, const QPointF &pos)
+{
+    Q_Q(QQuickTextControl);
+
+    QString link;
+    if (e->type() != QEvent::HoverLeave)
+        link = q->anchorAt(pos);
+
+    if (hoveredLink != link) {
+        hoveredLink = link;
+        emit q->linkHovered(link);
+    }
+}
+
 bool QQuickTextControl::hasImState() const
 {
     Q_D(const QQuickTextControl);
@@ -1415,6 +1457,12 @@ QRectF QQuickTextControl::cursorRect() const
     return cursorRect(d->cursor);
 }
 
+QString QQuickTextControl::hoveredLink() const
+{
+    Q_D(const QQuickTextControl);
+    return d->hoveredLink;
+}
+
 QString QQuickTextControl::anchorAt(const QPointF &pos) const
 {
     Q_D(const QQuickTextControl);
