@@ -63,8 +63,6 @@
 #include <QtGui/qclipboard.h>
 #include <QtCore/qmimedata.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -105,6 +103,7 @@ public:
     QRectF selectionRect(const QTextCursor &cursor) const;
     QRectF selectionRect() const;
 
+    QString hoveredLink() const;
     QString anchorAt(const QPointF &pos) const;
 
     void setCursorWidth(int width);
@@ -140,6 +139,7 @@ public Q_SLOTS:
 
 Q_SIGNALS:
     void textChanged();
+    void contentsChange(int from, int charsRemoved, int charsAdded);
     void undoAvailable(bool b);
     void redoAvailable(bool b);
     void currentCharFormatChanged(const QTextCharFormat &format);
@@ -152,6 +152,7 @@ Q_SIGNALS:
     void updateRequest();
     void cursorRectangleChanged();
     void linkActivated(const QString &link);
+    void linkHovered(const QString &link);
 
 public:
     virtual void processEvent(QEvent *e, const QMatrix &matrix);
@@ -196,6 +197,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQuickTextControl_H
