@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,10 +46,9 @@
 
 #include <QtGui/qtextoption.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
+class QQuickTextDocument;
 class QQuickTextEditPrivate;
 class Q_QUICK_PRIVATE_EXPORT QQuickTextEdit : public QQuickImplicitSizeItem
 {
@@ -91,6 +90,7 @@ class Q_QUICK_PRIVATE_EXPORT QQuickTextEdit : public QQuickImplicitSizeItem
 #ifndef QT_NO_IM
     Q_PROPERTY(Qt::InputMethodHints inputMethodHints READ inputMethodHints WRITE setInputMethodHints NOTIFY inputMethodHintsChanged)
 #endif
+    Q_PROPERTY(bool selectByKeyboard READ selectByKeyboard WRITE setSelectByKeyboard NOTIFY selectByKeyboardChanged REVISION 1)
     Q_PROPERTY(bool selectByMouse READ selectByMouse WRITE setSelectByMouse NOTIFY selectByMouseChanged)
     Q_PROPERTY(SelectionMode mouseSelectionMode READ mouseSelectionMode WRITE setMouseSelectionMode NOTIFY mouseSelectionModeChanged)
     Q_PROPERTY(bool canPaste READ canPaste NOTIFY canPasteChanged)
@@ -101,6 +101,8 @@ class Q_QUICK_PRIVATE_EXPORT QQuickTextEdit : public QQuickImplicitSizeItem
 #endif
     Q_PROPERTY(QUrl baseUrl READ baseUrl WRITE setBaseUrl RESET resetBaseUrl NOTIFY baseUrlChanged)
     Q_PROPERTY(RenderType renderType READ renderType WRITE setRenderType NOTIFY renderTypeChanged)
+    Q_PROPERTY(QQuickTextDocument *textDocument READ textDocument FINAL REVISION 1)
+    Q_PROPERTY(QString hoveredLink READ hoveredLink NOTIFY linkHovered REVISION 2)
 
 public:
     QQuickTextEdit(QQuickItem *parent=0);
@@ -201,6 +203,9 @@ public:
     void setInputMethodHints(Qt::InputMethodHints hints);
 #endif
 
+    bool selectByKeyboard() const;
+    void setSelectByKeyboard(bool);
+
     bool selectByMouse() const;
     void setSelectByMouse(bool);
 
@@ -249,6 +254,10 @@ public:
     Q_INVOKABLE QString getText(int start, int end) const;
     Q_INVOKABLE QString getFormattedText(int start, int end) const;
 
+    QQuickTextDocument *textDocument();
+
+    QString hoveredLink() const;
+
 Q_SIGNALS:
     void textChanged();
     void contentSizeChanged();
@@ -272,9 +281,11 @@ Q_SIGNALS:
     void activeFocusOnPressChanged(bool activeFocusOnPressed);
     void persistentSelectionChanged(bool isPersistentSelection);
     void textMarginChanged(qreal textMargin);
+    Q_REVISION(1) void selectByKeyboardChanged(bool selectByKeyboard);
     void selectByMouseChanged(bool selectByMouse);
     void mouseSelectionModeChanged(SelectionMode mode);
     void linkActivated(const QString &link);
+    Q_REVISION(2) void linkHovered(const QString &link);
     void canPasteChanged();
     void canUndoChanged();
     void canRedoChanged();
@@ -303,20 +314,23 @@ public Q_SLOTS:
     void redo();
     void insert(int position, const QString &text);
     void remove(int start, int end);
+    Q_REVISION(2) void append(const QString &text);
 
 private Q_SLOTS:
     void q_textChanged();
-    void updateSelectionMarkers();
+    void q_contentsChange(int, int, int);
+    void updateSelection();
     void moveCursorDelegate();
     void createCursor();
     void q_canPasteChanged();
-    void updateDocument();
+    void updateWholeDocument();
     void updateCursor();
     void q_updateAlignment();
     void updateSize();
     void triggerPreprocess();
 
 private:
+    void markDirtyNodesForRange(int start, int end, int charDelta);
     void updateTotalLines();
 
 protected:
@@ -327,6 +341,11 @@ protected:
     void keyPressEvent(QKeyEvent *);
     void keyReleaseEvent(QKeyEvent *);
     void focusInEvent(QFocusEvent *event);
+    void focusOutEvent(QFocusEvent *event);
+
+    void hoverEnterEvent(QHoverEvent *event);
+    void hoverMoveEvent(QHoverEvent *event);
+    void hoverLeaveEvent(QHoverEvent *event);
 
     // mouse filter?
     void mousePressEvent(QMouseEvent *event);
@@ -336,11 +355,10 @@ protected:
 #ifndef QT_NO_IM
     void inputMethodEvent(QInputMethodEvent *e);
 #endif
-    virtual void itemChange(ItemChange, const ItemChangeData &);
-
     QSGNode *updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updatePaintNodeData);
 
     friend class QQuickTextUtil;
+    friend class QQuickTextDocument;
 
 private:
     Q_DISABLE_COPY(QQuickTextEdit)
@@ -351,6 +369,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickTextEdit)
 
-QT_END_HEADER
-
 #endif // QQUICKTEXTEDIT_P_H
