@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,8 +46,6 @@
 #include <QtGui/qtextoption.h>
 #include <QtGui/qvalidator.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickTextInputPrivate;
@@ -161,7 +159,7 @@ public:
                     };
 
     //Auxilliary functions needed to control the TextInput from QML
-    Q_INVOKABLE void positionAt(QQmlV8Function *args) const;
+    Q_INVOKABLE void positionAt(QQmlV4Function *args) const;
     Q_INVOKABLE QRectF positionToRectangle(int pos) const;
     Q_INVOKABLE void moveCursorSelection(int pos);
     Q_INVOKABLE void moveCursorSelection(int pos, SelectionMode mode);
@@ -286,6 +284,7 @@ Q_SIGNALS:
     void selectedTextChanged();
     void accepted();
     void acceptableInputChanged();
+    Q_REVISION(2) void editingFinished();
     void colorChanged();
     void selectionColorChanged();
     void selectedTextColorChanged();
@@ -334,9 +333,9 @@ protected:
 #endif
     void mouseUngrabEvent();
     bool event(QEvent *e);
+    void focusOutEvent(QFocusEvent *event);
     void focusInEvent(QFocusEvent *event);
     void timerEvent(QTimerEvent *event);
-    virtual void itemChange(ItemChange, const ItemChangeData &);
     QSGNode *updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *data);
 
 public Q_SLOTS:
@@ -415,6 +414,4 @@ QML_DECLARE_TYPE(QQuickDoubleValidator)
 QML_DECLARE_TYPE(QRegExpValidator)
 #endif
 
-QT_END_HEADER
-
 #endif // QQUICKTEXTINPUT_P_H
