@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -55,10 +55,8 @@
 
 #include "qquickitem.h"
 #include "qquickwindow.h"
-#include <private/qqmlguard_p.h>
 
 #include <QtQuick/private/qsgcontext_p.h>
-#include <private/qquickdrag_p.h>
 
 #include <QtCore/qthread.h>
 #include <QtCore/qmutex.h>
@@ -73,7 +71,9 @@ QT_BEGIN_NAMESPACE
 
 //Make it easy to identify and customize the root item if needed
 
-class QQuickWindowManager;
+class QQuickAnimatorController;
+class QSGRenderLoop;
+class QQuickDragGrabber;
 
 class QQuickRootItem : public QQuickItem
 {
@@ -117,7 +117,7 @@ public:
     QQuickItem *cursorItem;
 #endif
 #ifndef QT_NO_DRAGANDDROP
-    QQuickDragGrabber dragGrabber;
+    QQuickDragGrabber *dragGrabber;
 #endif
     int touchMouseId;
     bool checkIfDoubleClicked(ulong newPressEventTimestamp);
@@ -127,6 +127,7 @@ public:
     QPointF lastMousePosition;
     bool translateTouchToMouse(QQuickItem *item, QTouchEvent *event);
     void translateTouchEvent(QTouchEvent *touchEvent);
+    void setMouseGrabber(QQuickItem *grabber);
     static void transformTouchPoints(QList<QTouchEvent::TouchPoint> &touchPoints, const QTransform &transform);
     static QMouseEvent *cloneMouseEvent(QMouseEvent *event, QPointF *transformedLocalPos = 0);
     bool deliverInitialMousePressEvent(QQuickItem *, QMouseEvent *);
@@ -163,9 +164,10 @@ public:
     };
     Q_DECLARE_FLAGS(FocusOptions, FocusOption)
 
-    void setFocusInScope(QQuickItem *scope, QQuickItem *item, FocusOptions = 0);
-    void clearFocusInScope(QQuickItem *scope, QQuickItem *item, FocusOptions = 0);
+    void setFocusInScope(QQuickItem *scope, QQuickItem *item, Qt::FocusReason reason, FocusOptions = 0);
+    void clearFocusInScope(QQuickItem *scope, QQuickItem *item, Qt::FocusReason reason, FocusOptions = 0);
     static void notifyFocusChangesRecur(QQuickItem **item, int remaining);
+    void clearFocusObject();
 
     void updateFocusItemTransform();
 
@@ -178,9 +180,6 @@ public:
 
     bool isRenderable() const;
 
-    bool renderWithoutShowing;
-    void setRenderWithoutShowing(bool enabled);
-
     QQuickItem::UpdatePaintNodeData updatePaintNodeData;
 
     QQuickItem *dirtyItemList;
@@ -195,12 +194,13 @@ public:
     void updateEffectiveOpacityRoot(QQuickItem *, qreal);
     void updateDirtyNode(QQuickItem *);
 
-    void fireFrameSwapped() { emit q_func()->frameSwapped(); }
+    void fireFrameSwapped() { Q_EMIT q_func()->frameSwapped(); }
 
-    QSGContext *context;
+    QSGRenderContext *context;
     QSGRenderer *renderer;
 
-    QQuickWindowManager *windowManager;
+    QSGRenderLoop *windowManager;
+    QQuickAnimatorController *animationController;
 
     QColor clearColor;
 
@@ -212,6 +212,7 @@ public:
     uint persistentSceneGraph : 1;
 
     uint lastWheelEventAccepted : 1;
+    bool componentCompleted : 1;
 
     QOpenGLFramebufferObject *renderTarget;
     uint renderTargetId;
@@ -222,15 +223,40 @@ public:
 
     mutable QQuickWindowIncubationController *incubationController;
 
-    static bool dragOverThreshold(qreal d, Qt::Axis axis, QMouseEvent *event);
+    static bool defaultAlphaBuffer;
+
+    static bool dragOverThreshold(qreal d, Qt::Axis axis, QMouseEvent *event, int startDragThreshold = -1);
+
+    // data property
+    static void data_append(QQmlListProperty<QObject> *, QObject *);
+    static int data_count(QQmlListProperty<QObject> *);
+    static QObject *data_at(QQmlListProperty<QObject> *, int);
+    static void data_clear(QQmlListProperty<QObject> *);
 
 private:
     static void cleanupNodesOnShutdown(QQuickItem *);
 };
 
+class Q_QUICK_PRIVATE_EXPORT QQuickCloseEvent : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(bool accepted READ isAccepted WRITE setAccepted)
+
+public:
+    QQuickCloseEvent()
+        : _accepted(true) {}
+
+    bool isAccepted() { return _accepted; }
+    void setAccepted(bool accepted) { _accepted = accepted; }
+
+private:
+    bool _accepted;
+};
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QQuickWindowPrivate::FocusOptions)
 
 QT_END_NAMESPACE
 
+QML_DECLARE_TYPE(QQuickCloseEvent)
+
 #endif // QQUICKWINDOW_P_H
