@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -39,14 +39,12 @@
 **
 ****************************************************************************/
 
-#ifndef NODEUPDATER_P_H
-#define NODEUPDATER_P_H
+#ifndef QSGNODEUPDATER_P_H
+#define QSGNODEUPDATER_P_H
 
 #include <private/qtquickglobal_p.h>
 #include <QtGui/private/qdatabuffer_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QSGNode;
@@ -66,9 +64,6 @@ public:
     virtual void updateStates(QSGNode *n);
     virtual bool isNodeBlocked(QSGNode *n, QSGNode *root) const;
 
-    void setToplevelOpacity(qreal alpha) { m_opacity_stack.last() = alpha; }
-    qreal toplevelOpacity() const { return m_opacity_stack.last(); }
-
 protected:
     virtual void enterTransformNode(QSGTransformNode *);
     virtual void leaveTransformNode(QSGTransformNode *);
@@ -90,12 +85,8 @@ protected:
     const QSGClipNode *m_current_clip;
 
     int m_force_update;
-
-    qreal m_toplevel_alpha;
 };
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
-#endif // NODEUPDATER_P_H
+#endif
