@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -39,8 +39,8 @@
 **
 ****************************************************************************/
 
-#ifndef ADAPTATIONINTERFACES_H
-#define ADAPTATIONINTERFACES_H
+#ifndef QSGADAPTATIONLAYER_P_H
+#define QSGADAPTATIONLAYER_P_H
 
 #include <QtQuick/qsgnode.h>
 #include <QtQuick/qsgtexture.h>
@@ -59,8 +59,6 @@
 // ### remove
 #include <QtQuick/private/qquicktext_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QSGNode;
@@ -68,6 +66,7 @@ class QImage;
 class TextureReference;
 class QSGDistanceFieldGlyphCacheManager;
 class QSGDistanceFieldGlyphNode;
+class QOpenGLContext;
 
 class Q_QUICK_PRIVATE_EXPORT QSGRectangleNode : public QSGGeometryNode
 {
@@ -232,7 +231,7 @@ protected:
     };
 
     virtual void requestGlyphs(const QSet<glyph_t> &glyphs) = 0;
-    virtual void storeGlyphs(const QHash<glyph_t, QImage> &glyphs) = 0;
+    virtual void storeGlyphs(const QList<QDistanceField> &glyphs) = 0;
     virtual void referenceGlyphs(const QSet<glyph_t> &glyphs) = 0;
     virtual void releaseGlyphs(const QSet<glyph_t> &glyphs) = 0;
 
@@ -248,7 +247,7 @@ protected:
 
     GlyphData &glyphData(glyph_t glyph);
 
-    QOpenGLContext *ctx;
+    inline bool isCoreProfile() const { return m_coreProfile; }
 
 private:
     QSGDistanceFieldGlyphCacheManager *m_manager;
@@ -257,6 +256,7 @@ private:
     int m_glyphCount;
 
     bool m_doubleGlyphResolution;
+    bool m_coreProfile;
 
     QList<Texture> m_textures;
     QHash<glyph_t, GlyphData> m_glyphsData;
@@ -292,6 +292,4 @@ inline bool QSGDistanceFieldGlyphCache::containsGlyph(glyph_t glyph)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
