@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -40,20 +40,18 @@
 ****************************************************************************/
 
 
-#ifndef DEFAULT_PIXMAPNODE_H
-#define DEFAULT_PIXMAPNODE_H
+#ifndef QSGDEFAULTIMAGENODE_P_H
+#define QSGDEFAULTIMAGENODE_P_H
 
 #include <private/qsgadaptationlayer_p.h>
 #include <QtQuick/qsgtexturematerial.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-class SmoothTextureMaterial : public QSGTextureMaterial
+class Q_QUICK_PRIVATE_EXPORT QSGSmoothTextureMaterial : public QSGTextureMaterial
 {
 public:
-    SmoothTextureMaterial();
+    QSGSmoothTextureMaterial();
 
     void setTexture(QSGTexture *texture);
 
@@ -62,7 +60,7 @@ protected:
     virtual QSGMaterialShader *createShader() const;
 };
 
-class QSGDefaultImageNode : public QSGImageNode
+class Q_QUICK_PRIVATE_EXPORT QSGDefaultImageNode : public QSGImageNode
 {
 public:
     QSGDefaultImageNode();
@@ -92,7 +90,7 @@ private:
 
     QSGOpaqueTextureMaterial m_material;
     QSGTextureMaterial m_materialO;
-    SmoothTextureMaterial m_smoothMaterial;
+    QSGSmoothTextureMaterial m_smoothMaterial;
 
     uint m_antialiasing : 1;
     uint m_mirror : 1;
@@ -103,6 +101,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
