@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -39,13 +39,11 @@
 **
 ****************************************************************************/
 
-#ifndef FLATCOLORMATERIAL_H
-#define FLATCOLORMATERIAL_H
+#ifndef QSGFLATCOLORMATERIAL_H
+#define QSGFLATCOLORMATERIAL_H
 
 #include <QtQuick/qsgmaterial.h>
-#include <qcolor.h>
-
-QT_BEGIN_HEADER
+#include <QtGui/qcolor.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -67,6 +65,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // FLATCOLORMATERIAL_H
