@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -70,13 +70,9 @@ static inline int qt_next_power_of_two(int v)
 QSGPainterTexture::QSGPainterTexture()
     : QSGPlainTexture()
 {
-
+    m_retain_image = true;
 }
 
-#ifdef QT_OPENGL_ES
-extern void qsg_swizzleBGRAToRGBA(QImage *image);
-#endif
-
 void QSGPainterTexture::bind()
 {
     if (m_dirty_rect.isNull()) {
@@ -84,33 +80,8 @@ void QSGPainterTexture::bind()
         return;
     }
 
-    bool oldMipmapsGenerated = m_mipmaps_generated;
-    m_mipmaps_generated = true;
+    setImage(m_image);
     QSGPlainTexture::bind();
-    m_mipmaps_generated = oldMipmapsGenerated;
-
-    QImage subImage = m_image.copy(m_dirty_rect);
-
-    int w = m_dirty_rect.width();
-    int h = m_dirty_rect.height();
-
-#ifdef QT_OPENGL_ES
-    qsg_swizzleBGRAToRGBA(&subImage);
-    glTexSubImage2D(GL_TEXTURE_2D, 0, m_dirty_rect.x(), m_dirty_rect.y(), w, h,
-                    GL_RGBA, GL_UNSIGNED_BYTE, subImage.constBits());
-#else
-    glTexSubImage2D(GL_TEXTURE_2D, 0, m_dirty_rect.x(), m_dirty_rect.y(), w, h,
-                    GL_BGRA, GL_UNSIGNED_BYTE, subImage.constBits());
-#endif
-
-    if (m_has_mipmaps && !m_mipmaps_generated) {
-        QOpenGLContext *ctx = QOpenGLContext::currentContext();
-        ctx->functions()->glGenerateMipmap(GL_TEXTURE_2D);
-        m_mipmaps_generated = true;
-    }
-
-    m_dirty_texture = false;
-    m_dirty_bind_options = false;
 
     m_dirty_rect = QRect();
 }
@@ -140,7 +111,7 @@ QSGPainterNode::QSGPainterNode(QQuickPaintedItem *item)
     , m_dirtyRenderTarget(false)
     , m_dirtyTexture(false)
 {
-    m_context = static_cast<QQuickPaintedItemPrivate *>(QObjectPrivate::get(item))->sceneGraphContext();
+    m_context = static_cast<QQuickPaintedItemPrivate *>(QObjectPrivate::get(item))->sceneGraphRenderContext();
 
     setMaterial(&m_materialO);
     setOpaqueMaterial(&m_material);
@@ -289,7 +260,7 @@ void QSGPainterNode::updateRenderTarget()
 
     if (m_actualRenderTarget == QQuickPaintedItem::FramebufferObject ||
             m_actualRenderTarget == QQuickPaintedItem::InvertedYFramebufferObject) {
-        const QOpenGLContext *ctx = m_context->glContext();
+        const QOpenGLContext *ctx = m_context->openglContext();
         if (m_fbo && !m_dirtyGeometry && (!ctx->format().samples() || !m_multisamplingSupported))
             return;
 
@@ -352,7 +323,7 @@ void QSGPainterNode::updateFBOSize()
         fboWidth = qMax(QT_MINIMUM_DYNAMIC_FBO_SIZE, qt_next_power_of_two(m_size.width()));
         fboHeight = qMax(QT_MINIMUM_DYNAMIC_FBO_SIZE, qt_next_power_of_two(m_size.height()));
     } else {
-        QSize minimumFBOSize = m_context->minimumFBOSize();
+        QSize minimumFBOSize = m_context->sceneGraphContext()->minimumFBOSize();
         fboWidth = qMax(minimumFBOSize.width(), m_size.width());
         fboHeight = qMax(minimumFBOSize.height(), m_size.height());
     }
