@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -50,8 +50,6 @@
 
 #include <QtGui/qcolor.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QOpenGLFramebufferObject;
@@ -115,7 +113,7 @@ private:
     void updateRenderTarget();
     void updateFBOSize();
 
-    QSGContext *m_context;
+    QSGRenderContext *m_context;
 
     QQuickPaintedItem::RenderTarget m_preferredRenderTarget;
     QQuickPaintedItem::RenderTarget m_actualRenderTarget;
@@ -151,8 +149,6 @@ private:
     bool m_dirtyTexture;
 };
 
-QT_END_HEADER
-
 QT_END_NAMESPACE
 
 #endif // QSGPAINTERNODE_P_H
