@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -47,8 +47,10 @@
 #include <qthread.h>
 #include <private/qqmlprofilerservice_p.h>
 #include <private/qqmlglobal_p.h>
+#include <QtGui/qguiapplication.h>
+#include <QtGui/qpa/qplatformnativeinterface.h>
 
-#if defined(Q_OS_LINUX) && !defined(Q_OS_LINUX_ANDROID)
+#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
 #define CAN_BACKTRACE_EXECINFO
 #endif
 
@@ -65,13 +67,29 @@
 #include <QHash>
 #endif
 
+#ifndef QT_NO_DEBUG
+static bool qsg_leak_check = !qgetenv("QML_LEAK_CHECK").isEmpty();
+#endif
+
+#ifndef QSG_NO_RENDER_TIMING
+static bool qsg_render_timing = !qgetenv("QSG_RENDER_TIMING").isEmpty();
+static QElapsedTimer qsg_renderer_timer;
+#endif
+
+#ifndef GL_BGRA
+#define GL_BGRA 0x80E1
+#endif
+
+
 QT_BEGIN_NAMESPACE
 
+#if !defined(QT_NO_DEBUG) && defined(QT_OPENGL_ES_2)
 inline static bool isPowerOfTwo(int x)
 {
     // Assumption: x >= 1
     return x == (x & -x);
 }
+#endif
 
 QSGTexturePrivate::QSGTexturePrivate()
     : wrapChanged(false)
@@ -87,7 +105,7 @@ QSGTexturePrivate::QSGTexturePrivate()
 
 static int qt_debug_texture_count = 0;
 
-#if defined(Q_OS_LINUX) || defined (Q_OS_MAC)
+#if (defined(Q_OS_LINUX) || defined (Q_OS_MAC)) && !defined(Q_OS_ANDROID)
 DEFINE_BOOL_CONFIG_OPTION(qmlDebugLeakBacktrace, QML_DEBUG_LEAK_BACKTRACE)
 
 #define BACKTRACE_SIZE 20
@@ -172,7 +190,7 @@ static void qt_debug_remove_texture(QSGTexture* texture)
     --qt_debug_texture_count;
 
     if (qt_debug_texture_count < 0)
-        qDebug("Material destroyed after qt_debug_print_texture_count() was called.");
+        qDebug("Texture destroyed after qt_debug_print_texture_count() was called.");
 }
 
 #endif // QT_NO_DEBUG
@@ -208,7 +226,7 @@ static void qt_debug_remove_texture(QSGTexture* texture)
 
     \section1 Texture Atlasses
 
-    Some scene graph backens use texture atlasses, grouping multiple
+    Some scene graph backends use texture atlasses, grouping multiple
     small textures into one large texture. If this is the case, the
     function isAtlasTexture() will return true. Atlasses are used to
     aid the rendering algorithm to do better sorting which increases
@@ -218,6 +236,8 @@ static void qt_debug_remove_texture(QSGTexture* texture)
     If the texture is used in such a way that atlas is not preferable,
     the function removedFromAtlas() can be used to extract a
     non-atlassed copy.
+
+    \sa {Scene Graph - Rendering FBOs}, {Scene Graph - Rendering FBOs in a thread}
  */
 
 /*!
@@ -259,7 +279,8 @@ QSGTexture::QSGTexture()
     : QObject(*(new QSGTexturePrivate))
 {
 #ifndef QT_NO_DEBUG
-    qt_debug_add_texture(this);
+    if (qsg_leak_check)
+        qt_debug_add_texture(this);
 #endif
 }
 
@@ -269,7 +290,8 @@ QSGTexture::QSGTexture()
 QSGTexture::~QSGTexture()
 {
 #ifndef QT_NO_DEBUG
-    qt_debug_remove_texture(this);
+    if (qsg_leak_check)
+        qt_debug_remove_texture(this);
 #endif
 }
 
@@ -473,6 +495,8 @@ QSGTexture::WrapMode QSGTexture::verticalWrapMode() const
 void QSGTexture::updateBindOptions(bool force)
 {
     Q_D(QSGTexture);
+    force |= isAtlasTexture();
+
     if (force || d->filteringChanged) {
         bool linear = d->filterMode == Linear;
         GLint minFilter = linear ? GL_LINEAR : GL_NEAREST;
@@ -510,9 +534,11 @@ QSGPlainTexture::QSGPlainTexture()
     , m_texture_id(0)
     , m_has_alpha(false)
     , m_has_mipmaps(false)
+    , m_dirty_texture(false)
     , m_dirty_bind_options(false)
     , m_owns_texture(true)
     , m_mipmaps_generated(false)
+    , m_retain_image(false)
 {
 }
 
@@ -523,7 +549,6 @@ QSGPlainTexture::~QSGPlainTexture()
         glDeleteTextures(1, &m_texture_id);
 }
 
-#ifdef QT_OPENGL_ES
 void qsg_swizzleBGRAToRGBA(QImage *image)
 {
     const int width = image->width();
@@ -534,7 +559,6 @@ void qsg_swizzleBGRAToRGBA(QImage *image)
             p[x] = ((p[x] << 16) & 0xff0000) | ((p[x] >> 16) & 0xff) | (p[x] & 0xff00ff00);
     }
 }
-#endif
 
 void QSGPlainTexture::setImage(const QImage &image)
 {
@@ -596,14 +620,36 @@ void QSGPlainTexture::bind()
 
     m_dirty_texture = false;
 
+#ifndef QSG_NO_RENDER_TIMING
+    bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+    if (profileFrames)
+        qsg_renderer_timer.start();
+#endif
 
     if (m_image.isNull()) {
-        if (m_texture_id && m_owns_texture)
+        if (m_texture_id && m_owns_texture) {
             glDeleteTextures(1, &m_texture_id);
+#ifndef QSG_NO_RENDER_TIMING
+            if (qsg_render_timing) {
+                qDebug("   - texture deleted in %dms (size: %dx%d)",
+                       (int) qsg_renderer_timer.elapsed(),
+                       m_texture_size.width(),
+                       m_texture_size.height());
+            }
+            if (QQmlProfilerService::enabled) {
+                QQmlProfilerService::sceneGraphFrame(
+                            QQmlProfilerService::SceneGraphTextureDeletion,
+                            qsg_renderer_timer.nsecsElapsed());
+            }
+#endif
+        }
         m_texture_id = 0;
         m_texture_size = QSize();
         m_has_mipmaps = false;
         m_has_alpha = false;
+
+
+
         return;
     }
 
@@ -611,6 +657,12 @@ void QSGPlainTexture::bind()
         glGenTextures(1, &m_texture_id);
     glBindTexture(GL_TEXTURE_2D, m_texture_id);
 
+#ifndef QSG_NO_RENDER_TIMING
+    qint64 bindTime = 0;
+    if (profileFrames)
+        bindTime = qsg_renderer_timer.nsecsElapsed();
+#endif
+
     // ### TODO: check for out-of-memory situations...
     int w = m_image.width();
     int h = m_image.height();
@@ -618,26 +670,108 @@ void QSGPlainTexture::bind()
     QImage tmp = (m_image.format() == QImage::Format_RGB32 || m_image.format() == QImage::Format_ARGB32_Premultiplied)
                  ? m_image
                  : m_image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
+    if (tmp.width() * 4 != tmp.bytesPerLine())
+        tmp = tmp.copy();
+
+#ifndef QSG_NO_RENDER_TIMING
+    qint64 convertTime = 0;
+    if (profileFrames)
+        convertTime = qsg_renderer_timer.nsecsElapsed();
+#endif
 
     updateBindOptions(m_dirty_bind_options);
 
-#ifdef QT_OPENGL_ES
-        qsg_swizzleBGRAToRGBA(&tmp);
-        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmp.constBits());
+    GLenum externalFormat = GL_RGBA;
+    GLenum internalFormat = GL_RGBA;
+
+#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
+    QString *deviceName =
+            static_cast<QString *>(QGuiApplication::platformNativeInterface()->nativeResourceForIntegration("AndroidDeviceName"));
+    static bool wrongfullyReportsBgra8888Support = deviceName->compare(QStringLiteral("samsung SM-T211"), Qt::CaseInsensitive) == 0
+                                                || deviceName->compare(QStringLiteral("samsung SM-T210"), Qt::CaseInsensitive) == 0
+                                                || deviceName->compare(QStringLiteral("samsung SM-T215"), Qt::CaseInsensitive) == 0;
 #else
-        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_BGRA, GL_UNSIGNED_BYTE, tmp.constBits());
+    static bool wrongfullyReportsBgra8888Support = false;
 #endif
 
+    QOpenGLContext *context = QOpenGLContext::currentContext();
+    if (context->hasExtension(QByteArrayLiteral("GL_EXT_bgra"))) {
+        externalFormat = GL_BGRA;
+#ifdef QT_OPENGL_ES
+        internalFormat = GL_BGRA;
+#endif
+    } else if (!wrongfullyReportsBgra8888Support
+               && (context->hasExtension(QByteArrayLiteral("GL_EXT_texture_format_BGRA8888"))
+                   || context->hasExtension(QByteArrayLiteral("GL_IMG_texture_format_BGRA8888")))) {
+        externalFormat = GL_BGRA;
+        internalFormat = GL_BGRA;
+#ifdef Q_OS_IOS
+    } else if (context->hasExtension(QByteArrayLiteral("GL_APPLE_texture_format_BGRA8888"))) {
+        externalFormat = GL_BGRA;
+        internalFormat = GL_RGBA;
+#endif
+    } else {
+        qsg_swizzleBGRAToRGBA(&tmp);
+    }
+
+#ifndef QSG_NO_RENDER_TIMING
+    qint64 swizzleTime = 0;
+    if (profileFrames)
+        swizzleTime = qsg_renderer_timer.nsecsElapsed();
+#endif
+    glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, w, h, 0, externalFormat, GL_UNSIGNED_BYTE, tmp.constBits());
+
+#ifndef QSG_NO_RENDER_TIMING
+    qint64 uploadTime = 0;
+    if (profileFrames)
+        uploadTime = qsg_renderer_timer.nsecsElapsed();
+#endif
+
+
     if (m_has_mipmaps) {
-        QOpenGLContext *ctx = QOpenGLContext::currentContext();
-        ctx->functions()->glGenerateMipmap(GL_TEXTURE_2D);
+        context->functions()->glGenerateMipmap(GL_TEXTURE_2D);
         m_mipmaps_generated = true;
     }
 
+#ifndef QSG_NO_RENDER_TIMING
+    qint64 mipmapTime = 0;
+    if (qsg_render_timing) {
+        mipmapTime = qsg_renderer_timer.nsecsElapsed();
+
+        qDebug("   - plaintexture(%dx%d) bind=%d, convert=%d, swizzle=%d (%s->%s), upload=%d, mipmap=%d, total=%d",
+               m_texture_size.width(), m_texture_size.height(),
+               int(bindTime/1000000),
+               int((convertTime - bindTime)/1000000),
+               int((swizzleTime - convertTime)/1000000),
+               externalFormat == GL_BGRA ? "BGRA" : "RGBA",
+               internalFormat == GL_BGRA ? "BGRA" : "RGBA",
+               int((uploadTime - swizzleTime)/1000000),
+               int((mipmapTime - uploadTime)/1000000),
+               (int) qsg_renderer_timer.elapsed());
+
+    }
+
+    if (QQmlProfilerService::enabled) {
+        mipmapTime = qsg_renderer_timer.nsecsElapsed();
+
+        QQmlProfilerService::sceneGraphFrame(
+                    QQmlProfilerService::SceneGraphTexturePrepare,
+                    bindTime,
+                    convertTime - bindTime,
+                    swizzleTime - convertTime,
+                    uploadTime - swizzleTime,
+                    mipmapTime - uploadTime);
+    }
+
+#endif
+
+
     m_texture_size = QSize(w, h);
     m_texture_rect = QRectF(0, 0, 1, 1);
 
     m_dirty_bind_options = false;
+    if (!m_retain_image)
+        m_image = QImage();
 }
 
 
