@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -54,7 +54,7 @@ QT_BEGIN_NAMESPACE
 
 class QSGTexturePrivate : public QObjectPrivate
 {
-    Q_DECLARE_PUBLIC(QSGTexture);
+    Q_DECLARE_PUBLIC(QSGTexture)
 public:
     QSGTexturePrivate();
 
@@ -112,6 +112,7 @@ protected:
     uint m_dirty_bind_options : 1;
     uint m_owns_texture : 1;
     uint m_mipmaps_generated : 1;
+    uint m_retain_image: 1;
 };
 
 QT_END_NAMESPACE
