@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,42 +46,23 @@
 
 QT_BEGIN_NAMESPACE
 
+#ifdef QT_OPENGL_ES_2
 inline static bool isPowerOfTwo(int x)
 {
     // Assumption: x >= 1
     return x == (x & -x);
 }
-
-const char qt_scenegraph_texture_material_vertex_code[] =
-    "uniform highp mat4 qt_Matrix;                      \n"
-    "attribute highp vec4 qt_VertexPosition;            \n"
-    "attribute highp vec2 qt_VertexTexCoord;            \n"
-    "varying highp vec2 qt_TexCoord;                    \n"
-    "void main() {                                      \n"
-    "    qt_TexCoord = qt_VertexTexCoord;               \n"
-    "    gl_Position = qt_Matrix * qt_VertexPosition;   \n"
-    "}";
-
-const char qt_scenegraph_texture_material_fragment[] =
-    "varying highp vec2 qt_TexCoord;                    \n"
-    "uniform sampler2D qt_Texture;                      \n"
-    "void main() {                                      \n"
-    "    gl_FragColor = texture2D(qt_Texture, qt_TexCoord);\n"
-    "}";
-
-
-const char *QSGOpaqueTextureMaterialShader::vertexShader() const
-{
-    return qt_scenegraph_texture_material_vertex_code;
-}
-
-const char *QSGOpaqueTextureMaterialShader::fragmentShader() const
-{
-    return qt_scenegraph_texture_material_fragment;
-}
+#endif
 
 QSGMaterialType QSGOpaqueTextureMaterialShader::type;
 
+QSGOpaqueTextureMaterialShader::QSGOpaqueTextureMaterialShader()
+    : QSGMaterialShader()
+{
+    setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/scenegraph/shaders/opaquetexture.vert"));
+    setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/opaquetexture.frag"));
+}
+
 char const *const *QSGOpaqueTextureMaterialShader::attributeNames() const
 {
     static char const *const attr[] = { "qt_VertexPosition", "qt_VertexTexCoord", 0 };
@@ -145,7 +126,7 @@ void QSGOpaqueTextureMaterialShader::updateState(const RenderState &state, QSGMa
     tuple. The QSGGeometry::defaultAttributes_TexturedPoint2D returns an
     attribute set compatible with this material.
 
-    The texture to be rendered is can be set using setTexture(). How the
+    The texture to be rendered can be set using setTexture(). How the
     texture should be rendered can be specified using setMipmapFiltering(),
     setFiltering(), setHorizontalWrapMode() and setVerticalWrapMode().
     The rendering state is set on the texture instance just before it
@@ -208,7 +189,7 @@ QSGMaterialShader *QSGOpaqueTextureMaterial::createShader() const
 /*!
     Sets the texture of this material to \a texture.
 
-    The material does not take ownership over the texture.
+    The material does not take ownership of the texture.
  */
 
 void QSGOpaqueTextureMaterial::setTexture(QSGTexture *texture)
@@ -337,7 +318,7 @@ int QSGOpaqueTextureMaterial::compare(const QSGMaterial *o) const
     tuple. The QSGGeometry::defaultAttributes_TexturedPoint2D returns an
     attribute set compatible with this material.
 
-    The texture to be rendered is set using setTexture(). How the
+    The texture to be rendered can be set using setTexture(). How the
     texture should be rendered can be specified using setMipmapFiltering(),
     setFiltering(), setHorizontalWrapMode() and setVerticalWrapMode().
     The rendering state is set on the texture instance just before it
@@ -351,14 +332,6 @@ int QSGOpaqueTextureMaterial::compare(const QSGMaterial *o) const
     a material in the scene graph.
  */
 
-static const char qt_scenegraph_texture_material_opacity_fragment[] =
-    "varying highp vec2 qt_TexCoord;                       \n"
-    "uniform sampler2D qt_Texture;                         \n"
-    "uniform lowp float opacity;                        \n"
-    "void main() {                                      \n"
-    "    gl_FragColor = texture2D(qt_Texture, qt_TexCoord) * opacity; \n"
-    "}";
-
 QSGMaterialType QSGTextureMaterialShader::type;
 
 
@@ -383,6 +356,12 @@ QSGMaterialShader *QSGTextureMaterial::createShader() const
     return new QSGTextureMaterialShader;
 }
 
+QSGTextureMaterialShader::QSGTextureMaterialShader()
+    : QSGOpaqueTextureMaterialShader()
+{
+    setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/scenegraph/shaders/texture.frag"));
+}
+
 void QSGTextureMaterialShader::updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect)
 {
     Q_ASSERT(oldEffect == 0 || newEffect->type() == oldEffect->type());
@@ -398,9 +377,4 @@ void QSGTextureMaterialShader::initialize()
     m_opacity_id = program()->uniformLocation("opacity");
 }
 
-const char *QSGTextureMaterialShader::fragmentShader() const
-{
-    return qt_scenegraph_texture_material_opacity_fragment;
-}
-
 QT_END_NAMESPACE
