@@ -46,10 +46,11 @@
 #include <qpa/qplatformintegration.h>
 #include <QtGui/QGuiApplication>
 #include <QtCore/QDebug>
+#include <QtQml/private/qqmlglobal_p.h>
 
 QT_BEGIN_NAMESPACE
 
-class QQuickApplicationPrivate : public QObjectPrivate
+class QQuickApplicationPrivate : public QQmlApplicationPrivate
 {
     Q_DECLARE_PUBLIC(QQuickApplication)
 public:
@@ -70,10 +71,13 @@ private:
 */
 
 QQuickApplication::QQuickApplication(QObject *parent)
-    : QObject(*new QQuickApplicationPrivate(), parent)
+    : QQmlApplication(*new QQuickApplicationPrivate(), parent)
 {
     if (qApp) {
         qApp->installEventFilter(this);
+
+        connect(qApp, SIGNAL(applicationStateChanged(Qt::ApplicationState)),
+                this, SIGNAL(stateChanged(Qt::ApplicationState)));
     }
 }
 
@@ -98,17 +102,28 @@ bool QQuickApplication::supportsMultipleWindows() const
     return QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::MultipleWindows);
 }
 
+Qt::ApplicationState QQuickApplication::state() const
+{
+    return QGuiApplication::applicationState();
+}
+
 bool QQuickApplication::eventFilter(QObject *, QEvent *event)
 {
     Q_D(QQuickApplication);
     if ((event->type() == QEvent::ApplicationActivate) ||
-        (event->type() == QEvent::ApplicationDeactivate)) {
+        (event->type() == QEvent::ApplicationDeactivate) ||
+        (event->type() == QEvent::ApplicationStateChange)) {
         bool wasActive = d->isActive;
-        d->isActive = (event->type() == QEvent::ApplicationActivate);
+        if (event->type() == QEvent::ApplicationStateChange) {
+            QApplicationStateChangeEvent * e= static_cast<QApplicationStateChangeEvent*>(event);
+            d->isActive = e->applicationState() == Qt::ApplicationActive;
+        } else {
+            d->isActive = (event->type() == QEvent::ApplicationActivate);
+        }
         if (d->isActive != wasActive) {
             emit activeChanged();
         }
-    } else if (event->type() == QEvent::LayoutDirectionChange) {
+    } else if (event->type() == QEvent::ApplicationLayoutDirectionChange) {
         Qt::LayoutDirection newDirection = QGuiApplication::layoutDirection();
         if (d->direction != newDirection) {
             d->direction = newDirection;
