@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -47,8 +47,6 @@
 #include <private/qqmlpropertyvalueinterceptor_p.h>
 #include <qqml.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickAbstractAnimation;
@@ -88,6 +86,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickBehavior)
 
-QT_END_HEADER
-
 #endif // QQUICKBEHAVIOR_H
