@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,7 +52,7 @@ public:
 
 /*!
     \class QQuickTextureFactory
-    \since QtQuick 2.0
+    \since 5.0
     \brief The QQuickTextureFactory class provides an interface for loading custom textures from QML.
     \inmodule QtQuick
 
@@ -127,7 +127,7 @@ QImage QQuickTextureFactory::image() const
 
 /*!
     \class QQuickImageProvider
-    \since QtQuick 2.0
+    \since 5.0
     \inmodule QtQuick
     \brief The QQuickImageProvider class provides an interface for supporting pixmaps and threaded image requests in QML.
 
@@ -169,7 +169,7 @@ QImage QQuickTextureFactory::image() const
     an image provider named "colors", and the images to be loaded are "yellow"
     and "red", respectively:
 
-    \snippet quick/imageprovider/imageprovider-example.qml 0
+    \snippet imageprovider/imageprovider-example.qml 0
 
     When these images are loaded by QML, it looks for a matching image provider
     and calls its requestImage() or requestPixmap() method (depending on its
@@ -180,9 +180,9 @@ QImage QQuickTextureFactory::image() const
     requested by the above QML. This implementation dynamically 
     generates QPixmap images that are filled with the requested color:
 
-    \snippet quick/imageprovider/imageprovider.cpp 0
+    \snippet imageprovider/imageprovider.cpp 0
     \codeline
-    \snippet quick/imageprovider/imageprovider.cpp 1
+    \snippet imageprovider/imageprovider.cpp 1
 
     To make this provider accessible to QML, it is registered with the QML engine
     with a "colors" identifier:
@@ -204,7 +204,7 @@ QImage QQuickTextureFactory::image() const
     \image imageprovider.png
 
     A complete example is available in Qt's 
-    \l {qml/imageprovider}{examples/qml/imageprovider}
+    \l {qml/imageprovider}{examples/quick/imageprovider}
     directory. Note the example registers the provider via a \l{QQmlExtensionPlugin}{plugin}
     instead of registering it in the application \c main() function as shown above.
 
@@ -242,7 +242,7 @@ QImage QQuickTextureFactory::image() const
     \c cache property to \c false for the relevant \l Image, \l BorderImage or
     \l AnimatedImage object.
 
-    The QtQuick 1 version of this class is named QDeclarativeImageProvider.
+    The \l {Qt Quick 1} version of this class is named QDeclarativeImageProvider.
 
     \sa QQmlEngine::addImageProvider()
 */
