@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -55,7 +55,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype PathElement
     \instantiates QQuickPathElement
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief PathElement is the base path type
 
@@ -68,7 +68,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype Path
     \instantiates QQuickPath
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a path for use by \l PathView
 
@@ -93,8 +93,8 @@ QQuickPath::~QQuickPath()
 }
 
 /*!
-    \qmlproperty real QtQuick2::Path::startX
-    \qmlproperty real QtQuick2::Path::startY
+    \qmlproperty real QtQuick::Path::startX
+    \qmlproperty real QtQuick::Path::startY
     These properties hold the starting position of the path.
 */
 qreal QQuickPath::startX() const
@@ -142,7 +142,7 @@ bool QQuickPath::hasStartY() const
 }
 
 /*!
-    \qmlproperty bool QtQuick2::Path::closed
+    \qmlproperty bool QtQuick::Path::closed
     This property holds whether the start and end of the path are identical.
 */
 bool QQuickPath::isClosed() const
@@ -166,7 +166,7 @@ bool QQuickPath::hasEnd() const
 }
 
 /*!
-    \qmlproperty list<PathElement> QtQuick2::Path::pathElements
+    \qmlproperty list<PathElement> QtQuick::Path::pathElements
     This property holds the objects composing the path.
 
     \default
@@ -188,8 +188,67 @@ bool QQuickPath::hasEnd() const
 
 QQmlListProperty<QQuickPathElement> QQuickPath::pathElements()
 {
-    Q_D(QQuickPath);
-    return QQmlListProperty<QQuickPathElement>(this, d->_pathElements);
+    return QQmlListProperty<QQuickPathElement>(this,
+                                               0,
+                                               pathElements_append,
+                                               pathElements_count,
+                                               pathElements_at,
+                                               pathElements_clear);
+}
+
+static QQuickPathPrivate *privatePath(QObject *object)
+{
+    QQuickPath *path = static_cast<QQuickPath*>(object);
+
+    return QQuickPathPrivate::get(path);
+}
+
+QQuickPathElement *QQuickPath::pathElements_at(QQmlListProperty<QQuickPathElement> *property, int index)
+{
+    QQuickPathPrivate *d = privatePath(property->object);
+
+    return d->_pathElements.at(index);
+}
+
+void QQuickPath::pathElements_append(QQmlListProperty<QQuickPathElement> *property, QQuickPathElement *pathElement)
+{
+    QQuickPathPrivate *d = privatePath(property->object);
+    QQuickPath *path = static_cast<QQuickPath*>(property->object);
+
+    d->_pathElements.append(pathElement);
+
+    if (d->componentComplete) {
+        QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement);
+        if (curve)
+            d->_pathCurves.append(curve);
+        else {
+            QQuickPathAttribute *attribute = qobject_cast<QQuickPathAttribute *>(pathElement);
+            if (attribute && !d->_attributes.contains(attribute->name()))
+                d->_attributes.append(attribute->name());
+        }
+
+        path->processPath();
+
+        connect(pathElement, SIGNAL(changed()), path, SLOT(processPath()));
+    }
+}
+
+int QQuickPath::pathElements_count(QQmlListProperty<QQuickPathElement> *property)
+{
+    QQuickPathPrivate *d = privatePath(property->object);
+
+    return d->_pathElements.count();
+}
+
+void QQuickPath::pathElements_clear(QQmlListProperty<QQuickPathElement> *property)
+{
+    QQuickPathPrivate *d = privatePath(property->object);
+    QQuickPath *path = static_cast<QQuickPath*>(property->object);
+
+    path->disconnectPathElements();
+    d->_pathElements.clear();
+    d->_pathCurves.clear();
+    d->_pointCache.clear();
 }
 
 void QQuickPath::interpolate(int idx, const QString &name, qreal value)
@@ -373,29 +432,51 @@ void QQuickPath::classBegin()
     d->componentComplete = false;
 }
 
-void QQuickPath::componentComplete()
+void QQuickPath::disconnectPathElements()
 {
     Q_D(QQuickPath);
-    QSet<QString> attrs;
-    d->componentComplete = true;
 
-    // First gather up all the attributes
-    foreach (QQuickPathElement *pathElement, d->_pathElements) {
-        if (QQuickCurve *curve =
-            qobject_cast<QQuickCurve *>(pathElement))
-            d->_pathCurves.append(curve);
-        else if (QQuickPathAttribute *attribute =
-                 qobject_cast<QQuickPathAttribute *>(pathElement))
-            attrs.insert(attribute->name());
-    }
-    d->_attributes = attrs.toList();
+    foreach (QQuickPathElement *pathElement, d->_pathElements)
+        disconnect(pathElement, SIGNAL(changed()), this, SLOT(processPath()));
+}
 
-    processPath();
+void QQuickPath::connectPathElements()
+{
+    Q_D(QQuickPath);
 
     foreach (QQuickPathElement *pathElement, d->_pathElements)
         connect(pathElement, SIGNAL(changed()), this, SLOT(processPath()));
 }
 
+void QQuickPath::gatherAttributes()
+{
+    Q_D(QQuickPath);
+
+    QSet<QString> attributes;
+
+    // First gather up all the attributes
+    foreach (QQuickPathElement *pathElement, d->_pathElements) {
+        if (QQuickCurve *curve = qobject_cast<QQuickCurve *>(pathElement))
+            d->_pathCurves.append(curve);
+        else if (QQuickPathAttribute *attribute = qobject_cast<QQuickPathAttribute *>(pathElement))
+            attributes.insert(attribute->name());
+    }
+
+    d->_attributes = attributes.toList();
+}
+
+void QQuickPath::componentComplete()
+{
+    Q_D(QQuickPath);
+    d->componentComplete = true;
+
+    gatherAttributes();
+
+    processPath();
+
+    connectPathElements();
+}
+
 QPainterPath QQuickPath::path() const
 {
     Q_D(const QQuickPath);
@@ -807,7 +888,7 @@ bool QQuickCurve::hasRelativeY()
 /*!
     \qmltype PathAttribute
     \instantiates QQuickPathAttribute
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Specifies how to set an attribute at a given position in a Path
 
@@ -838,7 +919,7 @@ bool QQuickCurve::hasRelativeY()
 */
 
 /*!
-    \qmlproperty string QtQuick2::PathAttribute::name
+    \qmlproperty string QtQuick::PathAttribute::name
     This property holds the name of the attribute to change.
 
     This attribute will be available to the delegate as PathView.<name>
@@ -867,7 +948,7 @@ void QQuickPathAttribute::setName(const QString &name)
 }
 
 /*!
-   \qmlproperty real QtQuick2::PathAttribute::value
+   \qmlproperty real QtQuick::PathAttribute::value
    This property holds the value for the attribute.
 
    The value specified can be used to influence the visual appearance
@@ -924,7 +1005,7 @@ void QQuickPathAttribute::setValue(qreal value)
 /*!
     \qmltype PathLine
     \instantiates QQuickPathLine
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a straight line
 
@@ -942,8 +1023,8 @@ void QQuickPathAttribute::setValue(qreal value)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathLine::x
-    \qmlproperty real QtQuick2::PathLine::y
+    \qmlproperty real QtQuick::PathLine::x
+    \qmlproperty real QtQuick::PathLine::y
 
     Defines the end point of the line.
 
@@ -951,8 +1032,8 @@ void QQuickPathAttribute::setValue(qreal value)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathLine::relativeX
-    \qmlproperty real QtQuick2::PathLine::relativeY
+    \qmlproperty real QtQuick::PathLine::relativeX
+    \qmlproperty real QtQuick::PathLine::relativeY
 
     Defines the end point of the line relative to its start.
 
@@ -983,7 +1064,7 @@ void QQuickPathLine::addToPath(QPainterPath &path, const QQuickPathData &data)
 /*!
     \qmltype PathQuad
     \instantiates QQuickPathQuad
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a quadratic Bezier curve with a control point
 
@@ -1004,8 +1085,8 @@ void QQuickPathLine::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathQuad::x
-    \qmlproperty real QtQuick2::PathQuad::y
+    \qmlproperty real QtQuick::PathQuad::x
+    \qmlproperty real QtQuick::PathQuad::y
 
     Defines the end point of the curve.
 
@@ -1013,8 +1094,8 @@ void QQuickPathLine::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathQuad::relativeX
-    \qmlproperty real QtQuick2::PathQuad::relativeY
+    \qmlproperty real QtQuick::PathQuad::relativeX
+    \qmlproperty real QtQuick::PathQuad::relativeY
 
     Defines the end point of the curve relative to its start.
 
@@ -1028,8 +1109,8 @@ void QQuickPathLine::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-   \qmlproperty real QtQuick2::PathQuad::controlX
-   \qmlproperty real QtQuick2::PathQuad::controlY
+   \qmlproperty real QtQuick::PathQuad::controlX
+   \qmlproperty real QtQuick::PathQuad::controlY
 
    Defines the position of the control point.
 */
@@ -1070,8 +1151,8 @@ void QQuickPathQuad::setControlY(qreal y)
 }
 
 /*!
-   \qmlproperty real QtQuick2::PathQuad::relativeControlX
-   \qmlproperty real QtQuick2::PathQuad::relativeControlY
+   \qmlproperty real QtQuick::PathQuad::relativeControlX
+   \qmlproperty real QtQuick::PathQuad::relativeControlY
 
     Defines the position of the control point relative to the curve's start.
 
@@ -1135,7 +1216,7 @@ void QQuickPathQuad::addToPath(QPainterPath &path, const QQuickPathData &data)
 /*!
     \qmltype PathCubic
     \instantiates QQuickPathCubic
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a cubic Bezier curve with two control points
 
@@ -1160,8 +1241,8 @@ void QQuickPathQuad::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathCubic::x
-    \qmlproperty real QtQuick2::PathCubic::y
+    \qmlproperty real QtQuick::PathCubic::x
+    \qmlproperty real QtQuick::PathCubic::y
 
     Defines the end point of the curve.
 
@@ -1169,8 +1250,8 @@ void QQuickPathQuad::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathCubic::relativeX
-    \qmlproperty real QtQuick2::PathCubic::relativeY
+    \qmlproperty real QtQuick::PathCubic::relativeX
+    \qmlproperty real QtQuick::PathCubic::relativeY
 
     Defines the end point of the curve relative to its start.
 
@@ -1184,8 +1265,8 @@ void QQuickPathQuad::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-   \qmlproperty real QtQuick2::PathCubic::control1X
-   \qmlproperty real QtQuick2::PathCubic::control1Y
+   \qmlproperty real QtQuick::PathCubic::control1X
+   \qmlproperty real QtQuick::PathCubic::control1Y
 
     Defines the position of the first control point.
 */
@@ -1218,8 +1299,8 @@ void QQuickPathCubic::setControl1Y(qreal y)
 }
 
 /*!
-   \qmlproperty real QtQuick2::PathCubic::control2X
-   \qmlproperty real QtQuick2::PathCubic::control2Y
+   \qmlproperty real QtQuick::PathCubic::control2X
+   \qmlproperty real QtQuick::PathCubic::control2Y
 
     Defines the position of the second control point.
 */
@@ -1252,10 +1333,10 @@ void QQuickPathCubic::setControl2Y(qreal y)
 }
 
 /*!
-   \qmlproperty real QtQuick2::PathCubic::relativeControl1X
-   \qmlproperty real QtQuick2::PathCubic::relativeControl1Y
-   \qmlproperty real QtQuick2::PathCubic::relativeControl2X
-   \qmlproperty real QtQuick2::PathCubic::relativeControl2Y
+   \qmlproperty real QtQuick::PathCubic::relativeControl1X
+   \qmlproperty real QtQuick::PathCubic::relativeControl1Y
+   \qmlproperty real QtQuick::PathCubic::relativeControl2X
+   \qmlproperty real QtQuick::PathCubic::relativeControl2Y
 
     Defines the positions of the control points relative to the curve's start.
 
@@ -1359,7 +1440,7 @@ void QQuickPathCubic::addToPath(QPainterPath &path, const QQuickPathData &data)
 /*!
     \qmltype PathCurve
     \instantiates QQuickPathCatmullRomCurve
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a point on a Catmull-Rom curve
 
@@ -1377,8 +1458,8 @@ void QQuickPathCubic::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathCurve::x
-    \qmlproperty real QtQuick2::PathCurve::y
+    \qmlproperty real QtQuick::PathCurve::x
+    \qmlproperty real QtQuick::PathCurve::y
 
     Defines the end point of the curve.
 
@@ -1386,8 +1467,8 @@ void QQuickPathCubic::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathCurve::relativeX
-    \qmlproperty real QtQuick2::PathCurve::relativeY
+    \qmlproperty real QtQuick::PathCurve::relativeX
+    \qmlproperty real QtQuick::PathCurve::relativeY
 
     Defines the end point of the curve relative to its start.
 
@@ -1465,7 +1546,7 @@ void QQuickPathCatmullRomCurve::addToPath(QPainterPath &path, const QQuickPathDa
         nextData.curves = data.curves;
         next = positionForCurve(nextData, point);
     } else {
-        if (point == QPointF(path.elementAt(0)) && qobject_cast<QQuickPathCatmullRomCurve*>(data.curves.at(0))) {
+        if (point == QPointF(path.elementAt(0)) && qobject_cast<QQuickPathCatmullRomCurve*>(data.curves.at(0)) && path.elementCount() >= 3) {
             //this is a closed path starting and ending with catmull-rom segments.
             //we try to smooth the join point
             next = QPointF(path.elementAt(3));  //the first catmull-rom point
@@ -1505,7 +1586,7 @@ void QQuickPathCatmullRomCurve::addToPath(QPainterPath &path, const QQuickPathDa
 /*!
     \qmltype PathArc
     \instantiates QQuickPathArc
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines an arc with the given radius
 
@@ -1526,8 +1607,8 @@ void QQuickPathCatmullRomCurve::addToPath(QPainterPath &path, const QQuickPathDa
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathArc::x
-    \qmlproperty real QtQuick2::PathArc::y
+    \qmlproperty real QtQuick::PathArc::x
+    \qmlproperty real QtQuick::PathArc::y
 
     Defines the end point of the arc.
 
@@ -1535,8 +1616,8 @@ void QQuickPathCatmullRomCurve::addToPath(QPainterPath &path, const QQuickPathDa
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathArc::relativeX
-    \qmlproperty real QtQuick2::PathArc::relativeY
+    \qmlproperty real QtQuick::PathArc::relativeX
+    \qmlproperty real QtQuick::PathArc::relativeY
 
     Defines the end point of the arc relative to its start.
 
@@ -1550,8 +1631,8 @@ void QQuickPathCatmullRomCurve::addToPath(QPainterPath &path, const QQuickPathDa
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathArc::radiusX
-    \qmlproperty real QtQuick2::PathArc::radiusY
+    \qmlproperty real QtQuick::PathArc::radiusX
+    \qmlproperty real QtQuick::PathArc::radiusY
 
     Defines the radius of the arc.
 
@@ -1593,7 +1674,7 @@ void QQuickPathArc::setRadiusY(qreal radius)
 }
 
 /*!
-    \qmlproperty bool QtQuick2::PathArc::useLargeArc
+    \qmlproperty bool QtQuick::PathArc::useLargeArc
     Whether to use a large arc as defined by the arc points.
 
     Given fixed start and end positions, radius, and direction,
@@ -1626,7 +1707,7 @@ void QQuickPathArc::setUseLargeArc(bool largeArc)
 }
 
 /*!
-    \qmlproperty enumeration QtQuick2::PathArc::direction
+    \qmlproperty enumeration QtQuick::PathArc::direction
 
     Defines the direction of the arc. Possible values are
     PathArc.Clockwise (default) and PathArc.Counterclockwise.
@@ -1676,7 +1757,7 @@ void QQuickPathArc::addToPath(QPainterPath &path, const QQuickPathData &data)
 /*!
     \qmltype PathSvg
     \instantiates QQuickPathSvg
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Defines a path using an SVG path data string
 
@@ -1697,7 +1778,7 @@ void QQuickPathArc::addToPath(QPainterPath &path, const QQuickPathData &data)
 */
 
 /*!
-    \qmlproperty string QtQuick2::PathSvg::path
+    \qmlproperty string QtQuick::PathSvg::path
 
     The SVG path data string specifying the path.
 
@@ -1729,7 +1810,7 @@ void QQuickPathSvg::addToPath(QPainterPath &path, const QQuickPathData &)
 /*!
     \qmltype PathPercent
     \instantiates QQuickPathPercent
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \ingroup qtquick-animation-paths
     \brief Manipulates the way a path is interpreted
 
@@ -1778,7 +1859,7 @@ void QQuickPathSvg::addToPath(QPainterPath &path, const QQuickPathData &)
 */
 
 /*!
-    \qmlproperty real QtQuick2::PathPercent::value
+    \qmlproperty real QtQuick::PathPercent::value
     The proportion of items that should be laid out up to this point.
 
     This value should always be higher than the last value specified
