@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -71,6 +71,8 @@
 #include <QQmlFile>
 #include <QMetaMethod>
 
+#include <private/qqmlprofilerservice_p.h>
+
 #define IMAGEREQUEST_MAX_REQUEST_COUNT       8
 #define IMAGEREQUEST_MAX_REDIRECT_RECURSION 16
 #define CACHE_EXPIRE_TIME 30
@@ -78,6 +80,11 @@
 
 QT_BEGIN_NAMESPACE
 
+
+#ifndef QT_NO_DEBUG
+static bool qsg_leak_check = !qgetenv("QML_LEAK_CHECK").isEmpty();
+#endif
+
 // The cache limit describes the maximum "junk" in the cache.
 static int cache_limit = 2048 * 1024; // 2048 KB cache limit for embedded in qpixmapcache.cpp
 
@@ -91,6 +98,17 @@ static inline QString imageId(const QUrl &url)
     return url.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority).mid(1);
 }
 
+QQuickDefaultTextureFactory::QQuickDefaultTextureFactory(const QImage &image)
+{
+    if (image.format() == QImage::Format_ARGB32_Premultiplied
+            || image.format() == QImage::Format_RGB32) {
+        im = image;
+    } else {
+        im = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
+    }
+}
+
+
 QSGTexture *QQuickDefaultTextureFactory::createTexture(QQuickWindow *) const
 {
     QSGPlainTexture *t = new QSGPlainTexture();
@@ -501,6 +519,9 @@ void QQuickPixmapReader::processJobs()
             runningJob->loading = true;
 
             QUrl url = runningJob->url;
+            QQmlPixmapProfiler pixmapProfiler;
+            pixmapProfiler.startLoading(url);
+
             QSize requestSize = runningJob->requestSize;
             locker.unlock();
             processJob(runningJob, url, requestSize);
@@ -730,7 +751,9 @@ QQuickPixmapStore::~QQuickPixmapStore()
 {
     m_destroying = true;
 
+#ifndef QT_NO_DEBUG
     int leakedPixmaps = 0;
+#endif
     QList<QQuickPixmapData*> cachedData = m_cache.values();
 
     // Prevent unreferencePixmap() from assuming it needs to kick
@@ -742,7 +765,9 @@ QQuickPixmapStore::~QQuickPixmapStore()
     foreach (QQuickPixmapData* pixmap, cachedData) {
         int currRefCount = pixmap->refCount;
         if (currRefCount) {
+#ifndef QT_NO_DEBUG
             leakedPixmaps++;
+#endif
             while (currRefCount > 0) {
                 pixmap->release();
                 currRefCount--;
@@ -755,8 +780,10 @@ QQuickPixmapStore::~QQuickPixmapStore()
         shrinkCache(20);
     }
 
-    if (leakedPixmaps)
+#ifndef QT_NO_DEBUG
+    if (leakedPixmaps && qsg_leak_check)
         qDebug("Number of leaked pixmaps: %i", leakedPixmaps);
+#endif
 }
 
 void QQuickPixmapStore::unreferencePixmap(QQuickPixmapData *data)
@@ -868,11 +895,14 @@ bool QQuickPixmapReply::event(QEvent *event)
         if (data) {
             Event *de = static_cast<Event *>(event);
             data->pixmapStatus = (de->error == NoError) ? QQuickPixmap::Ready : QQuickPixmap::Error;
-
+            QQmlPixmapProfiler pixmapProfiler;
             if (data->pixmapStatus == QQuickPixmap::Ready) {
+                pixmapProfiler.finishLoading(data->url);
                 data->textureFactory = de->textureFactory;
                 data->implicitSize = de->implicitSize;
+                pixmapProfiler.setSize(url, data->requestSize.width() > 0 ? data->requestSize : data->implicitSize);
             } else {
+                pixmapProfiler.errorLoading(data->url);
                 data->errorString = de->errorString;
                 data->removeFromCache(); // We don't continue to cache error'd pixmaps
             }
@@ -898,6 +928,7 @@ int QQuickPixmapData::cost() const
 void QQuickPixmapData::addref()
 {
     ++refCount;
+    QQmlPixmapProfiler().referenceCountChanged(url, refCount);
     if (prevUnreferencedPtr) 
         pixmapStore()->referencePixmap(this);
 }
@@ -906,6 +937,7 @@ void QQuickPixmapData::release()
 {
     Q_ASSERT(refCount > 0);
     --refCount;
+    QQmlPixmapProfiler().referenceCountChanged(url, refCount);
     if (refCount == 0) {
         if (reply) {
             QQuickPixmapReply *cancelReply = reply;
@@ -936,6 +968,8 @@ void QQuickPixmapData::addToCache()
         QQuickPixmapKey key = { &url, &requestSize };
         pixmapStore()->m_cache.insert(key, this);
         inCache = true;
+        QQmlPixmapProfiler pixmapProfiler;
+        pixmapProfiler.cacheCountChanged(url, pixmapStore()->m_cache.count());
     }
 }
 
@@ -943,6 +977,7 @@ void QQuickPixmapData::removeFromCache()
 {
     if (inCache) {
         QQuickPixmapKey key = { &url, &requestSize };
+        QQmlPixmapProfiler().cacheCountChanged(url, pixmapStore()->m_cache.count());
         pixmapStore()->m_cache.remove(key);
         inCache = false;
     }
@@ -1205,14 +1240,20 @@ void QQuickPixmap::load(QQmlEngine *engine, const QUrl &url, const QSize &reques
 
         if (!(options & QQuickPixmap::Asynchronous)) {
             bool ok = false;
+            QQmlPixmapProfiler pixmapProfiler;
+            pixmapProfiler.startLoading(url);
             d = createPixmapDataSync(this, engine, url, requestSize, &ok);
             if (ok) {
+                pixmapProfiler.finishLoading(url);
+                pixmapProfiler.setSize(url, d->requestSize.width() > 0 ? d->requestSize : d->implicitSize);
                 if (options & QQuickPixmap::Cache)
                     d->addToCache();
                 return;
             }
-            if (d)  // loadable, but encountered error while loading
+            if (d) { // loadable, but encountered error while loading
+                pixmapProfiler.errorLoading(url);
                 return;
+            }
         } 
 
         if (!engine)
