@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -45,8 +45,6 @@
 #include "qquickstatechangescript_p.h"
 #include <private/qqmlcustomparser_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickPropertyChangesPrivate;
@@ -105,6 +103,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickPropertyChanges)
 
-QT_END_HEADER
-
 #endif // QQUICKPROPERTYCHANGES_H
