@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -48,20 +48,18 @@
 #include <QtCore/qsharedpointer.h>
 #include <private/qtquickglobal_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-class QQuickActionEvent;
+class QQuickStateActionEvent;
 class QQmlAbstractBinding;
 class QQmlBinding;
 class QQmlExpression;
-class QQuickAction
+class QQuickStateAction
 {
 public:
-    QQuickAction();
-    QQuickAction(QObject *, const QString &, const QVariant &);
-    QQuickAction(QObject *, const QString &,
+    QQuickStateAction();
+    QQuickStateAction(QObject *, const QString &, const QVariant &);
+    QQuickStateAction(QObject *, const QString &,
                        QQmlContext *, const QVariant &);
 
     bool restore:1;
@@ -75,7 +73,7 @@ public:
 
     QQmlAbstractBinding *fromBinding;
     QWeakPointer<QQmlAbstractBinding> toBinding;
-    QQuickActionEvent *event;
+    QQuickStateActionEvent *event;
 
     //strictly for matching
     QObject *specifiedObject;
@@ -84,10 +82,10 @@ public:
     void deleteFromBinding();
 };
 
-class Q_AUTOTEST_EXPORT QQuickActionEvent
+class Q_AUTOTEST_EXPORT QQuickStateActionEvent
 {
 public:
-    virtual ~QQuickActionEvent();
+    virtual ~QQuickStateActionEvent();
 
     enum EventType { Script, SignalHandler, ParentChange, AnchorChanges };
     enum Reason { ActualChange, FastForward };
@@ -99,7 +97,7 @@ public:
     virtual void reverse(Reason reason = ActualChange);
     virtual void saveOriginals() {}
     virtual bool needsCopy() { return false; }
-    virtual void copyOriginals(QQuickActionEvent *) {}
+    virtual void copyOriginals(QQuickStateActionEvent *) {}
 
     virtual bool isRewindable() { return isReversable(); }
     virtual void rewind() {}
@@ -108,7 +106,7 @@ public:
 
     virtual bool changesBindings();
     virtual void clearBindings();
-    virtual bool override(QQuickActionEvent*other);
+    virtual bool override(QQuickStateActionEvent*other);
 };
 
 //### rename to QQuickStateChange?
@@ -121,7 +119,7 @@ class Q_QUICK_PRIVATE_EXPORT QQuickStateOperation : public QObject
 public:
     QQuickStateOperation(QObject *parent = 0)
         : QObject(parent) {}
-    typedef QList<QQuickAction> ActionList;
+    typedef QList<QQuickStateAction> ActionList;
 
     virtual ActionList actions();
 
@@ -184,9 +182,9 @@ public:
     bool changeValueInRevertList(QObject *target, const QString &name, const QVariant &revertValue);
     bool changeBindingInRevertList(QObject *target, const QString &name, QQmlAbstractBinding *binding);
     bool removeEntryFromRevertList(QObject *target, const QString &name);
-    void addEntryToRevertList(const QQuickAction &action);
+    void addEntryToRevertList(const QQuickStateAction &action);
     void removeAllEntriesFromRevertList(QObject *target);
-    void addEntriesToRevertList(const QList<QQuickAction> &actions);
+    void addEntriesToRevertList(const QList<QQuickStateAction> &actions);
     QVariant valueInRevertList(QObject *target, const QString &name) const;
     QQmlAbstractBinding *bindingInRevertList(QObject *target, const QString &name) const;
 
@@ -205,6 +203,4 @@ QT_END_NAMESPACE
 QML_DECLARE_TYPE(QQuickStateOperation)
 QML_DECLARE_TYPE(QQuickState)
 
-QT_END_HEADER
-
 #endif // QQUICKSTATE_H
