@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtQml module of the Qt Toolkit.
+** This file is part of the QtQuick module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -47,8 +47,6 @@
 #include <QtCore/qobject.h>
 #include <QPalette>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickSystemPalettePrivate;
@@ -115,6 +113,4 @@ QT_END_NAMESPACE
 
 QML_DECLARE_TYPE(QQuickSystemPalette)
 
-QT_END_HEADER
-
 #endif // QQUICKSYSTEMPALETTE_H
