@@ -50,8 +50,8 @@
 #include "../../../shared/util.h"
 #include "qqmlinspectorclient.h"
 
-#define PORT 3772
-#define STR_PORT "3772"
+#define STR_PORT_FROM "3772"
+#define STR_PORT_TO "3782"
 
 
 
@@ -87,8 +87,9 @@ private slots:
 
 void tst_QQmlInspector::startQmlsceneProcess(const char * /* qmlFile */)
 {
-    const QString argument = "-qmljsdebugger=port:" STR_PORT ",block";
+    const QString argument = "-qmljsdebugger=port:" STR_PORT_FROM "," STR_PORT_TO ",block";
 
+    // ### This should be using qml instead of qmlscene, but can't because of QTBUG-33376 (same as the XFAIL testcase)
     m_process = new QQmlDebugProcess(QLibraryInfo::location(QLibraryInfo::BinariesPath) + "/qmlscene", this);
     m_process->start(QStringList() << argument << testFile("qtquick2.qml"));
     QVERIFY2(m_process->waitForSessionStart(),
@@ -97,7 +98,8 @@ void tst_QQmlInspector::startQmlsceneProcess(const char * /* qmlFile */)
     QQmlDebugConnection *m_connection = new QQmlDebugConnection();
     m_client = new QQmlInspectorClient(m_connection);
 
-    m_connection->connectToHost(QLatin1String("127.0.0.1"), PORT);
+    const int port = m_process->debugPort();
+    m_connection->connectToHost(QLatin1String("127.0.0.1"), port);
 }
 
 void tst_QQmlInspector::init()
@@ -178,7 +180,7 @@ void tst_QQmlInspector::reloadQmlWindow()
     m_client->reloadQml(changesHash);
     QVERIFY(QQmlDebugTest::waitForSignal(m_client, SIGNAL(responseReceived())));
 
-    QEXPECT_FAIL("", "cannot debug with a QML file containing a top-level Window", Abort);
+    QEXPECT_FAIL("", "cannot debug with a QML file containing a top-level Window", Abort); // QTBUG-33376
     QTRY_COMPARE(m_process->output().contains(
                      QString("version 2.0")), true);
 
