@@ -93,13 +93,21 @@ public:
 
     void start(const QStringList &arguments);
     bool waitForSessionStart();
+    int debugPort() const;
+
+    bool waitForFinished();
+    QProcess::ExitStatus exitStatus() const;
 
     QString output() const;
     void stop();
 
+signals:
+    void readyReadStandardOutput();
+
 private slots:
     void timeout();
     void processAppOutput();
+    void processError(QProcess::ProcessError error);
 
 private:
     QString m_executable;
@@ -111,6 +119,7 @@ private:
     QMutex m_mutex;
     bool m_started;
     QStringList m_environment;
+    int m_port;
 };
 
 #endif // DEBUGUTIL_H
