@@ -96,6 +96,13 @@ public slots:
     }
 };
 
+static void gc(QQmlEngine &engine)
+{
+    engine.collectGarbage();
+    QCoreApplication::sendPostedEvents(0, QEvent::DeferredDelete);
+    QCoreApplication::processEvents();
+}
+
 class tst_qqmlcomponent : public QQmlDataTest
 {
     Q_OBJECT
@@ -105,6 +112,7 @@ public:
 private slots:
     void null();
     void loadEmptyUrl();
+    void qmlCreateWindow();
     void qmlCreateObject();
     void qmlCreateObjectWithProperties();
     void qmlIncubateObject();
@@ -163,6 +171,15 @@ void tst_qqmlcomponent::qmlIncubateObject()
     delete object;
 }
 
+void tst_qqmlcomponent::qmlCreateWindow()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("createWindow.qml"));
+    QQuickWindow* window = qobject_cast<QQuickWindow *>(component.create());
+    QVERIFY(window);
+}
+
 void tst_qqmlcomponent::qmlCreateObject()
 {
     QQmlEngine engine;
@@ -367,6 +384,7 @@ void tst_qqmlcomponent::onDestructionLookup()
     QQmlEngine engine;
     QQmlComponent component(&engine, testFileUrl("onDestructionLookup.qml"));
     QScopedPointer<QObject> object(component.create());
+    gc(engine);
     QVERIFY(object != 0);
     QVERIFY(object->property("success").toBool());
 }
