@@ -72,6 +72,7 @@ private slots:
 
     void qtbug_22535();
     void evalAfterInvalidate();
+    void qobjectDerived();
 
 private:
     QQmlEngine engine;
@@ -83,9 +84,9 @@ void tst_qqmlcontext::baseUrl()
 
     QCOMPARE(ctxt.baseUrl(), QUrl());
 
-    ctxt.setBaseUrl(QUrl("http://www.nokia.com/"));
+    ctxt.setBaseUrl(QUrl("http://www.qt-project.org/"));
 
-    QCOMPARE(ctxt.baseUrl(), QUrl("http://www.nokia.com/"));
+    QCOMPARE(ctxt.baseUrl(), QUrl("http://www.qt-project.org/"));
 }
 
 void tst_qqmlcontext::resolvedUrl()
@@ -93,18 +94,18 @@ void tst_qqmlcontext::resolvedUrl()
     // Relative to the component
     {
         QQmlContext ctxt(&engine);
-        ctxt.setBaseUrl(QUrl("http://www.nokia.com/"));
+        ctxt.setBaseUrl(QUrl("http://www.qt-project.org/"));
 
-        QCOMPARE(ctxt.resolvedUrl(QUrl("main.qml")), QUrl("http://www.nokia.com/main.qml"));
+        QCOMPARE(ctxt.resolvedUrl(QUrl("main.qml")), QUrl("http://www.qt-project.org/main.qml"));
     }
 
     // Relative to a parent
     {
         QQmlContext ctxt(&engine);
-        ctxt.setBaseUrl(QUrl("http://www.nokia.com/"));
+        ctxt.setBaseUrl(QUrl("http://www.qt-project.org/"));
 
         QQmlContext ctxt2(&ctxt);
-        QCOMPARE(ctxt2.resolvedUrl(QUrl("main2.qml")), QUrl("http://www.nokia.com/main2.qml"));
+        QCOMPARE(ctxt2.resolvedUrl(QUrl("main2.qml")), QUrl("http://www.qt-project.org/main2.qml"));
     }
 
     // Relative to the engine
@@ -116,10 +117,10 @@ void tst_qqmlcontext::resolvedUrl()
     // Relative to a deleted parent
     {
         QQmlContext *ctxt = new QQmlContext(&engine);
-        ctxt->setBaseUrl(QUrl("http://www.nokia.com/"));
+        ctxt->setBaseUrl(QUrl("http://www.qt-project.org/"));
 
         QQmlContext ctxt2(ctxt);
-        QCOMPARE(ctxt2.resolvedUrl(QUrl("main2.qml")), QUrl("http://www.nokia.com/main2.qml"));
+        QCOMPARE(ctxt2.resolvedUrl(QUrl("main2.qml")), QUrl("http://www.qt-project.org/main2.qml"));
 
         delete ctxt; ctxt = 0;
 
@@ -130,7 +131,7 @@ void tst_qqmlcontext::resolvedUrl()
     {
         QQmlContext ctxt(&engine);
 
-        QCOMPARE(ctxt.resolvedUrl(QUrl("http://www.nokia.com/main2.qml")), QUrl("http://www.nokia.com/main2.qml"));
+        QCOMPARE(ctxt.resolvedUrl(QUrl("http://www.qt-project.org/main2.qml")), QUrl("http://www.qt-project.org/main2.qml"));
         QCOMPARE(ctxt.resolvedUrl(QUrl("file:///main2.qml")), QUrl("file:///main2.qml"));
     }
 }
@@ -659,6 +660,26 @@ void tst_qqmlcontext::evalAfterInvalidate()
     QCoreApplication::processEvents();
 }
 
+void tst_qqmlcontext::qobjectDerived()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("refreshExpressions.qml"));
+
+    CountCommand command;
+    // This test is similar to refreshExpressions, but with the key difference that
+    // we use the QVariant overload of setContextProperty. That way, we test that
+    // QVariant knowledge that it contains a QObject derived pointer is used.
+    engine.rootContext()->setContextProperty("countCommand", QVariant::fromValue(&command));
+
+    // We use a fresh context here to bypass any root-context optimizations in
+    // the engine
+    QQmlContext context(engine.rootContext());
+
+    QObject *o1 = component.create(&context);
+
+    QCOMPARE(command.count, 2);
+}
+
 QTEST_MAIN(tst_qqmlcontext)
 
 #include "tst_qqmlcontext.moc"
