@@ -45,10 +45,11 @@
 #include <QObject>
 #include <QQmlEngine>
 #include <QQmlComponent>
-#include <private/v8.h>
 #include <private/qqmldirparser_p.h>
 #include <QDebug>
 
+#include <algorithm>
+
 // Test the parsing of qmldir files
 
 class tst_qqmldirparser : public QQmlDataTest
@@ -105,7 +106,7 @@ namespace {
         foreach (const QQmlDirParser::Component &c, components.values())
             rv.append(toString(c));
 
-        qSort(rv);
+        std::sort(rv.begin(), rv.end());
         return rv;
     }
 
@@ -156,8 +157,7 @@ void tst_qqmldirparser::parse_data()
 
     QTest::newRow("four-sections")
         << "four-sections/qmldir"
-        << (QStringList() << "qmldir:1:12: unexpected token"
-                          << "qmldir:1: invalid qmldir directive contains too many tokens")
+        << (QStringList() << "qmldir:1: a component declaration requires two or three arguments, but 4 were provided")
         << QStringList()
         << QStringList()
         << QStringList();
@@ -199,8 +199,7 @@ void tst_qqmldirparser::parse_data()
 
     QTest::newRow("excessive-plugin")
         << "excessive-plugin/qmldir"
-        << (QStringList() << "qmldir:1:15: unexpected token"
-                          << "qmldir:1: invalid qmldir directive contains too many tokens")
+        << (QStringList() << "qmldir:1: plugin directive requires one or two arguments, but 3 were provided")
         << QStringList()
         << QStringList()
         << QStringList();
