@@ -76,7 +76,7 @@ public:
 
     void registerTypes(const char *uri)
     {
-        Q_ASSERT(QLatin1String(uri) == "com.nokia.AutoTestQmlNestedPluginType");
+        Q_ASSERT(QLatin1String(uri) == "org.qtproject.AutoTestQmlNestedPluginType");
         qmlRegisterType<MyPluginType>(uri, 1, 0, "MyPluginType");
 
         QString nestedUri(uri);
