@@ -55,6 +55,8 @@ public:
     tst_QQmlPropertyMap() {}
 
 private slots:
+    void initTestCase();
+
     void insert();
     void operatorInsert();
     void operatorValue();
@@ -66,8 +68,45 @@ private slots:
     void crashBug();
     void QTBUG_17868();
     void metaObjectAccessibility();
+    void QTBUG_31226();
+    void QTBUG_29836();
+    void QTBUG_35233();
+    void disallowExtending();
+    void QTBUG_35906();
 };
 
+class LazyPropertyMap : public QQmlPropertyMap, public QQmlParserStatus
+{
+    Q_OBJECT
+    Q_INTERFACES(QQmlParserStatus)
+
+    Q_PROPERTY(int someFixedProperty READ someFixedProperty WRITE setSomeFixedProperty NOTIFY someFixedPropertyChanged)
+public:
+    LazyPropertyMap()
+        : QQmlPropertyMap(this, /*parent*/0)
+        , value(0)
+    {}
+
+    virtual void classBegin() {}
+    virtual void componentComplete() {
+        insert(QStringLiteral("lateProperty"), QStringLiteral("lateValue"));
+    }
+
+    int someFixedProperty() const { return value; }
+    void setSomeFixedProperty(int v) { value = v; emit someFixedPropertyChanged(); }
+
+signals:
+    void someFixedPropertyChanged();
+
+private:
+    int value;
+};
+
+void tst_QQmlPropertyMap::initTestCase()
+{
+    qmlRegisterType<LazyPropertyMap>("QTBUG_35233", 1, 0, "LazyPropertyMap");
+}
+
 void tst_QQmlPropertyMap::insert()
 {
     QQmlPropertyMap map;
@@ -286,13 +325,17 @@ class MyEnhancedPropertyMap : public QQmlPropertyMap
 {
     Q_OBJECT
 public:
-    MyEnhancedPropertyMap() : QQmlPropertyMap(this, 0) {}
+    MyEnhancedPropertyMap() : QQmlPropertyMap(this, 0), m_testSlotCalled(false) {}
+    bool testSlotCalled() const { return m_testSlotCalled; }
 
 signals:
     void testSignal();
 
 public slots:
-    void testSlot() {}
+    void testSlot() { m_testSlotCalled = true; }
+
+private:
+    bool m_testSlotCalled;
 };
 
 void tst_QQmlPropertyMap::metaObjectAccessibility()
@@ -312,6 +355,126 @@ void tst_QQmlPropertyMap::metaObjectAccessibility()
     QVERIFY2(messageHandler.messages().isEmpty(), qPrintable(messageHandler.messageString()));
 }
 
+void tst_QQmlPropertyMap::QTBUG_31226()
+{
+    /* Instantiate a property map from QML, and verify that property changes
+     * made from C++ are visible from QML */
+    QQmlEngine engine;
+    QQmlContext context(&engine);
+    qmlRegisterType<QQmlPropertyMap>("QTBUG_31226", 1, 0, "PropertyMap");
+    QQmlComponent c(&engine);
+    c.setData("import QtQuick 2.0\nimport QTBUG_31226 1.0\n"
+              "Item {\n"
+              "  property string myProp\n"
+              "  PropertyMap { id: qmlPropertyMap; objectName: \"qmlPropertyMap\" }\n"
+              "  Timer { interval: 5; running: true; onTriggered: { myProp = qmlPropertyMap.greeting; } }\n"
+              "}",
+              QUrl());
+    QObject *obj = c.create(&context);
+    QVERIFY(obj);
+
+    QQmlPropertyMap *qmlPropertyMap = obj->findChild<QQmlPropertyMap*>(QString("qmlPropertyMap"));
+    QVERIFY(qmlPropertyMap);
+
+    qmlPropertyMap->insert("greeting", QString("Hello world!"));
+    QTRY_COMPARE(obj->property("myProp").toString(), QString("Hello world!"));
+
+    delete obj;
+
+}
+
+void tst_QQmlPropertyMap::QTBUG_29836()
+{
+    MyEnhancedPropertyMap map;
+    QCOMPARE(map.testSlotCalled(), false);
+
+    QQmlEngine engine;
+    QQmlContext context(&engine);
+    context.setContextProperty("enhancedMap", &map);
+    QQmlComponent c(&engine);
+    c.setData("import QtQuick 2.0\n"
+              "Item {\n"
+              "  Timer { interval: 5; running: true; onTriggered: enhancedMap.testSlot() }\n"
+              "}",
+              QUrl());
+    QObject *obj = c.create(&context);
+    QVERIFY(obj);
+
+    QTRY_COMPARE(map.testSlotCalled(), true);
+
+    delete obj;
+
+}
+
+void tst_QQmlPropertyMap::QTBUG_35233()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQml 2.0\n"
+                      "import QTBUG_35233 1.0\n"
+                      "QtObject {\n"
+                      "    property QtObject testMap: LazyPropertyMap {\n"
+                      "        id: map\n"
+                      "    }\n"
+                      "    property QtObject sibling: QtObject {\n"
+                      "        objectName: \"sibling\"\n"
+                      "        property string testValue: map.lateProperty\n"
+                      "    }\n"
+                      "}", QUrl());
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(!obj.isNull());
+
+    QObject *sibling = obj->findChild<QObject*>("sibling");
+    QVERIFY(sibling);
+    QCOMPARE(sibling->property("testValue").toString(), QString("lateValue"));
+}
+
+void tst_QQmlPropertyMap::disallowExtending()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQml 2.0\n"
+                      "import QTBUG_35233 1.0\n"
+                      "LazyPropertyMap {\n"
+                      "    id: blah\n"
+                      "    someFixedProperty: 42\n"
+                      "}\n", QUrl());
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(!obj.isNull());
+
+    component.setData("import QtQml 2.0\n"
+                      "import QTBUG_35233 1.0\n"
+                      "LazyPropertyMap {\n"
+                      "    id: blah\n"
+                      "    property int someNewProperty;\n"
+                      "}\n", QUrl());
+    obj.reset(component.create());
+    QVERIFY(obj.isNull());
+    QCOMPARE(component.errors().count(), 1);
+    QCOMPARE(component.errors().at(0).toString(), QStringLiteral("<Unknown File>: Fully dynamic types cannot declare new properties."));
+}
+
+void tst_QQmlPropertyMap::QTBUG_35906()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQml 2.0\n"
+                      "import QTBUG_35233 1.0\n"
+                      "QtObject {\n"
+                      "    property int testValue: mapById.someFixedProperty\n"
+                      "\n"
+                      "    property QtObject maProperty: LazyPropertyMap {\n"
+                      "        id: mapById\n"
+                      "        someFixedProperty: 42\n"
+                      "    }\n"
+                      "}\n", QUrl());
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(!obj.isNull());
+    QVariant value = obj->property("testValue");
+    QVERIFY(value.type() == QVariant::Int);
+    QCOMPARE(value.toInt(), 42);
+}
+
 QTEST_MAIN(tst_QQmlPropertyMap)
 
 #include "tst_qqmlpropertymap.moc"
