@@ -152,9 +152,8 @@ void tst_qquickanimatedimage::mirror_running()
 
     QQuickView window;
     window.setSource(testFileUrl("hearts.qml"));
-    window.requestActivate();
     window.show();
-    QTest::qWaitForWindowActive(&window);
+    QTest::qWaitForWindowExposed(&window);
 
     QQuickAnimatedImage *anim = qobject_cast<QQuickAnimatedImage *>(window.rootObject());
     QVERIFY(anim);
@@ -202,26 +201,28 @@ void tst_qquickanimatedimage::mirror_notRunning()
     QFETCH(QUrl, fileUrl);
 
     QQuickView window;
-    window.show();
-
     window.setSource(fileUrl);
+    window.show();
+    QTRY_VERIFY(window.isExposed());
+
     QQuickAnimatedImage *anim = qobject_cast<QQuickAnimatedImage *>(window.rootObject());
     QVERIFY(anim);
 
     int width = anim->property("width").toInt();
-    QPixmap screenshot = QPixmap::fromImage(window.grabWindow());
+    QImage screenshot = window.grabWindow();
 
     QTransform transform;
     transform.translate(width, 0).scale(-1, 1.0);
-    QPixmap expected = screenshot.transformed(transform);
+    QImage expected = screenshot.transformed(transform);
 
     int frame = anim->currentFrame();
     bool playing = anim->isPlaying();
     bool paused = anim->isPlaying();
 
     anim->setProperty("mirror", true);
-    screenshot = QPixmap::fromImage(window.grabWindow());
+    screenshot = window.grabWindow();
 
+    screenshot.save("screen.png");
     QCOMPARE(screenshot, expected);
 
     // mirroring should not change the current frame or playing status
