@@ -53,6 +53,7 @@ public:
 private slots:
     void initTestCase();
     void test_properties();
+    void test_runningChangedSignal();
     void test_frameChangedSignal();
 };
 
@@ -74,10 +75,10 @@ void tst_qquickanimatedsprite::test_properties()
     QQuickAnimatedSprite* sprite = window->rootObject()->findChild<QQuickAnimatedSprite*>("sprite");
     QVERIFY(sprite);
 
-    QVERIFY(sprite->running());
+    QTRY_VERIFY(sprite->running());
     QVERIFY(!sprite->paused());
     QVERIFY(sprite->interpolate());
-    QCOMPARE(sprite->loops(), 3);
+    QCOMPARE(sprite->loops(), 30);
 
     sprite->setRunning(false);
     QVERIFY(!sprite->running());
@@ -87,14 +88,36 @@ void tst_qquickanimatedsprite::test_properties()
     delete window;
 }
 
+void tst_qquickanimatedsprite::test_runningChangedSignal()
+{
+    QQuickView *window = new QQuickView(0);
+
+    window->setSource(testFileUrl("runningChange.qml"));
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+
+    QVERIFY(window->rootObject());
+    QQuickAnimatedSprite* sprite = window->rootObject()->findChild<QQuickAnimatedSprite*>("sprite");
+    QVERIFY(sprite);
+
+    QVERIFY(!sprite->running());
+
+    QSignalSpy runningChangedSpy(sprite, SIGNAL(runningChanged(bool)));
+    sprite->setRunning(true);
+    QTRY_COMPARE(runningChangedSpy.count(), 1);
+    QTRY_VERIFY(!sprite->running());
+    QTRY_COMPARE(runningChangedSpy.count(), 2);
+
+    delete window;
+}
+
 void tst_qquickanimatedsprite::test_frameChangedSignal()
 {
     QQuickView *window = new QQuickView(0);
 
     window->setSource(testFileUrl("frameChange.qml"));
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowActive(window));
+    QVERIFY(QTest::qWaitForWindowExposed(window));
 
     QVERIFY(window->rootObject());
     QQuickAnimatedSprite* sprite = window->rootObject()->findChild<QQuickAnimatedSprite*>("sprite");
