@@ -67,6 +67,7 @@ private slots:
     void create();
     void horizontalViewportSize();
     void verticalViewportSize();
+    void visibleAreaRatiosUpdate();
     void properties();
     void boundsBehavior();
     void rebound();
@@ -91,9 +92,14 @@ private slots:
     void cancelOnMouseGrab();
     void clickAndDragWhenTransformed();
     void flickTwiceUsingTouches();
+    void nestedStopAtBounds();
+    void nestedStopAtBounds_data();
+    void stopAtBounds();
+    void stopAtBounds_data();
+    void nestedMouseAreaUsingTouch();
 
 private:
-    void flickWithTouch(QWindow *window, QTouchDevice *touchDevice);
+    void flickWithTouch(QWindow *window, QTouchDevice *touchDevice, const QPoint &from, const QPoint &to);
     QQmlEngine engine;
 };
 
@@ -156,6 +162,28 @@ void tst_qquickflickable::verticalViewportSize()
     delete obj;
 }
 
+void tst_qquickflickable::visibleAreaRatiosUpdate()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("ratios.qml"));
+    QQuickItem *obj = qobject_cast<QQuickItem*>(c.create());
+
+    QVERIFY(obj != 0);
+    // check initial ratio values
+    QCOMPARE(obj->property("heightRatioIs").toDouble(), obj->property("heightRatioShould").toDouble());
+    QCOMPARE(obj->property("widthRatioIs").toDouble(), obj->property("widthRatioShould").toDouble());
+    // change flickable geometry so that flicking is enabled (content size > flickable size)
+    obj->setProperty("forceNoFlicking", false);
+    QCOMPARE(obj->property("heightRatioIs").toDouble(), obj->property("heightRatioShould").toDouble());
+    QCOMPARE(obj->property("widthRatioIs").toDouble(), obj->property("widthRatioShould").toDouble());
+    // change flickable geometry so that flicking is disabled (content size == flickable size)
+    obj->setProperty("forceNoFlicking", true);
+    QCOMPARE(obj->property("heightRatioIs").toDouble(), obj->property("heightRatioShould").toDouble());
+    QCOMPARE(obj->property("widthRatioIs").toDouble(), obj->property("widthRatioShould").toDouble());
+
+    delete obj;
+}
+
 void tst_qquickflickable::properties()
 {
     QQmlEngine engine;
@@ -208,10 +236,14 @@ void tst_qquickflickable::boundsBehavior()
 
 void tst_qquickflickable::rebound()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("rebound.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
+
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -227,7 +259,7 @@ void tst_qquickflickable::rebound()
     QSignalSpy hMoveSpy(flickable, SIGNAL(movingHorizontallyChanged()));
 
     // flick and test the transition is run
-    flick(window, QPoint(20,20), QPoint(120,120), 200);
+    flick(window.data(), QPoint(20,20), QPoint(120,120), 200);
 
     QTRY_COMPARE(window->rootObject()->property("transitionsStarted").toInt(), 2);
     QCOMPARE(hMoveSpy.count(), 1);
@@ -255,16 +287,10 @@ void tst_qquickflickable::rebound()
     window->rootObject()->setProperty("transitionsStarted", 0);
     window->rootObject()->setProperty("transitionsFinished", 0);
 
-#ifdef Q_OS_MAC
-    QSKIP("QTBUG-26696 - sometimes fails on Mac");
-    delete window;
-    return;
-#endif
-
     // flick and trigger the transition multiple times
     // (moving signals are emitted as soon as the first transition starts)
-    flick(window, QPoint(20,20), QPoint(120,120), 200);     // both x and y will bounce back
-    flick(window, QPoint(20,120), QPoint(120,20), 200);     // only x will bounce back
+    flick(window.data(), QPoint(20,20), QPoint(120,120), 200);     // both x and y will bounce back
+    flick(window.data(), QPoint(20,120), QPoint(120,20), 200);     // only x will bounce back
 
     QVERIFY(flickable->isMoving());
     QVERIFY(window->rootObject()->property("transitionsStarted").toInt() >= 1);
@@ -293,7 +319,7 @@ void tst_qquickflickable::rebound()
     // (i.e. moving but transition->running = false)
     window->rootObject()->setProperty("transitionEnabled", false);
 
-    flick(window, QPoint(20,20), QPoint(120,120), 200);
+    flick(window.data(), QPoint(20,20), QPoint(120,120), 200);
     QCOMPARE(window->rootObject()->property("transitionsStarted").toInt(), 0);
     QCOMPARE(hMoveSpy.count(), 1);
     QCOMPARE(vMoveSpy.count(), 1);
@@ -306,8 +332,6 @@ void tst_qquickflickable::rebound()
     QCOMPARE(movementStartedSpy.count(), 1);
     QCOMPARE(movementEndedSpy.count(), 1);
     QCOMPARE(window->rootObject()->property("transitionsStarted").toInt(), 0);
-
-    delete window;
 }
 
 void tst_qquickflickable::maximumFlickVelocity()
@@ -352,8 +376,11 @@ void tst_qquickflickable::pressDelay()
 {
     QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("pressDelay.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -384,6 +411,45 @@ void tst_qquickflickable::pressDelay()
     // On release the clicked signal should be emitted
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(150, 150));
     QCOMPARE(clickedSpy.count(),1);
+
+    // Press and release position should match
+    QCOMPARE(flickable->property("pressX").toReal(), flickable->property("releaseX").toReal());
+    QCOMPARE(flickable->property("pressY").toReal(), flickable->property("releaseY").toReal());
+
+
+    // Test a quick tap within the pressDelay timeout
+    clickedSpy.clear();
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(180, 180));
+
+    // The press should not occur immediately
+    QVERIFY(mouseArea->property("pressed").toBool() == false);
+
+    QCOMPARE(clickedSpy.count(),0);
+
+    // On release the press, release and clicked signal should be emitted
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(180, 180));
+    QCOMPARE(clickedSpy.count(),1);
+
+    // Press and release position should match
+    QCOMPARE(flickable->property("pressX").toReal(), flickable->property("releaseX").toReal());
+    QCOMPARE(flickable->property("pressY").toReal(), flickable->property("releaseY").toReal());
+
+
+    // QTBUG-31168
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(150, 110));
+
+    // The press should not occur immediately
+    QVERIFY(mouseArea->property("pressed").toBool() == false);
+
+    QTest::mouseMove(window.data(), QPoint(150, 190));
+
+    // As we moved pass the drag threshold, we should never receive the press
+    QVERIFY(mouseArea->property("pressed").toBool() == false);
+    QTRY_VERIFY(mouseArea->property("pressed").toBool() == false);
+
+    // On release the clicked signal should *not* be emitted
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(150, 190));
+    QCOMPARE(clickedSpy.count(),1);
 }
 
 // QTBUG-17361
@@ -391,8 +457,11 @@ void tst_qquickflickable::nestedPressDelay()
 {
     QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("nestedPressDelay.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *outer = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -501,7 +570,8 @@ void tst_qquickflickable::returnToBounds()
 {
     QFETCH(bool, setRebound);
 
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
+
     window->rootContext()->setContextProperty("setRebound", setRebound);
     window->setSource(testFileUrl("resize.qml"));
     QVERIFY(window->rootObject() != 0);
@@ -532,8 +602,6 @@ void tst_qquickflickable::returnToBounds()
 
     QVERIFY(!rebound->running());
     QCOMPARE(reboundSpy.count(), setRebound ? 2 : 0);
-
-    delete window;
 }
 
 void tst_qquickflickable::returnToBounds_data()
@@ -546,10 +614,10 @@ void tst_qquickflickable::returnToBounds_data()
 
 void tst_qquickflickable::wheel()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("wheel.qml"));
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flick = window->rootObject()->findChild<QQuickFlickable*>("flick");
@@ -559,7 +627,7 @@ void tst_qquickflickable::wheel()
         QPoint pos(200, 200);
         QWheelEvent event(pos, window->mapToGlobal(pos), QPoint(), QPoint(0,-120), -120, Qt::Vertical, Qt::NoButton, Qt::NoModifier);
         event.setAccepted(false);
-        QGuiApplication::sendEvent(window, &event);
+        QGuiApplication::sendEvent(window.data(), &event);
     }
 
     QTRY_VERIFY(flick->contentY() > 0);
@@ -573,13 +641,11 @@ void tst_qquickflickable::wheel()
         QWheelEvent event(pos, window->mapToGlobal(pos), QPoint(), QPoint(-120,0), -120, Qt::Horizontal, Qt::NoButton, Qt::NoModifier);
 
         event.setAccepted(false);
-        QGuiApplication::sendEvent(window, &event);
+        QGuiApplication::sendEvent(window.data(), &event);
     }
 
     QTRY_VERIFY(flick->contentX() > 0);
     QVERIFY(flick->contentY() == 0);
-
-    delete window;
 }
 
 void tst_qquickflickable::movingAndFlicking_data()
@@ -614,11 +680,13 @@ void tst_qquickflickable::movingAndFlicking()
 
     const QPoint flickFrom(50, 200);   // centre
 
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("flickable03.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowActive(window));
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -637,7 +705,7 @@ void tst_qquickflickable::movingAndFlicking()
     QSignalSpy flickEndSpy(flickable, SIGNAL(flickEnded()));
 
     // do a flick that keeps the view within the bounds
-    flick(window, flickFrom, flickToWithoutSnapBack, 200);
+    flick(window.data(), flickFrom, flickToWithoutSnapBack, 200);
 
     QTRY_VERIFY(flickable->isMoving());
     QCOMPARE(flickable->isMovingHorizontally(), horizontalEnabled);
@@ -645,6 +713,9 @@ void tst_qquickflickable::movingAndFlicking()
     QVERIFY(flickable->isFlicking());
     QCOMPARE(flickable->isFlickingHorizontally(), horizontalEnabled);
     QCOMPARE(flickable->isFlickingVertically(), verticalEnabled);
+    // contentX/contentY are either unchanged, or moving is true when the value changed.
+    QCOMPARE(flickable->property("movingInContentX").value<bool>(), true);
+    QCOMPARE(flickable->property("movingInContentY").value<bool>(), true);
 
     QCOMPARE(moveSpy.count(), 1);
     QCOMPARE(vMoveSpy.count(), verticalEnabled ? 1 : 0);
@@ -693,7 +764,7 @@ void tst_qquickflickable::movingAndFlicking()
     flickable->setContentX(0);
     flickable->setContentY(0);
     QTRY_VERIFY(!flickable->isMoving());
-    flick(window, flickFrom, flickToWithSnapBack, 200);
+    flick(window.data(), flickFrom, flickToWithSnapBack, 10);
 
     QTRY_VERIFY(flickable->isMoving());
     QCOMPARE(flickable->isMovingHorizontally(), horizontalEnabled);
@@ -737,8 +808,6 @@ void tst_qquickflickable::movingAndFlicking()
 
     QCOMPARE(flickable->contentX(), 0.0);
     QCOMPARE(flickable->contentY(), 0.0);
-
-    delete window;
 }
 
 
@@ -774,11 +843,13 @@ void tst_qquickflickable::movingAndDragging()
 
     const QPoint moveFrom(50, 200);   // centre
 
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("flickable03.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -797,10 +868,10 @@ void tst_qquickflickable::movingAndDragging()
     QSignalSpy moveEndSpy(flickable, SIGNAL(movementEnded()));
 
     // start the drag
-    QTest::mousePress(window, Qt::LeftButton, 0, moveFrom);
-    QTest::mouseMove(window, moveFrom + moveByWithoutSnapBack);
-    QTest::mouseMove(window, moveFrom + moveByWithoutSnapBack*2);
-    QTest::mouseMove(window, moveFrom + moveByWithoutSnapBack*3);
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, moveFrom);
+    QTest::mouseMove(window.data(), moveFrom + moveByWithoutSnapBack);
+    QTest::mouseMove(window.data(), moveFrom + moveByWithoutSnapBack*2);
+    QTest::mouseMove(window.data(), moveFrom + moveByWithoutSnapBack*3);
 
     QTRY_VERIFY(flickable->isMoving());
     QCOMPARE(flickable->isMovingHorizontally(), horizontalEnabled);
@@ -808,6 +879,11 @@ void tst_qquickflickable::movingAndDragging()
     QVERIFY(flickable->isDragging());
     QCOMPARE(flickable->isDraggingHorizontally(), horizontalEnabled);
     QCOMPARE(flickable->isDraggingVertically(), verticalEnabled);
+    // contentX/contentY are either unchanged, or moving and dragging are true when the value changes.
+    QCOMPARE(flickable->property("movingInContentX").value<bool>(), true);
+    QCOMPARE(flickable->property("movingInContentY").value<bool>(), true);
+    QCOMPARE(flickable->property("draggingInContentX").value<bool>(), true);
+    QCOMPARE(flickable->property("draggingInContentY").value<bool>(), true);
 
     QCOMPARE(moveSpy.count(), 1);
     QCOMPARE(vMoveSpy.count(), verticalEnabled ? 1 : 0);
@@ -819,7 +895,7 @@ void tst_qquickflickable::movingAndDragging()
     QCOMPARE(moveStartSpy.count(), 1);
     QCOMPARE(dragStartSpy.count(), 1);
 
-    QTest::mouseRelease(window, Qt::LeftButton, 0, moveFrom + moveByWithoutSnapBack*3);
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, moveFrom + moveByWithoutSnapBack*3);
 
     QVERIFY(!flickable->isDragging());
     QVERIFY(!flickable->isDraggingHorizontally());
@@ -868,10 +944,10 @@ void tst_qquickflickable::movingAndDragging()
      flickable->setContentX(0);
      flickable->setContentY(0);
      QTRY_VERIFY(!flickable->isMoving());
-     QTest::mousePress(window, Qt::LeftButton, 0, moveFrom);
-     QTest::mouseMove(window, moveFrom + moveByWithSnapBack);
-     QTest::mouseMove(window, moveFrom + moveByWithSnapBack*2);
-     QTest::mouseMove(window, moveFrom + moveByWithSnapBack*3);
+     QTest::mousePress(window.data(), Qt::LeftButton, 0, moveFrom);
+     QTest::mouseMove(window.data(), moveFrom + moveByWithSnapBack);
+     QTest::mouseMove(window.data(), moveFrom + moveByWithSnapBack*2);
+     QTest::mouseMove(window.data(), moveFrom + moveByWithSnapBack*3);
 
      QVERIFY(flickable->isMoving());
      QCOMPARE(flickable->isMovingHorizontally(), horizontalEnabled);
@@ -892,7 +968,7 @@ void tst_qquickflickable::movingAndDragging()
      QCOMPARE(dragStartSpy.count(), 1);
      QCOMPARE(dragEndSpy.count(), 0);
 
-     QTest::mouseRelease(window, Qt::LeftButton, 0, moveFrom + moveByWithSnapBack*3);
+     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, moveFrom + moveByWithSnapBack*3);
 
      // should now start snapping back to bounds (moving but not dragging)
      QVERIFY(flickable->isMoving());
@@ -935,17 +1011,14 @@ void tst_qquickflickable::movingAndDragging()
 
      QCOMPARE(flickable->contentX(), 0.0);
      QCOMPARE(flickable->contentY(), 0.0);
-
-    delete window;
 }
 
 void tst_qquickflickable::flickOnRelease()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("flickable03.qml"));
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -958,9 +1031,9 @@ void tst_qquickflickable::flickOnRelease()
     // underlying drivers will hopefully provide a pre-calculated velocity
     // (based on more data than what the UI gets), thus making this use case
     // working even with small movements.
-    QTest::mousePress(window, Qt::LeftButton, 0, QPoint(50, 300));
-    QTest::mouseMove(window, QPoint(50, 10), 10);
-    QTest::mouseRelease(window, Qt::LeftButton, 0, QPoint(50, 10), 10);
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(50, 300));
+    QTest::mouseMove(window.data(), QPoint(50, 10), 10);
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(50, 10), 10);
 
     QCOMPARE(vFlickSpy.count(), 1);
 
@@ -972,17 +1045,17 @@ void tst_qquickflickable::flickOnRelease()
 #endif
     // Stop on a full pixel after user interaction
     QCOMPARE(flickable->contentY(), (qreal)qRound(flickable->contentY()));
-
-    delete window;
 }
 
 void tst_qquickflickable::pressWhileFlicking()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("flickable03.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
@@ -997,7 +1070,7 @@ void tst_qquickflickable::pressWhileFlicking()
 
     // flick then press while it is still moving
     // flicking == false, moving == true;
-    flick(window, QPoint(20,190), QPoint(20, 50), 200);
+    flick(window.data(), QPoint(20,190), QPoint(20, 50), 200);
     QVERIFY(flickable->verticalVelocity() > 0.0);
     QTRY_VERIFY(flickable->isFlicking());
     QVERIFY(flickable->isFlickingVertically());
@@ -1012,77 +1085,75 @@ void tst_qquickflickable::pressWhileFlicking()
     QCOMPARE(hFlickSpy.count(), 0);
     QCOMPARE(flickSpy.count(), 1);
 
-    QTest::mousePress(window, Qt::LeftButton, 0, QPoint(20, 50));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(20, 50));
     QTRY_VERIFY(!flickable->isFlicking());
     QVERIFY(!flickable->isFlickingVertically());
     QVERIFY(flickable->isMoving());
     QVERIFY(flickable->isMovingVertically());
 
-    QTest::mouseRelease(window, Qt::LeftButton, 0, QPoint(20,50));
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(20,50));
     QVERIFY(!flickable->isFlicking());
     QVERIFY(!flickable->isFlickingVertically());
     QTRY_VERIFY(!flickable->isMoving());
     QVERIFY(!flickable->isMovingVertically());
     // Stop on a full pixel after user interaction
     QCOMPARE(flickable->contentX(), (qreal)qRound(flickable->contentX()));
-
-    delete window;
 }
 
 void tst_qquickflickable::disabled()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("disabled.qml"));
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flick = window->rootObject()->findChild<QQuickFlickable*>("flickable");
     QVERIFY(flick != 0);
 
-    QTest::mousePress(window, Qt::LeftButton, 0, QPoint(50, 90));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(50, 90));
 
-    QTest::mouseMove(window, QPoint(50, 80));
-    QTest::mouseMove(window, QPoint(50, 70));
-    QTest::mouseMove(window, QPoint(50, 60));
+    QTest::mouseMove(window.data(), QPoint(50, 80));
+    QTest::mouseMove(window.data(), QPoint(50, 70));
+    QTest::mouseMove(window.data(), QPoint(50, 60));
 
     QVERIFY(flick->isMoving() == false);
 
-    QTest::mouseRelease(window, Qt::LeftButton, 0, QPoint(50, 60));
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(50, 60));
 
     // verify that mouse clicks on other elements still work (QTBUG-20584)
-    QTest::mousePress(window, Qt::LeftButton, 0, QPoint(50, 10));
-    QTest::mouseRelease(window, Qt::LeftButton, 0, QPoint(50, 10));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(50, 10));
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(50, 10));
 
     QTRY_VERIFY(window->rootObject()->property("clicked").toBool() == true);
-
-    delete window;
 }
 
 void tst_qquickflickable::flickVelocity()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("flickable03.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
     QVERIFY(flickable != 0);
 
     // flick up
-    flick(window, QPoint(20,190), QPoint(20, 50), 200);
+    flick(window.data(), QPoint(20,190), QPoint(20, 50), 200);
     QVERIFY(flickable->verticalVelocity() > 0.0);
     QTRY_VERIFY(flickable->verticalVelocity() == 0.0);
 
     // flick down
-    flick(window, QPoint(20,10), QPoint(20, 140), 200);
-    QVERIFY(flickable->verticalVelocity() < 0.0);
+    flick(window.data(), QPoint(20,10), QPoint(20, 140), 200);
+    QTRY_VERIFY(flickable->verticalVelocity() < 0.0);
     QTRY_VERIFY(flickable->verticalVelocity() == 0.0);
 
 #ifdef Q_OS_MAC
     QSKIP("boost doesn't work on OS X");
-    delete window;
     return;
 #endif
 
@@ -1090,17 +1161,15 @@ void tst_qquickflickable::flickVelocity()
     QQuickFlickablePrivate *fp = QQuickFlickablePrivate::get(flickable);
     bool boosted = false;
     for (int i = 0; i < 6; ++i) {
-        flick(window, QPoint(20,390), QPoint(20, 50), 100);
+        flick(window.data(), QPoint(20,390), QPoint(20, 50), 100);
         boosted |= fp->flickBoost > 1.0;
     }
     QVERIFY(boosted);
 
     // Flick in opposite direction -> boost cancelled.
-    flick(window, QPoint(20,10), QPoint(20, 340), 200);
+    flick(window.data(), QPoint(20,10), QPoint(20, 340), 200);
     QTRY_VERIFY(flickable->verticalVelocity() < 0.0);
     QVERIFY(fp->flickBoost == 1.0);
-
-    delete window;
 }
 
 void tst_qquickflickable::margins()
@@ -1130,8 +1199,7 @@ void tst_qquickflickable::margins()
     // position to the far right, including margin
     obj->setContentX(1600 + 50 - obj->width());
     obj->returnToBounds();
-    QTest::qWait(200);
-    QCOMPARE(obj->contentX(), 1600. + 50. - obj->width());
+    QTRY_COMPARE(obj->contentX(), 1600. + 50. - obj->width());
 
     // position beyond the far right, including margin
     obj->setContentX(1600 + 50 - obj->width() + 1.);
@@ -1146,8 +1214,7 @@ void tst_qquickflickable::margins()
     // position to the far bottom, including margin
     obj->setContentY(600 + 30 - obj->height());
     obj->returnToBounds();
-    QTest::qWait(200);
-    QCOMPARE(obj->contentY(), 600. + 30. - obj->height());
+    QTRY_COMPARE(obj->contentY(), 600. + 30. - obj->height());
 
     // position beyond the far bottom, including margin
     obj->setContentY(600 + 30 - obj->height() + 1.);
@@ -1164,20 +1231,23 @@ void tst_qquickflickable::margins()
 
 void tst_qquickflickable::cancelOnMouseGrab()
 {
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("cancel.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
     QVERIFY(flickable != 0);
 
-    QTest::mousePress(window, Qt::LeftButton, 0, QPoint(10, 10));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(10, 10));
     // drag out of bounds
-    QTest::mouseMove(window, QPoint(50, 50));
-    QTest::mouseMove(window, QPoint(100, 100));
-    QTest::mouseMove(window, QPoint(150, 150));
+    QTest::mouseMove(window.data(), QPoint(50, 50));
+    QTest::mouseMove(window.data(), QPoint(100, 100));
+    QTest::mouseMove(window.data(), QPoint(150, 150));
 
     QVERIFY(flickable->contentX() != 0);
     QVERIFY(flickable->contentY() != 0);
@@ -1193,56 +1263,53 @@ void tst_qquickflickable::cancelOnMouseGrab()
     QTRY_VERIFY(!flickable->isMoving());
     QTRY_VERIFY(!flickable->isDragging());
 
-    QTest::mouseRelease(window, Qt::LeftButton, 0, QPoint(50, 10));
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(50, 10));
 
-    delete window;
 }
 
 void tst_qquickflickable::clickAndDragWhenTransformed()
 {
-    QQuickView *view = new QQuickView;
+    QScopedPointer<QQuickView> view(new QQuickView);
     view->setSource(testFileUrl("transformedFlickable.qml"));
+    QTRY_COMPARE(view->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(view.data());
+    QQuickViewTestUtil::moveMouseAway(view.data());
     view->show();
-    view->requestActivate();
-    QVERIFY(QTest::qWaitForWindowExposed(view));
+    QVERIFY(QTest::qWaitForWindowExposed(view.data()));
     QVERIFY(view->rootObject() != 0);
 
     QQuickFlickable *flickable = view->rootObject()->findChild<QQuickFlickable*>("flickable");
     QVERIFY(flickable != 0);
 
     // click outside child rect
-    QTest::mousePress(view, Qt::LeftButton, 0, QPoint(190, 190));
-    QTest::qWait(10);
-    QCOMPARE(flickable->property("itemPressed").toBool(), false);
-    QTest::mouseRelease(view, Qt::LeftButton, 0, QPoint(190, 190));
+    QTest::mousePress(view.data(), Qt::LeftButton, 0, QPoint(190, 190));
+    QTRY_COMPARE(flickable->property("itemPressed").toBool(), false);
+    QTest::mouseRelease(view.data(), Qt::LeftButton, 0, QPoint(190, 190));
 
     // click inside child rect
-    QTest::mousePress(view, Qt::LeftButton, 0, QPoint(200, 200));
-    QTest::qWait(10);
-    QCOMPARE(flickable->property("itemPressed").toBool(), true);
-    QTest::mouseRelease(view, Qt::LeftButton, 0, QPoint(200, 200));
+    QTest::mousePress(view.data(), Qt::LeftButton, 0, QPoint(200, 200));
+    QTRY_COMPARE(flickable->property("itemPressed").toBool(), true);
+    QTest::mouseRelease(view.data(), Qt::LeftButton, 0, QPoint(200, 200));
 
     const int threshold = qApp->styleHints()->startDragDistance();
 
     // drag outside bounds
-    QTest::mousePress(view, Qt::LeftButton, 0, QPoint(160, 160));
+    QTest::mousePress(view.data(), Qt::LeftButton, 0, QPoint(160, 160));
     QTest::qWait(10);
-    QTest::mouseMove(view, QPoint(160 + threshold * 2, 160));
-    QTest::mouseMove(view, QPoint(160 + threshold * 3, 160));
+    QTest::mouseMove(view.data(), QPoint(160 + threshold * 2, 160));
+    QTest::mouseMove(view.data(), QPoint(160 + threshold * 3, 160));
     QCOMPARE(flickable->isDragging(), false);
     QCOMPARE(flickable->property("itemPressed").toBool(), false);
-    QTest::mouseRelease(view, Qt::LeftButton, 0, QPoint(180, 160));
+    QTest::mouseRelease(view.data(), Qt::LeftButton, 0, QPoint(180, 160));
 
     // drag inside bounds
-    QTest::mousePress(view, Qt::LeftButton, 0, QPoint(200, 140));
+    QTest::mousePress(view.data(), Qt::LeftButton, 0, QPoint(200, 140));
     QTest::qWait(10);
-    QTest::mouseMove(view, QPoint(200 + threshold * 2, 140));
-    QTest::mouseMove(view, QPoint(200 + threshold * 3, 140));
+    QTest::mouseMove(view.data(), QPoint(200 + threshold * 2, 140));
+    QTest::mouseMove(view.data(), QPoint(200 + threshold * 3, 140));
     QCOMPARE(flickable->isDragging(), true);
     QCOMPARE(flickable->property("itemPressed").toBool(), false);
-    QTest::mouseRelease(view, Qt::LeftButton, 0, QPoint(220, 140));
-
-    delete view;
+    QTest::mouseRelease(view.data(), Qt::LeftButton, 0, QPoint(220, 140));
 }
 
 void tst_qquickflickable::flickTwiceUsingTouches()
@@ -1253,64 +1320,236 @@ void tst_qquickflickable::flickTwiceUsingTouches()
     touchDevice->setCapabilities(QTouchDevice::Position);
     QWindowSystemInterface::registerTouchDevice(touchDevice);
 
-    QQuickView *window = new QQuickView;
+    QScopedPointer<QQuickView> window(new QQuickView);
     window->setSource(testFileUrl("longList.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
     QVERIFY(flickable != 0);
 
     QCOMPARE(flickable->contentY(), 0.0f);
-    flickWithTouch(window, touchDevice);
+    flickWithTouch(window.data(), touchDevice, QPoint(100, 400), QPoint(100, 240));
 
     qreal contentYAfterFirstFlick = flickable->contentY();
     qDebug() << "contentYAfterFirstFlick " << contentYAfterFirstFlick;
     QVERIFY(contentYAfterFirstFlick > 50.0f);
+    // Wait until view stops moving
+    QTRY_VERIFY(!flickable->isMoving());
 
-    flickWithTouch(window, touchDevice);
+    flickWithTouch(window.data(), touchDevice, QPoint(100, 400), QPoint(100, 240));
 
     // In the original bug, that second flick would cause Flickable to halt immediately
     qreal contentYAfterSecondFlick = flickable->contentY();
     qDebug() << "contentYAfterSecondFlick " << contentYAfterSecondFlick;
-    QVERIFY(contentYAfterSecondFlick > (contentYAfterFirstFlick + 80.0f));
-
-    delete window;
+    QTRY_VERIFY(contentYAfterSecondFlick > (contentYAfterFirstFlick + 80.0f));
 }
 
-void tst_qquickflickable::flickWithTouch(QWindow *window, QTouchDevice *touchDevice)
+void tst_qquickflickable::flickWithTouch(QWindow *window, QTouchDevice *touchDevice, const QPoint &from, const QPoint &to)
 {
     QTest::touchEvent(window, touchDevice)
-        .press(0, QPoint(100, 400), window);
+        .press(0, from, window);
     QTest::qWait(1);
+    QPoint diff = to - from;
+    for (int i = 1; i <= 8; ++i) {
+        QTest::touchEvent(window, touchDevice)
+            .move(0, from + i*diff/8, window);
+        QTest::qWait(1);
+    }
     QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 380), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 360), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 340), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 320), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 300), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 280), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .move(0, QPoint(100, 260), window);
-    QTest::qWait(1);
-    QTest::touchEvent(window, touchDevice)
-        .release(0, QPoint(100, 240), window);
+        .release(0, to, window);
     QTest::qWait(1);
 }
 
+void tst_qquickflickable::nestedStopAtBounds_data()
+{
+    QTest::addColumn<bool>("transpose");
+    QTest::addColumn<bool>("invert");
+
+    QTest::newRow("left") << false << false;
+    QTest::newRow("right") << false << true;
+    QTest::newRow("top") << true << false;
+    QTest::newRow("bottom") << true << true;
+}
+
+void tst_qquickflickable::nestedStopAtBounds()
+{
+    QFETCH(bool, transpose);
+    QFETCH(bool, invert);
+
+    QQuickView view;
+    view.setSource(testFileUrl("nestedStopAtBounds.qml"));
+    QTRY_COMPARE(view.status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(&view);
+    QQuickViewTestUtil::moveMouseAway(&view);
+    view.show();
+    view.requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+    QVERIFY(view.rootObject());
+
+    QQuickFlickable *outer = qobject_cast<QQuickFlickable*>(view.rootObject());
+    QVERIFY(outer);
+
+    QQuickFlickable *inner = outer->findChild<QQuickFlickable*>("innerFlickable");
+    QVERIFY(inner);
+    inner->setFlickableDirection(transpose ? QQuickFlickable::VerticalFlick : QQuickFlickable::HorizontalFlick);
+    inner->setContentX(invert ? 0 : 100);
+    inner->setContentY(invert ? 0 : 100);
+
+    const int threshold = qApp->styleHints()->startDragDistance();
+
+    QPoint position(200, 200);
+    int &axis = transpose ? position.ry() : position.rx();
+
+    // drag toward the aligned boundary.  Outer flickable dragged.
+    QTest::mousePress(&view, Qt::LeftButton, 0, position);
+    QTest::qWait(10);
+    axis += invert ? threshold * 2 : -threshold * 2;
+    QTest::mouseMove(&view, position);
+    axis += invert ? threshold : -threshold;
+    QTest::mouseMove(&view, position);
+    QCOMPARE(outer->isDragging(), true);
+    QCOMPARE(inner->isDragging(), false);
+    QTest::mouseRelease(&view, Qt::LeftButton, 0, position);
+
+    QVERIFY(!outer->isDragging());
+    QTRY_VERIFY(!outer->isMoving());
+
+    axis = 200;
+    outer->setContentX(50);
+    outer->setContentY(50);
+
+    // drag away from the aligned boundary.  Inner flickable dragged.
+    QTest::mousePress(&view, Qt::LeftButton, 0, position);
+    QTest::qWait(10);
+    axis += invert ? -threshold * 2 : threshold * 2;
+    QTest::mouseMove(&view, position);
+    axis += invert ? -threshold : threshold;
+    QTest::mouseMove(&view, position);
+    QCOMPARE(outer->isDragging(), false);
+    QCOMPARE(inner->isDragging(), true);
+    QTest::mouseRelease(&view, Qt::LeftButton, 0, position);
+
+    QTRY_VERIFY(!outer->isMoving());
+}
+
+void tst_qquickflickable::stopAtBounds_data()
+{
+    QTest::addColumn<bool>("transpose");
+    QTest::addColumn<bool>("invert");
+
+    QTest::newRow("left") << false << false;
+    QTest::newRow("right") << false << true;
+    QTest::newRow("top") << true << false;
+    QTest::newRow("bottom") << true << true;
+}
+
+void tst_qquickflickable::stopAtBounds()
+{
+    QFETCH(bool, transpose);
+    QFETCH(bool, invert);
+
+    QQuickView view;
+    view.setSource(testFileUrl("stopAtBounds.qml"));
+    QTRY_COMPARE(view.status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(&view);
+    QQuickViewTestUtil::moveMouseAway(&view);
+    view.show();
+    view.requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+    QVERIFY(view.rootObject());
+
+    QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(view.rootObject());
+    QVERIFY(flickable);
+
+    if (transpose)
+        flickable->setContentY(invert ? 100 : 0);
+    else
+        flickable->setContentX(invert ? 100 : 0);
+
+    const int threshold = qApp->styleHints()->startDragDistance();
+
+    QPoint position(200, 200);
+    int &axis = transpose ? position.ry() : position.rx();
+
+    // drag away from the aligned boundary. View should not move
+    QTest::mousePress(&view, Qt::LeftButton, 0, position);
+    QTest::qWait(10);
+    for (int i = 0; i < 3; ++i) {
+        axis += invert ? -threshold : threshold;
+        QTest::mouseMove(&view, position);
+    }
+    QCOMPARE(flickable->isDragging(), false);
+    if (invert)
+        QCOMPARE(transpose ? flickable->isAtYEnd() : flickable->isAtXEnd(), true);
+    else
+        QCOMPARE(transpose ? flickable->isAtYBeginning() : flickable->isAtXBeginning(), true);
+
+    // drag back towards boundary
+    for (int i = 0; i < 24; ++i) {
+        axis += invert ? threshold / 3 : -threshold / 3;
+        QTest::mouseMove(&view, position);
+    }
+    QTRY_COMPARE(flickable->isDragging(), true);
+    if (invert)
+        QCOMPARE(transpose ? flickable->isAtYEnd() : flickable->isAtXEnd(), false);
+    else
+        QCOMPARE(transpose ? flickable->isAtYBeginning() : flickable->isAtXBeginning(), false);
+
+    // Drag away from the aligned boundary again.
+    // None of the mouse movements will position the view at the boundary exactly,
+    // but the view should end up aligned on the boundary
+    for (int i = 0; i < 5; ++i) {
+        axis += invert ? -threshold * 2 : threshold * 2;
+        QTest::mouseMove(&view, position);
+    }
+    QCOMPARE(flickable->isDragging(), true);
+
+    // we should have hit the boundary and stopped
+    if (invert) {
+        QCOMPARE(transpose ? flickable->isAtYEnd() : flickable->isAtXEnd(), true);
+        QCOMPARE(transpose ? flickable->contentY() : flickable->contentX(), 100.0);
+    } else {
+        QCOMPARE(transpose ? flickable->isAtYBeginning() : flickable->isAtXBeginning(), true);
+        QCOMPARE(transpose ? flickable->contentY() : flickable->contentX(), 0.0);
+    }
+
+    QTest::mouseRelease(&view, Qt::LeftButton, 0, position);
+}
+
+void tst_qquickflickable::nestedMouseAreaUsingTouch()
+{
+    QTouchDevice *touchDevice = new QTouchDevice;
+    touchDevice->setName("Fake Touchscreen");
+    touchDevice->setType(QTouchDevice::TouchScreen);
+    touchDevice->setCapabilities(QTouchDevice::Position);
+    QWindowSystemInterface::registerTouchDevice(touchDevice);
+
+    QScopedPointer<QQuickView> window(new QQuickView);
+    window->setSource(testFileUrl("nestedmousearea.qml"));
+    QTRY_COMPARE(window->status(), QQuickView::Ready);
+    QQuickViewTestUtil::centerOnScreen(window.data());
+    QQuickViewTestUtil::moveMouseAway(window.data());
+    window->show();
+    QVERIFY(window->rootObject() != 0);
+
+    QQuickFlickable *flickable = qobject_cast<QQuickFlickable*>(window->rootObject());
+    QVERIFY(flickable != 0);
+
+    QCOMPARE(flickable->contentY(), 50.0f);
+    flickWithTouch(window.data(), touchDevice, QPoint(100, 300), QPoint(100, 200));
+
+    // flickable should not have moved
+    QCOMPARE(flickable->contentY(), 50.0);
+
+    // draggable item should have moved up
+    QQuickItem *nested = window->rootObject()->findChild<QQuickItem*>("nested");
+    QVERIFY(nested->y() < 100.0);
+}
+
 QTEST_MAIN(tst_qquickflickable)
 
 #include "tst_qquickflickable.moc"
