@@ -99,6 +99,7 @@ private slots:
     void sourceSize_QTBUG_16389();
     void nullPixmapPaint();
     void imageCrash_QTBUG_22125();
+    void imageCrash_QTBUG_32513();
     void sourceSize_data();
     void sourceSize();
     void progressAndStatusChanges();
@@ -315,8 +316,7 @@ void tst_qquickimage::mirror()
         obj->setFillMode(fillMode);
         obj->setProperty("mirror", true);
         window->show();
-        window->requestActivate();
-        QVERIFY(QTest::qWaitForWindowActive(window));
+        QVERIFY(QTest::qWaitForWindowExposed(window));
 
         QImage screenshot = window->grabWindow();
         screenshots[fillMode] = screenshot;
@@ -500,8 +500,7 @@ void tst_qquickimage::tiling_QTBUG_6716()
 
     QQuickView view(testFileUrl(source));
     view.show();
-    view.requestActivate();
-    QVERIFY(QTest::qWaitForWindowActive(&view));
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
 
     QQuickImage *tiling = findItem<QQuickImage>(view.rootObject(), "tiling");
 
@@ -701,11 +700,20 @@ void tst_qquickimage::imageCrash_QTBUG_22125()
     }
 
     // shouldn't crash when deleting cancelled QQmlPixmapReplys.
-    QTest::qWait(520); // Delay mode delays for 500 ms.
+    server.sendDelayedItem();
     QCoreApplication::sendPostedEvents(0, QEvent::DeferredDelete);
     QCoreApplication::processEvents();
 }
 
+void tst_qquickimage::imageCrash_QTBUG_32513()
+{
+    QQuickView view(testFileUrl("qtbug_32513.qml"));
+    view.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+    QTest::qWait(1000);
+    // shouldn't crash when the image changes sources
+}
+
 void tst_qquickimage::sourceSize_data()
 {
     QTest::addColumn<int>("sourceWidth");
