@@ -1009,9 +1009,8 @@ void tst_qquickstates::anchorRewindBug()
     view->setSource(testFileUrl("anchorRewindBug.qml"));
 
     view->show();
-    view->requestActivate();
 
-    QVERIFY(QTest::qWaitForWindowActive(view));
+    QVERIFY(QTest::qWaitForWindowExposed(view));
 
     QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(view->rootObject());
     QVERIFY(rect != 0);
@@ -1568,7 +1567,7 @@ void tst_qquickstates::QTBUG_14830()
     QVERIFY(rect != 0);
     QQuickItem *item = rect->findChild<QQuickItem*>("area");
 
-    QCOMPARE(item->width(), qreal(171));
+    QCOMPARE(item->width(), qreal(170));
 }
 
 void tst_qquickstates::avoidFastForward()
