@@ -48,6 +48,7 @@
 #include <QtQml/QQmlComponent>
 #include <QtQuick/private/qquickrectangle_p.h>
 #include "../../shared/util.h"
+#include "../shared/visualtestutil.h"
 #include <QSignalSpy>
 #include <qpa/qwindowsysteminterface.h>
 #include <private/qquickwindow_p.h>
@@ -284,8 +285,11 @@ private slots:
 
 
     void constantUpdates();
+    void constantUpdatesOnWindow_data();
+    void constantUpdatesOnWindow();
     void mouseFiltering();
     void headless();
+    void noUpdateWhenNothingChanges();
 
     void touchEvent_basic();
     void touchEvent_propagation();
@@ -300,7 +304,9 @@ private slots:
 
     void qmlCreation();
     void clearColor();
+    void defaultState();
 
+    void grab_data();
     void grab();
     void multipleWindows();
 
@@ -312,6 +318,19 @@ private slots:
 
     void ownershipRootItem();
 
+    void hideThenDelete_data();
+    void hideThenDelete();
+
+    void showHideAnimate();
+
+    void testExpose();
+
+    void requestActivate();
+
+    void blockClosing();
+
+    void crashWhenHoverItemDeleted();
+
 #ifndef QT_NO_CURSOR
     void cursor();
 #endif
@@ -328,7 +347,75 @@ void tst_qquickwindow::constantUpdates()
     window.resize(250, 250);
     ConstantUpdateItem item(window.contentItem());
     window.show();
-    QTRY_VERIFY(item.iterations > 60);
+
+    QSignalSpy spy(&window, SIGNAL(beforeSynchronizing()));
+
+    QTRY_VERIFY(item.iterations > 10);
+    QTRY_VERIFY(spy.count() > 10);
+}
+
+void tst_qquickwindow::constantUpdatesOnWindow_data()
+{
+    QTest::addColumn<bool>("blockedGui");
+    QTest::addColumn<QByteArray>("signal");
+
+    QQuickWindow window;
+    window.setGeometry(100, 100, 300, 200);
+    window.show();
+    QTest::qWaitForWindowExposed(&window);
+    bool threaded = window.openglContext()->thread() != QGuiApplication::instance()->thread();
+
+    if (threaded) {
+        QTest::newRow("blocked, beforeRender") << true << QByteArray(SIGNAL(beforeRendering()));
+        QTest::newRow("blocked, afterRender") << true << QByteArray(SIGNAL(afterRendering()));
+        QTest::newRow("blocked, swapped") << true << QByteArray(SIGNAL(frameSwapped()));
+    }
+    QTest::newRow("unblocked, beforeRender") << false << QByteArray(SIGNAL(beforeRendering()));
+    QTest::newRow("unblocked, afterRender") << false << QByteArray(SIGNAL(afterRendering()));
+    QTest::newRow("unblocked, swapped") << false << QByteArray(SIGNAL(frameSwapped()));
+}
+
+class FrameCounter : public QObject
+{
+    Q_OBJECT
+public slots:
+    void incr() { QMutexLocker locker(&m_mutex); ++m_counter; }
+public:
+    FrameCounter() : m_counter(0) {}
+    int count() { QMutexLocker locker(&m_mutex); int x = m_counter; return x; }
+private:
+    int m_counter;
+    QMutex m_mutex;
+};
+
+void tst_qquickwindow::constantUpdatesOnWindow()
+{
+    QFETCH(bool, blockedGui);
+    QFETCH(QByteArray, signal);
+
+    QQuickWindow window;
+    window.setGeometry(100, 100, 300, 200);
+
+    bool ok = connect(&window, signal.constData(), &window, SLOT(update()), Qt::DirectConnection);
+    Q_ASSERT(ok);
+    window.show();
+    QTest::qWaitForWindowExposed(&window);
+
+    FrameCounter counter;
+    connect(&window, SIGNAL(frameSwapped()), &counter, SLOT(incr()), Qt::DirectConnection);
+
+    int frameCount = 10;
+    QElapsedTimer timer;
+    timer.start();
+    if (blockedGui) {
+        while (counter.count() < frameCount)
+            QTest::qSleep(100);
+        QVERIFY(counter.count() >= frameCount);
+    } else {
+        window.update();
+        QTRY_VERIFY(counter.count() > frameCount);
+    }
+    window.hide();
 }
 
 void tst_qquickwindow::touchEvent_basic()
@@ -843,15 +930,15 @@ void tst_qquickwindow::qmlCreation()
     QQmlEngine engine;
     QQmlComponent component(&engine);
     component.loadUrl(testFileUrl("window.qml"));
-    QObject* created = component.create();
+    QObject *created = component.create();
     QScopedPointer<QObject> cleanup(created);
     QVERIFY(created);
 
-    QQuickWindow* window = qobject_cast<QQuickWindow*>(created);
+    QQuickWindow *window = qobject_cast<QQuickWindow*>(created);
     QVERIFY(window);
     QCOMPARE(window->color(), QColor(Qt::green));
 
-    QQuickItem* item = window->findChild<QQuickItem*>("item");
+    QQuickItem *item = window->findChild<QQuickItem*>("item");
     QVERIFY(item);
     QCOMPARE(item->window(), window);
 }
@@ -869,15 +956,47 @@ void tst_qquickwindow::clearColor()
     QCOMPARE(window->color(), QColor(Qt::blue));
 }
 
+void tst_qquickwindow::defaultState()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQuick 2.0; import QtQuick.Window 2.1; Window { }", QUrl());
+    QObject *created = component.create();
+    QScopedPointer<QObject> cleanup(created);
+    QVERIFY(created);
+
+    QQuickWindow *qmlWindow = qobject_cast<QQuickWindow*>(created);
+    QVERIFY(qmlWindow);
+
+    QQuickWindow cppWindow;
+    cppWindow.show();
+    QTest::qWaitForWindowExposed(&cppWindow);
+
+    QCOMPARE(qmlWindow->windowState(), cppWindow.windowState());
+}
+
+void tst_qquickwindow::grab_data()
+{
+    QTest::addColumn<bool>("visible");
+    QTest::newRow("visible") << true;
+    QTest::newRow("invisible") << false;
+}
+
 void tst_qquickwindow::grab()
 {
+    QFETCH(bool, visible);
+
     QQuickWindow window;
     window.setColor(Qt::red);
 
     window.resize(250, 250);
-    window.show();
 
-    QVERIFY(QTest::qWaitForWindowExposed(&window));
+    if (visible) {
+        window.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&window));
+    } else {
+        window.create();
+    }
 
     QImage content = window.grabWindow();
     QCOMPARE(content.width(), window.width());
@@ -919,10 +1038,10 @@ void tst_qquickwindow::animationsWhileHidden()
     QQmlEngine engine;
     QQmlComponent component(&engine);
     component.loadUrl(testFileUrl("AnimationsWhileHidden.qml"));
-    QObject* created = component.create();
+    QObject *created = component.create();
     QScopedPointer<QObject> cleanup(created);
 
-    QQuickWindow* window = qobject_cast<QQuickWindow*>(created);
+    QQuickWindow *window = qobject_cast<QQuickWindow*>(created);
     QVERIFY(window);
     QVERIFY(window->isVisible());
 
@@ -941,10 +1060,12 @@ void tst_qquickwindow::headless()
     QQmlEngine engine;
     QQmlComponent component(&engine);
     component.loadUrl(testFileUrl("Headless.qml"));
-    QObject* created = component.create();
+    QObject *created = component.create();
     QScopedPointer<QObject> cleanup(created);
 
-    QQuickWindow* window = qobject_cast<QQuickWindow*>(created);
+    QQuickWindow *window = qobject_cast<QQuickWindow*>(created);
+    window->setPersistentOpenGLContext(false);
+    window->setPersistentSceneGraph(false);
     QVERIFY(window);
     window->show();
 
@@ -984,6 +1105,33 @@ void tst_qquickwindow::headless()
     QCOMPARE(originalContent, newContent);
 }
 
+void tst_qquickwindow::noUpdateWhenNothingChanges()
+{
+    QQuickWindow window;
+    window.setGeometry(100, 100, 300, 200);
+
+    QQuickRectangle rect(window.contentItem());
+
+    window.showNormal();
+    QTest::qWaitForWindowExposed(&window);
+    // Many platforms are broken in the sense that that they follow up
+    // the initial expose with a second expose or more. Let these go
+    // through before we let the test continue.
+    QTest::qWait(100);
+
+    if (window.openglContext()->thread() == QGuiApplication::instance()->thread()) {
+        QSKIP("Only threaded renderloop implements this feature");
+        return;
+    }
+
+    QSignalSpy spy(&window, SIGNAL(frameSwapped()));
+    rect.update();
+    // Wait a while and verify that no more frameSwapped come our way.
+    QTest::qWait(100);
+
+    QCOMPARE(spy.size(), 0);
+}
+
 void tst_qquickwindow::focusObject()
 {
     QQmlEngine engine;
@@ -997,26 +1145,46 @@ void tst_qquickwindow::focusObject()
     QQuickWindow *window = qobject_cast<QQuickWindow*>(created);
     QVERIFY(window);
 
+    QSignalSpy focusObjectSpy(window, SIGNAL(focusObjectChanged(QObject*)));
+
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+
+    QCOMPARE(window->contentItem(), window->focusObject());
+    QCOMPARE(focusObjectSpy.count(), 1);
+
     QQuickItem *item1 = window->findChild<QQuickItem*>("item1");
     QVERIFY(item1);
     item1->setFocus(true);
     QCOMPARE(item1, window->focusObject());
+    QCOMPARE(focusObjectSpy.count(), 2);
 
     QQuickItem *item2 = window->findChild<QQuickItem*>("item2");
     QVERIFY(item2);
     item2->setFocus(true);
     QCOMPARE(item2, window->focusObject());
+    QCOMPARE(focusObjectSpy.count(), 3);
+
+    // set focus for item in non-focused focus scope and
+    // ensure focusObject does not change and signal is not emitted
+    QQuickItem *item3 = window->findChild<QQuickItem*>("item3");
+    QVERIFY(item3);
+    item3->setFocus(true);
+    QCOMPARE(item2, window->focusObject());
+    QCOMPARE(focusObjectSpy.count(), 3);
 }
 
 void tst_qquickwindow::ignoreUnhandledMouseEvents()
 {
-    QQuickWindow* window = new QQuickWindow;
+    QQuickWindow *window = new QQuickWindow;
     QScopedPointer<QQuickWindow> cleanup(window);
     window->resize(100, 100);
     window->show();
     QVERIFY(QTest::qWaitForWindowExposed(window));
 
-    QQuickItem* item = new QQuickItem;
+    QQuickItem *item = new QQuickItem;
     item->setSize(QSizeF(100, 100));
     item->setParentItem(window->contentItem());
 
@@ -1053,10 +1221,10 @@ void tst_qquickwindow::ownershipRootItem()
     QQmlEngine engine;
     QQmlComponent component(&engine);
     component.loadUrl(testFileUrl("ownershipRootItem.qml"));
-    QObject* created = component.create();
+    QObject *created = component.create();
     QScopedPointer<QObject> cleanup(created);
 
-    QQuickWindow* window = qobject_cast<QQuickWindow*>(created);
+    QQuickWindow *window = qobject_cast<QQuickWindow*>(created);
     QVERIFY(window);
     window->show();
     QVERIFY(QTest::qWaitForWindowExposed(window));
@@ -1184,29 +1352,214 @@ void tst_qquickwindow::cursor()
     QTest::mouseRelease(&window, Qt::LeftButton, 0, QPoint(100, 100));
 
     // Remove the cursor item from the scene. Theoretically this should make parentItem the
-    // cursorItem, but given the situation will correct itself after the next mouse move it's
-    // probably better left as is to avoid unnecessary work during tear down.
+    // cursorItem, but given the situation will correct itself after the next mouse move it
+    // simply unsets the window cursor for now.
     childItem.setParentItem(0);
-    QCOMPARE(window.cursor().shape(), Qt::WaitCursor);
+    QCOMPARE(window.cursor().shape(), Qt::ArrowCursor);
 
     parentItem.setCursor(Qt::SizeAllCursor);
     QCOMPARE(parentItem.cursor().shape(), Qt::SizeAllCursor);
-    QCOMPARE(window.cursor().shape(), Qt::WaitCursor);
+    QCOMPARE(window.cursor().shape(), Qt::ArrowCursor);
 
     // Changing the cursor of an un-parented item doesn't affect the window's cursor.
     childItem.setCursor(Qt::ClosedHandCursor);
     QCOMPARE(childItem.cursor().shape(), Qt::ClosedHandCursor);
-    QCOMPARE(window.cursor().shape(), Qt::WaitCursor);
+    QCOMPARE(window.cursor().shape(), Qt::ArrowCursor);
 
     childItem.unsetCursor();
     QCOMPARE(childItem.cursor().shape(), Qt::ArrowCursor);
-    QCOMPARE(window.cursor().shape(), Qt::WaitCursor);
+    QCOMPARE(window.cursor().shape(), Qt::ArrowCursor);
 
     QTest::mouseRelease(&window, Qt::LeftButton, 0, QPoint(100, 101));
     QCOMPARE(window.cursor().shape(), Qt::SizeAllCursor);
 }
 #endif
 
+void tst_qquickwindow::hideThenDelete_data()
+{
+    QTest::addColumn<bool>("persistentSG");
+    QTest::addColumn<bool>("persistentGL");
+
+    QTest::newRow("persistent:SG=false,GL=false") << false << false;
+    QTest::newRow("persistent:SG=true,GL=false") << true << false;
+    QTest::newRow("persistent:SG=false,GL=true") << false << true;
+    QTest::newRow("persistent:SG=true,GL=true") << true << true;
+}
+
+void tst_qquickwindow::hideThenDelete()
+{
+    QFETCH(bool, persistentSG);
+    QFETCH(bool, persistentGL);
+
+    QSignalSpy *openglDestroyed = 0;
+    QSignalSpy *sgInvalidated = 0;
+
+    {
+        QQuickWindow window;
+        window.setColor(Qt::red);
+
+        window.setPersistentSceneGraph(persistentSG);
+        window.setPersistentOpenGLContext(persistentGL);
+
+        window.resize(400, 300);
+        window.show();
+
+        QTest::qWaitForWindowExposed(&window);
+
+        openglDestroyed = new QSignalSpy(window.openglContext(), SIGNAL(aboutToBeDestroyed()));
+        sgInvalidated = new QSignalSpy(&window, SIGNAL(sceneGraphInvalidated()));
+
+        window.hide();
+
+        QTRY_VERIFY(!window.isExposed());
+
+        if (!persistentSG) {
+            QVERIFY(sgInvalidated->size() > 0);
+            if (!persistentGL)
+                QVERIFY(openglDestroyed->size() > 0);
+            else
+                QVERIFY(openglDestroyed->size() == 0);
+        } else {
+            QVERIFY(sgInvalidated->size() == 0);
+            QVERIFY(openglDestroyed->size() == 0);
+        }
+    }
+
+    QVERIFY(sgInvalidated->size() > 0);
+    QVERIFY(openglDestroyed->size() > 0);
+}
+
+void tst_qquickwindow::showHideAnimate()
+{
+    // This test tries to mimick a bug triggered in the qquickanimatedimage test
+    // A window is shown, then removed again before it is exposed. This left
+    // traces in the render loop which prevent other animations from running
+    // later on.
+    {
+        QQuickWindow window;
+        window.resize(400, 300);
+        window.show();
+    }
+
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("showHideAnimate.qml"));
+    QQuickItem* created = qobject_cast<QQuickItem *>(component.create());
+
+    QVERIFY(created);
+
+    QTRY_VERIFY(created->opacity() > 0.5);
+    QTRY_VERIFY(created->opacity() < 0.5);
+}
+
+void tst_qquickwindow::testExpose()
+{
+    QQuickWindow window;
+    window.setGeometry(100, 100, 300, 200);
+
+    window.show();
+    QTRY_VERIFY(window.isExposed());
+
+    QSignalSpy swapSpy(&window, SIGNAL(frameSwapped()));
+
+    // exhaust pending exposes, as some platforms send us plenty
+    // while showing the first time
+    QTest::qWait(1000);
+    while (swapSpy.size() != 0) {
+        swapSpy.clear();
+        QTest::qWait(100);
+    }
+
+    QWindowSystemInterface::handleExposeEvent(&window, QRegion(10, 10, 20, 20));
+    QTRY_COMPARE(swapSpy.size(), 1);
+}
+
+void tst_qquickwindow::requestActivate()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("active.qml"));
+    QQuickWindow *window1 = qobject_cast<QQuickWindow *>(component.create());
+    QVERIFY(window1);
+
+    QWindowList windows = QGuiApplication::topLevelWindows();
+    QVERIFY(windows.size() == 2);
+
+    for (int i = 0; i < windows.size(); ++i) {
+        if (windows.at(i)->objectName() == window1->objectName()) {
+            windows.removeAt(i);
+            break;
+        }
+    }
+    QVERIFY(windows.size() == 1);
+    QVERIFY(windows.at(0)->objectName() == "window2");
+
+    window1->show();
+    window1->requestActivate();
+
+    QTRY_VERIFY(QGuiApplication::focusWindow() == window1);
+    QVERIFY(window1->isActive() == true);
+
+    QQuickItem *item = QQuickVisualTestUtil::findItem<QQuickItem>(window1->contentItem(), "item1");
+    QVERIFY(item);
+
+    //copied from src/qmltest/quicktestevent.cpp
+    QPoint pos = item->mapToScene(QPointF(item->width()/2, item->height()/2)).toPoint();
+
+    QMouseEvent me(QEvent::MouseButtonPress, pos, window1->mapToGlobal(pos), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    QSpontaneKeyEvent::setSpontaneous(&me);
+    if (!qApp->notify(window1, &me)) {
+        QString warning = QString::fromLatin1("Mouse event MousePress not accepted by receiving window");
+        QWARN(warning.toLatin1().data());
+    }
+    me = QMouseEvent(QEvent::MouseButtonPress, pos, window1->mapToGlobal(pos), Qt::LeftButton, 0, Qt::NoModifier);
+    QSpontaneKeyEvent::setSpontaneous(&me);
+    if (!qApp->notify(window1, &me)) {
+        QString warning = QString::fromLatin1("Mouse event MouseRelease not accepted by receiving window");
+        QWARN(warning.toLatin1().data());
+    }
+
+    QTRY_VERIFY(QGuiApplication::focusWindow() == windows.at(0));
+    QVERIFY(windows.at(0)->isActive());
+}
+
+void tst_qquickwindow::blockClosing()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("ucantclosethis.qml"));
+    QQuickWindow *window = qobject_cast<QQuickWindow *>(component.create());
+    QVERIFY(window);
+    window->show();
+    QTest::qWaitForWindowExposed(window);
+    QVERIFY(window->isVisible());
+    QWindowSystemInterface::handleCloseEvent(window);
+    QVERIFY(window->isVisible());
+    QWindowSystemInterface::handleCloseEvent(window);
+    QVERIFY(window->isVisible());
+    window->setProperty("canCloseThis", true);
+    QWindowSystemInterface::handleCloseEvent(window);
+    QTRY_VERIFY(!window->isVisible());
+}
+
+void tst_qquickwindow::crashWhenHoverItemDeleted()
+{
+    // QTBUG-32771
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("hoverCrash.qml"));
+    QQuickWindow *window = qobject_cast<QQuickWindow *>(component.create());
+    QVERIFY(window);
+    window->show();
+    QTest::qWaitForWindowExposed(window);
+
+    // Simulate a move from the first rectangle to the second. Crash will happen in here
+    // Moving instantaneously from (0, 99) to (0, 102) does not cause the crash
+    for (int i = 99; i < 102; ++i) {
+        QTest::mouseMove(window, QPoint(0, i));
+    }
+}
+
 QTEST_MAIN(tst_qquickwindow)
 
 #include "tst_qquickwindow.moc"
