@@ -58,6 +58,8 @@
 #include <QtQml/qqmlcomponent.h>
 #include "../../../../src/imports/xmllistmodel/qqmlxmllistmodel_p.h"
 
+#include <algorithm>
+
 typedef QPair<int, int> QQuickXmlListRange;
 typedef QList<QVariantList> QQmlXmlModelData;
 
@@ -101,6 +103,7 @@ private slots:
     void threading();
     void threading_data();
     void propertyChanges();
+    void selectAncestor();
 
     void roleCrash();
 
@@ -607,7 +610,7 @@ void tst_qquickxmllistmodel::useKeys()
     }
 
     QList<int> roles = model->roleNames().keys();
-    qSort(roles);
+    std::sort(roles.begin(), roles.end());
     for (int i=0; i<model->rowCount(); i++) {
         QModelIndex index = model->index(i, 0);
         for (int j=0; j<roles.count(); j++)
@@ -758,7 +761,7 @@ void tst_qquickxmllistmodel::noKeysValueChanges()
     model->setProperty("xml",xml);
 
     QList<int> roles = model->roleNames().keys();
-    qSort(roles);
+    std::sort(roles.begin(), roles.end());
     // wait for the new xml data to be set, and verify no signals were emitted
     QTRY_VERIFY(model->data(model->index(0, 0), roles.at(2)).toString() != QLatin1String("Football"));
     QCOMPARE(model->data(model->index(0, 0), roles.at(2)).toString(), QLatin1String("AussieRules"));
@@ -859,21 +862,21 @@ void tst_qquickxmllistmodel::threading()
         for (int i=0; i<dataCount; i++) {
             QModelIndex index = m1->index(i, 0);
             QList<int> roles = m1->roleNames().keys();
-            qSort(roles);
+            std::sort(roles.begin(), roles.end());
             QCOMPARE(m1->data(index, roles.at(0)).toString(), QString("A" + QString::number(i)));
             QCOMPARE(m1->data(index, roles.at(1)).toString(), QString("1" + QString::number(i)));
             QCOMPARE(m1->data(index, roles.at(2)).toString(), QString("Football"));
 
             index = m2->index(i, 0);
             roles = m2->roleNames().keys();
-            qSort(roles);
+            std::sort(roles.begin(), roles.end());
             QCOMPARE(m2->data(index, roles.at(0)).toString(), QString("B" + QString::number(i)));
             QCOMPARE(m2->data(index, roles.at(1)).toString(), QString("2" + QString::number(i)));
             QCOMPARE(m2->data(index, roles.at(2)).toString(), QString("Athletics"));
 
             index = m3->index(i, 0);
             roles = m3->roleNames().keys();
-            qSort(roles);
+            std::sort(roles.begin(), roles.end());
             QCOMPARE(m3->data(index, roles.at(0)).toString(), QString("C" + QString::number(i)));
             QCOMPARE(m3->data(index, roles.at(1)).toString(), QString("3" + QString::number(i)));
             QCOMPARE(m3->data(index, roles.at(2)).toString(), QString("Curling"));
@@ -964,6 +967,18 @@ void tst_qquickxmllistmodel::propertyChanges()
     delete model;
 }
 
+void tst_qquickxmllistmodel::selectAncestor()
+{
+    QQmlComponent component(&engine, testFileUrl("groups.qml"));
+    QAbstractItemModel *model = qobject_cast<QAbstractItemModel *>(component.create());
+    QVERIFY(model != 0);
+    QTRY_COMPARE(model->rowCount(), 1);
+
+    QModelIndex index = model->index(0, 0);
+    QCOMPARE(model->data(index, Qt::UserRole).toInt(), 12);
+    QCOMPARE(model->data(index, Qt::UserRole+1).toString(), QLatin1String("cats"));
+}
+
 void tst_qquickxmllistmodel::roleCrash()
 {
     // don't crash
