@@ -53,6 +53,9 @@ namespace QQuickViewTestUtil
     QQuickView *createView();
 
     void flick(QQuickView *window, const QPoint &from, const QPoint &to, int duration);
+    void centerOnScreen(QQuickView *window, const QSize &size);
+    void centerOnScreen(QQuickView *window);
+    void moveMouseAway(QQuickView *window);
 
     QList<int> adjustIndexesForAddDisplaced(const QList<int> &indexes, int index, int count);
     QList<int> adjustIndexesForMove(const QList<int> &indexes, int from, int to, int count);
