@@ -148,12 +148,13 @@ private slots:
     void initTestCase();
 
     void simpleTouchEvent();
-    void eventFilter();
+    void testEventFilter();
     void mouse();
     void touchOverMouse();
     void mouseOverTouch();
 
     void buttonOnFlickable();
+    void buttonOnDelayedPressFlickable();
     void buttonOnTouch();
 
     void pinchOnFlickable();
@@ -162,9 +163,22 @@ private slots:
 
     void tapOnDismissiveTopMouseAreaClicksBottomOne();
 
+protected:
+    bool eventFilter(QObject *, QEvent *event)
+    {
+        if (event->type() == QEvent::MouseButtonPress ||
+                event->type() == QEvent::MouseMove ||
+                event->type() == QEvent::MouseButtonRelease) {
+            QMouseEvent *me = static_cast<QMouseEvent*>(event);
+            filteredEventList.append(Event(me->type(), me->pos(), me->globalPos()));
+        }
+        return false;
+    }
+
 private:
     QQuickView *createView();
     QTouchDevice *device;
+    QList<Event> filteredEventList;
 };
 
 QQuickView *tst_TouchMouse::createView()
@@ -195,8 +209,9 @@ void tst_TouchMouse::simpleTouchEvent()
 
     window->setSource(testFileUrl("singleitem.qml"));
     window->show();
-    window->requestActivate();
     QVERIFY(QTest::qWaitForWindowExposed(window));
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     EventItem *eventItem1 = window->rootObject()->findChild<EventItem*>("eventItem1");
@@ -303,7 +318,7 @@ void tst_TouchMouse::simpleTouchEvent()
     delete window;
 }
 
-void tst_TouchMouse::eventFilter()
+void tst_TouchMouse::testEventFilter()
 {
 //    // install event filter on item and see that it can grab events
 //    QQuickView *window = createView();
@@ -344,7 +359,9 @@ void tst_TouchMouse::mouse()
 
     window->setSource(testFileUrl("twoitems.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     EventItem *eventItem1 = window->rootObject()->findChild<EventItem*>("eventItem1");
@@ -376,7 +393,9 @@ void tst_TouchMouse::touchOverMouse()
 
     window->setSource(testFileUrl("twoitems.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     EventItem *eventItem1 = window->rootObject()->findChild<EventItem*>("eventItem1");
@@ -417,7 +436,9 @@ void tst_TouchMouse::mouseOverTouch()
 
     window->setSource(testFileUrl("twoitems.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     EventItem *eventItem1 = window->rootObject()->findChild<EventItem*>("eventItem1");
@@ -461,7 +482,9 @@ void tst_TouchMouse::buttonOnFlickable()
 
     window->setSource(testFileUrl("buttononflickable.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     QQuickFlickable *flickable = window->rootObject()->findChild<QQuickFlickable*>("flickable");
@@ -487,7 +510,7 @@ void tst_TouchMouse::buttonOnFlickable()
     QCOMPARE(eventItem1->eventList.size(), 0);
     QPoint p1 = QPoint(20, 130);
     QTest::touchEvent(window, device).press(0, p1, window);
-    QCOMPARE(eventItem1->eventList.size(), 2);
+    QTRY_COMPARE(eventItem1->eventList.size(), 2);
     QCOMPARE(eventItem1->eventList.at(0).type, QEvent::TouchBegin);
     QCOMPARE(eventItem1->eventList.at(1).type, QEvent::MouseButtonPress);
     QTest::touchEvent(window, device).release(0, p1, window);
@@ -550,12 +573,96 @@ void tst_TouchMouse::buttonOnFlickable()
     QCOMPARE(eventItem1->eventList.at(3).type, QEvent::MouseMove);
 
     QCOMPARE(windowPriv->mouseGrabberItem, flickable);
+    QCOMPARE(windowPriv->touchMouseId, 0);
+    QCOMPARE(windowPriv->itemForTouchPointId[0], flickable);
     QVERIFY(flickable->isMovingVertically());
 
     QTest::touchEvent(window, device).release(0, p3, window);
     delete window;
 }
 
+void tst_TouchMouse::buttonOnDelayedPressFlickable()
+{
+    // flickable - height 500 / 1000
+    //   - eventItem1 y: 100, height 100
+    //   - eventItem2 y: 300, height 100
+
+    qApp->setAttribute(Qt::AA_SynthesizeMouseForUnhandledTouchEvents, true);
+    filteredEventList.clear();
+
+    QQuickView *window = createView();
+
+    window->setSource(testFileUrl("buttononflickable.qml"));
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
+    QVERIFY(window->rootObject() != 0);
+
+    QQuickFlickable *flickable = window->rootObject()->findChild<QQuickFlickable*>("flickable");
+    QVERIFY(flickable);
+
+    window->installEventFilter(this);
+
+    flickable->setPressDelay(60);
+
+    // should a mouse area button be clickable on top of flickable? yes :)
+    EventItem *eventItem1 = window->rootObject()->findChild<EventItem*>("eventItem1");
+    QVERIFY(eventItem1);
+    eventItem1->setAcceptedMouseButtons(Qt::LeftButton);
+    eventItem1->acceptMouse = true;
+
+    // should a touch button be touchable on top of flickable? yes :)
+    EventItem *eventItem2 = window->rootObject()->findChild<EventItem*>("eventItem2");
+    QVERIFY(eventItem2);
+    QCOMPARE(eventItem2->eventList.size(), 0);
+    eventItem2->acceptTouch = true;
+
+    // wait to avoid getting a double click event
+    QTest::qWait(qApp->styleHints()->mouseDoubleClickInterval() + 10);
+
+    // check that flickable moves - mouse button
+    QCOMPARE(eventItem1->eventList.size(), 0);
+    QPoint p1 = QPoint(10, 110);
+    QTest::touchEvent(window, device).press(0, p1, window);
+    // Flickable initially steals events
+    QCOMPARE(eventItem1->eventList.size(), 0);
+    // but we'll get the delayed mouse press after a delay
+    QTRY_COMPARE(eventItem1->eventList.size(), 1);
+    QCOMPARE(eventItem1->eventList.at(0).type, QEvent::MouseButtonPress);
+
+    // eventItem1 should have the mouse grab, and have moved the itemForTouchPointId
+    // for the touchMouseId to the new grabber.
+    QQuickWindowPrivate *windowPriv = QQuickWindowPrivate::get(window);
+    QCOMPARE(windowPriv->touchMouseId, 0);
+    QCOMPARE(windowPriv->itemForTouchPointId[0], eventItem1);
+    QCOMPARE(windowPriv->mouseGrabberItem, eventItem1);
+
+    p1 += QPoint(0, -10);
+    QPoint p2 = p1 + QPoint(0, -10);
+    QPoint p3 = p2 + QPoint(0, -10);
+    QTest::qWait(10);
+    QTest::touchEvent(window, device).move(0, p1, window);
+    QTest::qWait(10);
+    QTest::touchEvent(window, device).move(0, p2, window);
+    QTest::qWait(10);
+    QTest::touchEvent(window, device).move(0, p3, window);
+    QVERIFY(flickable->isMovingVertically());
+
+    // flickable should have the mouse grab, and have moved the itemForTouchPointId
+    // for the touchMouseId to the new grabber.
+    QCOMPARE(windowPriv->mouseGrabberItem, flickable);
+    QCOMPARE(windowPriv->touchMouseId, 0);
+    QCOMPARE(windowPriv->itemForTouchPointId[0], flickable);
+
+    QTest::touchEvent(window, device).release(0, p3, window);
+
+    // We should not have received any synthesised mouse events from Qt gui.
+    QCOMPARE(filteredEventList.count(), 0);
+
+    delete window;
+}
+
 void tst_TouchMouse::buttonOnTouch()
 {
     // 400x800
@@ -569,7 +676,9 @@ void tst_TouchMouse::buttonOnTouch()
     QQuickView *window = createView();
     window->setSource(testFileUrl("buttonontouch.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     QQuickPinchArea *pinchArea = window->rootObject()->findChild<QQuickPinchArea*>("pincharea");
@@ -691,7 +800,9 @@ void tst_TouchMouse::pinchOnFlickable()
     QQuickView *window = createView();
     window->setSource(testFileUrl("pinchonflickable.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     QQuickPinchArea *pinchArea = window->rootObject()->findChild<QQuickPinchArea*>("pincharea");
@@ -758,7 +869,9 @@ void tst_TouchMouse::flickableOnPinch()
     QQuickView *window = createView();
     window->setSource(testFileUrl("flickableonpinch.qml"));
     window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     QQuickPinchArea *pinchArea = window->rootObject()->findChild<QQuickPinchArea*>("pincharea");
@@ -824,8 +937,7 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     QQuickView *window = createView();
     window->setSource(testFileUrl("mouseonflickableonpinch.qml"));
     window->show();
-    window->requestActivate();
-    QVERIFY(QTest::qWaitForWindowActive(window));
+    QVERIFY(QTest::qWaitForWindowExposed(window));
     QVERIFY(window->rootObject() != 0);
     QRect windowRect = QRect(window->position(), window->size());
     QCursor::setPos(windowRect.center());
@@ -853,6 +965,9 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     QGuiApplication::processEvents();
 
     //QVERIFY(flickable->isMovingHorizontally());
+
+    // Wait for flick to end
+    QTRY_VERIFY(!flickable->isMoving());
     qDebug() << "Pos: " << rect->position();
 
     // pinch
@@ -936,8 +1051,9 @@ void tst_TouchMouse::tapOnDismissiveTopMouseAreaClicksBottomOne()
 
     window->setSource(testFileUrl("twoMouseAreas.qml"));
     window->show();
-    window->requestActivate();
     QVERIFY(QTest::qWaitForWindowExposed(window));
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(window));
     QVERIFY(window->rootObject() != 0);
 
     QQuickMouseArea *bottomMouseArea =
