@@ -44,6 +44,8 @@
 #include <QtCore/qdir.h>
 #include <QtCore/qmath.h>
 #include <QtCore/qdatetime.h>
+#include <QtCore/qpointer.h>
+#include <QtCore/qscopedpointer.h>
 
 #include <QtGui/QGuiApplication>
 
@@ -152,6 +154,7 @@ struct Options
         , slowAnimations(false)
         , quitImmediately(false)
         , resizeViewToRootItem(false)
+        , multisample(false)
     {
     }
 
@@ -167,6 +170,7 @@ struct Options
     bool slowAnimations;
     bool quitImmediately;
     bool resizeViewToRootItem;
+    bool multisample;
     QString translationFile;
 };
 
@@ -349,7 +353,7 @@ static void usage()
     qWarning("  --maximized ............................... Run maximized");
     qWarning("  --fullscreen .............................. Run fullscreen");
     qWarning("  --transparent ............................. Make the window transparent");
-    qWarning("  --no-multisample .......................... Disable multisampling (anti-aliasing)");
+    qWarning("  --multisample ............................. Enable multisampling (OpenGL anti-aliasing)");
     qWarning("  --no-version-detection .................... Do not try to detect the version of the .qml file");
     qWarning("  --slow-animations ......................... Run all animations in slow motion");
     qWarning("  --resize-to-root .......................... Resize the window to the size of the root item");
@@ -391,6 +395,8 @@ int main(int argc, char ** argv)
                 options.translationFile = QLatin1String(argv[++i]);
             else if (lowerArgument == QLatin1String("--resize-to-root"))
                 options.resizeViewToRootItem = true;
+            else if (lowerArgument == QLatin1String("--multisample"))
+                options.multisample = true;
             else if (lowerArgument == QLatin1String("-i") && i + 1 < argc)
                 imports.append(QString::fromLatin1(argv[++i]));
             else if (lowerArgument == QLatin1String("-b") && i + 2 < argc) {
@@ -457,7 +463,7 @@ int main(int argc, char ** argv)
             // TODO: as soon as the engine construction completes, the debug service is
             // listening for connections.  But actually we aren't ready to debug anything.
             QQmlEngine engine;
-            QQmlComponent *component = new QQmlComponent(&engine);
+            QPointer<QQmlComponent> component = new QQmlComponent(&engine);
             for (int i = 0; i < imports.size(); ++i)
                 engine.addImportPath(imports.at(i));
             for (int i = 0; i < bundles.size(); ++i)
@@ -477,17 +483,21 @@ int main(int argc, char ** argv)
             }
 
             QObject *topLevel = component->create();
-            QQuickWindow *window = qobject_cast<QQuickWindow *>(topLevel);
-            QQuickView* qxView = 0;
-            if (!window) {
+            if (!topLevel && component->isError()) {
+                qWarning("%s", qPrintable(component->errorString()));
+                return -1;
+            }
+            QScopedPointer<QQuickWindow> window(qobject_cast<QQuickWindow *>(topLevel));
+            if (window) {
+                engine.setIncubationController(window->incubationController());
+            } else {
                 QQuickItem *contentItem = qobject_cast<QQuickItem *>(topLevel);
                 if (contentItem) {
-                    qxView = new QQuickView(&engine, NULL);
-                    window = qxView;
+                    QQuickView* qxView = new QQuickView(&engine, NULL);
+                    window.reset(qxView);
                     // Set window default properties; the qml can still override them
                     QString oname = contentItem->objectName();
                     window->setTitle(oname.isEmpty() ? QString::fromLatin1("qmlscene") : QString::fromLatin1("qmlscene: ") + oname);
-                    window->setFlags(Qt::Window | Qt::WindowSystemMenuHint | Qt::WindowTitleHint | Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint);
                     if (options.resizeViewToRootItem)
                         qxView->setResizeMode(QQuickView::SizeViewToRootObject);
                     else
@@ -497,20 +507,25 @@ int main(int argc, char ** argv)
             }
 
             if (window) {
+                QSurfaceFormat surfaceFormat = window->requestedFormat();
+                if (options.multisample)
+                    surfaceFormat.setSamples(16);
                 if (options.transparent) {
-                    QSurfaceFormat surfaceFormat;
                     surfaceFormat.setAlphaBufferSize(8);
-                    window->setFormat(surfaceFormat);
                     window->setClearBeforeRendering(true);
                     window->setColor(QColor(Qt::transparent));
                     window->setFlags(Qt::FramelessWindowHint);
                 }
+                window->setFormat(surfaceFormat);
+
+                if (window->flags() == Qt::Window) // Fix window flags unless set by QML.
+                    window->setFlags(Qt::Window | Qt::WindowSystemMenuHint | Qt::WindowTitleHint | Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint | Qt::WindowFullscreenButtonHint);
 
                 if (options.fullscreen)
                     window->showFullScreen();
                 else if (options.maximized)
                     window->showMaximized();
-                else
+                else if (!window->isVisible())
                     window->show();
             }
 
@@ -524,14 +539,10 @@ int main(int argc, char ** argv)
 #ifdef QML_RUNTIME_TESTING
             RenderStatistics::printTotalStats();
 #endif
-            // Ready to exit.  If we created qxView, it owns the component;
-            // otherwise, the ownership is still right here.  Nobody deletes the engine
-            // (which is odd since the container constructor takes the engine pointer),
-            // but it's stack-allocated anyway.
-            if (qxView)
-                delete qxView;
-            else
-                delete component;
+            // Ready to exit. Notice that the component might be owned by
+            // QQuickView if one was created. That case is tracked by
+            // QPointer, so it is safe to delete the component here.
+            delete component;
         }
     }
 
